@@ -37,7 +37,7 @@ BELLE_SIP_BEGIN_DECLS
 
 typedef struct _belle_sip_header_address belle_sip_header_address_t;
 
-BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_new();
+BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_new(void);
 /*
  * creates an address from a display name and an uri
  * Note the uri not copied but only its ref count is incremented
@@ -59,6 +59,12 @@ BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_create2(con
 
 BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_parse (const char* address) ;
 
+/*
+ same as belle_sip_header_address_parse but with less syntax checking
+ */
+BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_fast_parse (const char* address) ;
+
+
 /**
  * returns a sip uri. A header address cannot have both a sip uri and an absolute uri.
  */
@@ -112,7 +118,7 @@ BELLESIP_EXPORT const char *belle_sip_header_get_unparsed_value(belle_sip_header
  ******************************/
 typedef struct _belle_sip_header_allow belle_sip_header_allow_t;
 
-belle_sip_header_allow_t* belle_sip_header_allow_new();
+belle_sip_header_allow_t* belle_sip_header_allow_new(void);
 
 BELLESIP_EXPORT belle_sip_header_allow_t* belle_sip_header_allow_parse (const char* allow) ;
 BELLESIP_EXPORT belle_sip_header_allow_t* belle_sip_header_allow_create (const char* methods) ;
@@ -127,7 +133,7 @@ BELLESIP_EXPORT void belle_sip_header_allow_set_method(belle_sip_header_allow_t*
  ************************/
 typedef struct _belle_sip_header_contact belle_sip_header_contact_t;
 
-BELLESIP_EXPORT belle_sip_header_contact_t* belle_sip_header_contact_new();
+BELLESIP_EXPORT belle_sip_header_contact_t* belle_sip_header_contact_new(void);
 
 
 BELLESIP_EXPORT belle_sip_header_contact_t* belle_sip_header_contact_parse (const char* contact) ;
@@ -199,7 +205,7 @@ BELLESIP_EXPORT belle_sip_header_contact_t* belle_sip_header_contact_create (con
  ******************************/
  typedef struct _belle_sip_header_from belle_sip_header_from_t;
 
- BELLESIP_EXPORT belle_sip_header_from_t* belle_sip_header_from_new();
+ BELLESIP_EXPORT belle_sip_header_from_t* belle_sip_header_from_new(void);
 
  BELLESIP_EXPORT belle_sip_header_from_t* belle_sip_header_from_create(const belle_sip_header_address_t* address, const char *tag);
 
@@ -221,7 +227,7 @@ BELLESIP_EXPORT belle_sip_header_contact_t* belle_sip_header_contact_create (con
  ******************************/
  typedef struct _belle_sip_header_to belle_sip_header_to_t;
 
- BELLESIP_EXPORT belle_sip_header_to_t* belle_sip_header_to_new();
+ BELLESIP_EXPORT belle_sip_header_to_t* belle_sip_header_to_new(void);
 
  BELLESIP_EXPORT belle_sip_header_to_t* belle_sip_header_to_parse(const char* to) ;
 
@@ -238,13 +244,36 @@ BELLESIP_EXPORT belle_sip_header_contact_t* belle_sip_header_contact_create (con
 #define BELLE_SIP_HEADER_TO(t) BELLE_SIP_CAST(t,belle_sip_header_to_t)
 #define BELLE_SIP_TO "To"
 
+/******************************
+ * Diversion header object inherent from header_address
+ *
+ ******************************/
+ typedef struct _belle_sip_header_diversion belle_sip_header_diversion_t;
+
+ BELLESIP_EXPORT belle_sip_header_diversion_t* belle_sip_header_diversion_new(void);
+
+ BELLESIP_EXPORT belle_sip_header_diversion_t* belle_sip_header_diversion_parse(const char* diversion) ;
+
+ BELLESIP_EXPORT belle_sip_header_diversion_t* belle_sip_header_diversion_create(const belle_sip_header_address_t *address, const char *tag);
+
+ BELLESIP_EXPORT belle_sip_header_diversion_t* belle_sip_header_diversion_create2(const char *address, const char *tag);
+
+ BELLESIP_EXPORT void belle_sip_header_diversion_set_tag(belle_sip_header_diversion_t* diversion, const char* tag);
+
+ BELLESIP_EXPORT const char* belle_sip_header_diversion_get_tag(const belle_sip_header_diversion_t* from);
+
+ BELLESIP_EXPORT void belle_sip_header_diversion_set_random_tag(belle_sip_header_diversion_t *obj);
+
+#define BELLE_SIP_HEADER_DIVERSION(t) BELLE_SIP_CAST(t,belle_sip_header_diversion_t)
+#define BELLE_SIP_DIVERSION "Diversion"
+
 /******************************
  * Via header object inherent from header_address
  *
  ******************************/
 typedef struct _belle_sip_header_via belle_sip_header_via_t;
 
-BELLESIP_EXPORT belle_sip_header_via_t* belle_sip_header_via_new();
+BELLESIP_EXPORT belle_sip_header_via_t* belle_sip_header_via_new(void);
 BELLESIP_EXPORT belle_sip_header_via_t* belle_sip_header_via_create(const char *host, int port, const char *transport, const char *branch);
 BELLESIP_EXPORT belle_sip_header_via_t* belle_sip_header_via_parse (const char* via) ;
 BELLESIP_EXPORT const char*	belle_sip_header_via_get_branch(const belle_sip_header_via_t* via);
@@ -282,7 +311,7 @@ BELLESIP_EXPORT int belle_sip_header_via_set_ttl(belle_sip_header_via_t* via, in
  ******************************/
 typedef struct _belle_sip_header_call_id belle_sip_header_call_id_t;
 
-BELLESIP_EXPORT belle_sip_header_call_id_t* belle_sip_header_call_id_new();
+BELLESIP_EXPORT belle_sip_header_call_id_t* belle_sip_header_call_id_new(void);
 
 BELLESIP_EXPORT belle_sip_header_call_id_t* belle_sip_header_call_id_parse (const char* call_id) ;
 BELLESIP_EXPORT const char*	belle_sip_header_call_id_get_call_id(const belle_sip_header_call_id_t* call_id);
@@ -296,7 +325,7 @@ unsigned int belle_sip_header_call_id_equals(const belle_sip_header_call_id_t* a
  ******************************/
 typedef struct _belle_sip_header_cseq belle_sip_header_cseq_t;
 
-BELLESIP_EXPORT belle_sip_header_cseq_t* belle_sip_header_cseq_new();
+BELLESIP_EXPORT belle_sip_header_cseq_t* belle_sip_header_cseq_new(void);
 BELLESIP_EXPORT belle_sip_header_cseq_t* belle_sip_header_cseq_create(unsigned int number, const char *method);
 BELLESIP_EXPORT belle_sip_header_cseq_t* belle_sip_header_cseq_parse (const char* cseq) ;
 BELLESIP_EXPORT const char*	belle_sip_header_cseq_get_method(const belle_sip_header_cseq_t* cseq);
@@ -311,7 +340,7 @@ BELLESIP_EXPORT void belle_sip_header_cseq_set_seq_number(belle_sip_header_cseq_
  ******************************/
 typedef struct _belle_sip_header_content_type belle_sip_header_content_type_t;
 
-BELLESIP_EXPORT belle_sip_header_content_type_t* belle_sip_header_content_type_new();
+BELLESIP_EXPORT belle_sip_header_content_type_t* belle_sip_header_content_type_new(void);
 BELLESIP_EXPORT belle_sip_header_content_type_t* belle_sip_header_content_type_parse (const char* content_type) ;
 BELLESIP_EXPORT belle_sip_header_content_type_t* belle_sip_header_content_type_create (const char* type,const char* sub_type) ;
 
@@ -329,7 +358,7 @@ BELLESIP_EXPORT void belle_sip_header_content_type_set_subtype(belle_sip_header_
  ******************************/
 typedef struct _belle_sip_header_expires belle_sip_header_expires_t;
 
-BELLESIP_EXPORT belle_sip_header_expires_t* belle_sip_header_expires_new();
+BELLESIP_EXPORT belle_sip_header_expires_t* belle_sip_header_expires_new(void);
 
 BELLESIP_EXPORT belle_sip_header_expires_t* belle_sip_header_expires_parse (const char* expires) ;
 BELLESIP_EXPORT int belle_sip_header_expires_get_expires(const belle_sip_header_expires_t* expires);
@@ -344,7 +373,7 @@ BELLESIP_EXPORT belle_sip_header_expires_t* belle_sip_header_expires_create(int
  ******************************/
 typedef struct _belle_sip_header_route belle_sip_header_route_t;
 
-BELLESIP_EXPORT belle_sip_header_route_t* belle_sip_header_route_new();
+BELLESIP_EXPORT belle_sip_header_route_t* belle_sip_header_route_new(void);
 BELLESIP_EXPORT belle_sip_header_route_t* belle_sip_header_route_parse (const char* route) ;
 BELLESIP_EXPORT belle_sip_header_route_t* belle_sip_header_route_create(const belle_sip_header_address_t* route);
 
@@ -356,9 +385,9 @@ BELLESIP_EXPORT belle_sip_header_route_t* belle_sip_header_route_create(const be
  ******************************/
 typedef struct _belle_sip_header_record_route belle_sip_header_record_route_t;
 
-BELLESIP_EXPORT belle_sip_header_record_route_t* belle_sip_header_record_route_new();
+BELLESIP_EXPORT belle_sip_header_record_route_t* belle_sip_header_record_route_new(void);
 BELLESIP_EXPORT belle_sip_header_record_route_t* belle_sip_header_record_route_parse (const char* route);
-BELLESIP_EXPORT belle_sip_header_record_route_t* belle_sip_header_record_route_new_auto_outgoing();
+BELLESIP_EXPORT belle_sip_header_record_route_t* belle_sip_header_record_route_new_auto_outgoing(void);
 
 BELLESIP_EXPORT unsigned char belle_sip_header_record_route_get_auto_outgoing(const belle_sip_header_record_route_t *a);
  
@@ -372,7 +401,7 @@ BELLESIP_EXPORT unsigned char belle_sip_header_record_route_get_auto_outgoing(co
   ******************************/
   typedef struct _belle_sip_header_service_route belle_sip_header_service_route_t;
 
-  BELLESIP_EXPORT belle_sip_header_service_route_t* belle_sip_header_service_route_new();
+  BELLESIP_EXPORT belle_sip_header_service_route_t* belle_sip_header_service_route_new(void);
   BELLESIP_EXPORT belle_sip_header_service_route_t* belle_sip_header_service_route_parse (const char* route) ;
 
  #define BELLE_SIP_HEADER_SERVICE_ROUTE(t) BELLE_SIP_CAST(t,belle_sip_header_service_route_t)
@@ -384,7 +413,7 @@ BELLESIP_EXPORT unsigned char belle_sip_header_record_route_get_auto_outgoing(co
   ******************************/
  typedef struct _belle_sip_header_user_agent belle_sip_header_user_agent_t;
 
- BELLESIP_EXPORT belle_sip_header_user_agent_t* belle_sip_header_user_agent_new();
+ BELLESIP_EXPORT belle_sip_header_user_agent_t* belle_sip_header_user_agent_new(void);
 
  BELLESIP_EXPORT belle_sip_header_user_agent_t* belle_sip_header_user_agent_parse (const char* user_agent) ;
  BELLESIP_EXPORT belle_sip_list_t* belle_sip_header_user_agent_get_products(const belle_sip_header_user_agent_t* user_agent);
@@ -407,12 +436,12 @@ BELLESIP_EXPORT unsigned char belle_sip_header_record_route_get_auto_outgoing(co
  ******************************/
 typedef struct _belle_sip_header_content_length belle_sip_header_content_length_t;
 
-BELLESIP_EXPORT belle_sip_header_content_length_t* belle_sip_header_content_length_new();
+BELLESIP_EXPORT belle_sip_header_content_length_t* belle_sip_header_content_length_new(void);
 
 BELLESIP_EXPORT belle_sip_header_content_length_t* belle_sip_header_content_length_parse (const char* content_length) ;
-BELLESIP_EXPORT belle_sip_header_content_length_t* belle_sip_header_content_length_create (int content_length) ;
-BELLESIP_EXPORT unsigned int belle_sip_header_content_length_get_content_length(const belle_sip_header_content_length_t* content_length);
-BELLESIP_EXPORT void belle_sip_header_content_length_set_content_length(belle_sip_header_content_length_t* content_length,unsigned int length);
+BELLESIP_EXPORT belle_sip_header_content_length_t* belle_sip_header_content_length_create (size_t content_length) ;
+BELLESIP_EXPORT size_t belle_sip_header_content_length_get_content_length(const belle_sip_header_content_length_t* content_length);
+BELLESIP_EXPORT void belle_sip_header_content_length_set_content_length(belle_sip_header_content_length_t* content_length,size_t length);
 #define BELLE_SIP_HEADER_CONTENT_LENGTH(t) BELLE_SIP_CAST(t,belle_sip_header_content_length_t)
 #define BELLE_SIP_CONTENT_LENGTH "Content-Length"
 
@@ -422,7 +451,7 @@ BELLESIP_EXPORT void belle_sip_header_content_length_set_content_length(belle_si
  ******************************/
 typedef struct _belle_sip_header_authorization belle_sip_header_authorization_t;
 
-BELLESIP_EXPORT belle_sip_header_authorization_t* belle_sip_header_authorization_new();
+BELLESIP_EXPORT belle_sip_header_authorization_t* belle_sip_header_authorization_new(void);
 BELLESIP_EXPORT belle_sip_header_authorization_t* belle_sip_header_authorization_parse(const char* authorization);
 BELLESIP_EXPORT const char*	belle_sip_header_authorization_get_algorithm(const belle_sip_header_authorization_t* authorization );
 BELLESIP_EXPORT const char*	belle_sip_header_authorization_get_cnonce(const belle_sip_header_authorization_t* authorization );
@@ -459,7 +488,7 @@ BELLESIP_EXPORT void belle_sip_header_authorization_set_username(belle_sip_heade
  * proxy_authorization inherit from Authorization
  */
 typedef struct _belle_sip_header_proxy_authorization belle_sip_header_proxy_authorization_t;
-BELLESIP_EXPORT belle_sip_header_proxy_authorization_t* belle_sip_header_proxy_authorization_new();
+BELLESIP_EXPORT belle_sip_header_proxy_authorization_t* belle_sip_header_proxy_authorization_new(void);
 BELLESIP_EXPORT belle_sip_header_proxy_authorization_t* belle_sip_header_proxy_authorization_parse(const char* proxy_authorization);
 #define BELLE_SIP_HEADER_PROXY_AUTHORIZATION(t) BELLE_SIP_CAST(t,belle_sip_header_proxy_authorization_t)
 #define BELLE_SIP_PROXY_AUTHORIZATION "Proxy-Authorization"
@@ -468,7 +497,7 @@ BELLESIP_EXPORT belle_sip_header_proxy_authorization_t* belle_sip_header_proxy_a
  * http_authorization inherit from Authorization
  */
 typedef struct _belle_http_header_authorization belle_http_header_authorization_t;
-BELLESIP_EXPORT belle_http_header_authorization_t* belle_http_header_authorization_new();
+BELLESIP_EXPORT belle_http_header_authorization_t* belle_http_header_authorization_new(void);
 /*cannot be parsed for now
 BELLESIP_EXPORT belle_http_header_authorization_t* belle_http_header_authorization_parse(const char* proxy_authorization);
 */
@@ -482,7 +511,7 @@ BELLESIP_EXPORT belle_generic_uri_t* belle_http_header_authorization_get_uri(con
  * www_authenticate inherit from parameters
  */
 typedef struct _belle_sip_header_www_authenticate belle_sip_header_www_authenticate_t;
-BELLESIP_EXPORT belle_sip_header_www_authenticate_t* belle_sip_header_www_authenticate_new();
+BELLESIP_EXPORT belle_sip_header_www_authenticate_t* belle_sip_header_www_authenticate_new(void);
 BELLESIP_EXPORT belle_sip_header_www_authenticate_t* belle_sip_header_www_authenticate_parse(const char* www_authenticate);
 BELLESIP_EXPORT const char*	belle_sip_header_www_authenticate_get_algorithm(const belle_sip_header_www_authenticate_t* www_authenticate );
 BELLESIP_EXPORT const char* belle_sip_header_www_authenticate_get_nonce(const belle_sip_header_www_authenticate_t* www_authenticate);
@@ -509,7 +538,7 @@ BELLESIP_EXPORT void belle_sip_header_www_authenticate_set_stale(belle_sip_heade
  * proxy_authenticate inherit from www_authenticate
  */
 typedef struct _belle_sip_header_proxy_authenticate belle_sip_header_proxy_authenticate_t;
-BELLESIP_EXPORT belle_sip_header_proxy_authenticate_t* belle_sip_header_proxy_authenticate_new();
+BELLESIP_EXPORT belle_sip_header_proxy_authenticate_t* belle_sip_header_proxy_authenticate_new(void);
 BELLESIP_EXPORT belle_sip_header_proxy_authenticate_t* belle_sip_header_proxy_authenticate_parse(const char* proxy_authenticate);
 #define BELLE_SIP_HEADER_PROXY_AUTHENTICATE(t) BELLE_SIP_CAST(t,belle_sip_header_proxy_authenticate_t)
 #define BELLE_SIP_PROXY_AUTHENTICATE "Proxy-Authenticate"
@@ -521,7 +550,7 @@ BELLESIP_EXPORT belle_sip_header_proxy_authenticate_t* belle_sip_header_proxy_au
  ******************************/
 typedef struct _belle_sip_header_max_forwards belle_sip_header_max_forwards_t;
 
-BELLESIP_EXPORT belle_sip_header_max_forwards_t* belle_sip_header_max_forwards_new();
+BELLESIP_EXPORT belle_sip_header_max_forwards_t* belle_sip_header_max_forwards_new(void);
 BELLESIP_EXPORT belle_sip_header_max_forwards_t* belle_sip_header_max_forwards_create(int value);
 
 BELLESIP_EXPORT belle_sip_header_max_forwards_t* belle_sip_header_max_forwards_parse (const char* max_forwards) ;
@@ -538,7 +567,7 @@ BELLESIP_EXPORT int belle_sip_header_max_forwards_decrement_max_forwards(belle_s
  ******************************/
 typedef struct _belle_sip_header_subscription_state belle_sip_header_subscription_state_t;
 
-BELLESIP_EXPORT belle_sip_header_subscription_state_t* belle_sip_header_subscription_state_new();
+BELLESIP_EXPORT belle_sip_header_subscription_state_t* belle_sip_header_subscription_state_new(void);
 
 BELLESIP_EXPORT belle_sip_header_subscription_state_t* belle_sip_header_subscription_state_parse (const char* subscription_state) ;
 BELLESIP_EXPORT belle_sip_header_subscription_state_t* belle_sip_header_subscription_state_create (const char* subscription_state,int expires);
@@ -565,7 +594,7 @@ BELLESIP_EXPORT void belle_sip_header_subscription_state_set_retry_after(belle_s
  *
  ******************************/
  typedef struct _belle_sip_header_refer_to belle_sip_header_refer_to_t;
- BELLESIP_EXPORT belle_sip_header_refer_to_t* belle_sip_header_refer_to_new();
+ BELLESIP_EXPORT belle_sip_header_refer_to_t* belle_sip_header_refer_to_new(void);
  BELLESIP_EXPORT belle_sip_header_refer_to_t* belle_sip_header_refer_to_parse(const char* refer_to) ;
  BELLESIP_EXPORT belle_sip_header_refer_to_t* belle_sip_header_refer_to_create(const belle_sip_header_address_t *address);
 #define BELLE_SIP_HEADER_REFER_TO(t) BELLE_SIP_CAST(t,belle_sip_header_refer_to_t)
@@ -576,7 +605,7 @@ BELLESIP_EXPORT void belle_sip_header_subscription_state_set_retry_after(belle_s
   *
   ******************************/
   typedef struct _belle_sip_header_referred_by belle_sip_header_referred_by_t;
-  BELLESIP_EXPORT belle_sip_header_referred_by_t* belle_sip_header_referred_by_new();
+  BELLESIP_EXPORT belle_sip_header_referred_by_t* belle_sip_header_referred_by_new(void);
   BELLESIP_EXPORT belle_sip_header_referred_by_t* belle_sip_header_referred_by_parse(const char* referred_by) ;
   BELLESIP_EXPORT belle_sip_header_referred_by_t* belle_sip_header_referred_by_create(const belle_sip_header_address_t *address);
  #define BELLE_SIP_HEADER_REFERRED_BY(t) BELLE_SIP_CAST(t,belle_sip_header_referred_by_t)
@@ -587,7 +616,7 @@ BELLESIP_EXPORT void belle_sip_header_subscription_state_set_retry_after(belle_s
    *
    ******************************/
 typedef struct _belle_sip_header_replaces belle_sip_header_replaces_t;
-BELLESIP_EXPORT belle_sip_header_replaces_t* belle_sip_header_replaces_new();
+BELLESIP_EXPORT belle_sip_header_replaces_t* belle_sip_header_replaces_new(void);
 BELLESIP_EXPORT belle_sip_header_replaces_t* belle_sip_header_replaces_parse(const char* replaces) ;
 
 BELLESIP_EXPORT belle_sip_header_replaces_t* belle_sip_header_replaces_create(const char* call_id,const char* from_tag,const char* to_tag);
@@ -617,7 +646,7 @@ BELLESIP_EXPORT char* belle_sip_header_replaces_value_to_escaped_string(const be
 
 typedef struct belle_sip_header_date belle_sip_header_date_t;
 
-BELLESIP_EXPORT belle_sip_header_date_t* belle_sip_header_date_new();
+BELLESIP_EXPORT belle_sip_header_date_t* belle_sip_header_date_new(void);
 BELLESIP_EXPORT belle_sip_header_date_t* belle_sip_header_date_parse(const char* date) ;
 
 BELLESIP_EXPORT belle_sip_header_date_t* belle_sip_header_date_create_from_time(const time_t *utc_time);
@@ -639,7 +668,7 @@ BELLESIP_EXPORT void belle_sip_header_date_set_date(belle_sip_header_date_t *obj
 ******************************/
 typedef struct _belle_sip_header_p_preferred_identity belle_sip_header_p_preferred_identity_t;
 
-BELLESIP_EXPORT belle_sip_header_p_preferred_identity_t* belle_sip_header_p_preferred_identity_new();
+BELLESIP_EXPORT belle_sip_header_p_preferred_identity_t* belle_sip_header_p_preferred_identity_new(void);
 
 BELLESIP_EXPORT belle_sip_header_p_preferred_identity_t* belle_sip_header_p_preferred_identity_parse(const char* p_preferred_identity) ;
 
@@ -654,7 +683,7 @@ BELLESIP_EXPORT belle_sip_header_p_preferred_identity_t* belle_sip_header_p_pref
 ******************************/
 typedef struct _belle_sip_header_privacy belle_sip_header_privacy_t;
 
-BELLESIP_EXPORT belle_sip_header_privacy_t* belle_sip_header_privacy_new();
+BELLESIP_EXPORT belle_sip_header_privacy_t* belle_sip_header_privacy_new(void);
 
 BELLESIP_EXPORT belle_sip_header_privacy_t* belle_sip_header_privacy_parse(const char* privacy) ;
 
@@ -676,7 +705,7 @@ BELLESIP_EXPORT belle_sip_list_t* belle_sip_header_privacy_get_privacy(const bel
 *
 ******************************/
 typedef struct _belle_sip_header_event belle_sip_header_event_t;
-BELLESIP_EXPORT belle_sip_header_event_t* belle_sip_header_event_new();
+BELLESIP_EXPORT belle_sip_header_event_t* belle_sip_header_event_new(void);
 BELLESIP_EXPORT belle_sip_header_event_t* belle_sip_header_event_parse(const char* event) ;
 BELLESIP_EXPORT belle_sip_header_event_t* belle_sip_header_event_create(const char* event);
 BELLESIP_EXPORT const char* belle_sip_header_event_get_package_name(const belle_sip_header_event_t* event);
@@ -686,6 +715,96 @@ BELLESIP_EXPORT void belle_sip_header_event_set_id(belle_sip_header_event_t* eve
 #define BELLE_SIP_HEADER_EVENT(t) BELLE_SIP_CAST(t,belle_sip_header_event_t)
 #define BELLE_SIP_EVENT "Event"
 
+
+/******************************
+ * Supported header object inherent from header
+ *
+ ******************************/
+typedef struct _belle_sip_header_supported belle_sip_header_supported_t;
+BELLESIP_EXPORT belle_sip_header_supported_t* belle_sip_header_supported_new(void);
+BELLESIP_EXPORT belle_sip_header_supported_t* belle_sip_header_supported_parse(const char* supported) ;
+BELLESIP_EXPORT belle_sip_header_supported_t* belle_sip_header_supported_create(const char* supported);
+BELLESIP_EXPORT void belle_sip_header_supported_add_supported(belle_sip_header_supported_t* supported, const char* value);
+BELLESIP_EXPORT void belle_sip_header_supported_set_supported(belle_sip_header_supported_t* supported, belle_sip_list_t* supported_values);
+BELLESIP_EXPORT belle_sip_list_t* belle_sip_header_supported_get_supported(const belle_sip_header_supported_t* supported);
+#define BELLE_SIP_HEADER_SUPPORTED(t) BELLE_SIP_CAST(t,belle_sip_header_supported_t)
+#define BELLE_SIP_SUPPORTED "Supported"
+
+/******************************
+ * Content Disposition header object inherent from header
+ *
+ ******************************/
+typedef struct _belle_sip_header_content_disposition belle_sip_header_content_disposition_t;
+BELLESIP_EXPORT belle_sip_header_content_disposition_t* belle_sip_header_content_disposition_new(void);
+BELLESIP_EXPORT belle_sip_header_content_disposition_t* belle_sip_header_content_disposition_parse (const char* content_disposition) ;
+BELLESIP_EXPORT belle_sip_header_content_disposition_t* belle_sip_header_content_disposition_create (const char* content_disposition);
+BELLESIP_EXPORT const char* belle_sip_header_content_disposition_get_content_disposition(const belle_sip_header_content_disposition_t* content_disposition);
+BELLESIP_EXPORT void belle_sip_header_content_disposition_set_content_disposition(belle_sip_header_content_disposition_t* obj,const char* content_disposition);
+#define BELLE_SIP_HEADER_CONTENT_DISPOSITION(t) BELLE_SIP_CAST(t,belle_sip_header_content_disposition_t)
+#define BELLE_SIP_CONTENT_DISPOSITION "Content-Disposition"
+
+/******************************
+ * Accept header object inherent from parameters
+ *
+ ******************************/
+typedef struct _belle_sip_header_accept belle_sip_header_accept_t;
+BELLESIP_EXPORT belle_sip_header_accept_t* belle_sip_header_accept_new(void);
+BELLESIP_EXPORT belle_sip_header_accept_t* belle_sip_header_accept_parse (const char* accept) ;
+BELLESIP_EXPORT belle_sip_header_accept_t* belle_sip_header_accept_create (const char* type,const char* sub_type) ;
+BELLESIP_EXPORT belle_sip_header_accept_t* belle_sip_header_accept_parse (const char* accept) ;
+BELLESIP_EXPORT const char*	belle_sip_header_accept_get_type(const belle_sip_header_accept_t* accept);
+BELLESIP_EXPORT void belle_sip_header_accept_set_type(belle_sip_header_accept_t* accept,const char* type);
+BELLESIP_EXPORT const char*	belle_sip_header_accept_get_subtype(const belle_sip_header_accept_t* accept);
+BELLESIP_EXPORT void belle_sip_header_accept_set_subtype(belle_sip_header_accept_t* accept,const char* sub_type);
+#define BELLE_SIP_HEADER_ACCEPT(t) BELLE_SIP_CAST(t,belle_sip_header_accept_t)
+#define BELLE_SIP_ACCEPT "Accept"
+
+/******************************
+ * Reason header object inherent from parameters
+ *
+ ******************************/
+typedef struct _belle_sip_header_reason belle_sip_header_reason_t;
+BELLESIP_EXPORT belle_sip_header_reason_t* belle_sip_header_reason_new(void);
+BELLESIP_EXPORT belle_sip_header_reason_t* belle_sip_header_reason_parse (const char* reason) ;
+BELLESIP_EXPORT const char*	belle_sip_header_reason_get_protocol(const belle_sip_header_reason_t* reason);
+BELLESIP_EXPORT void belle_sip_header_reason_set_protocol(belle_sip_header_reason_t* reason,const char* protocol);
+BELLESIP_EXPORT int	belle_sip_header_reason_get_cause(const belle_sip_header_reason_t* reason);
+BELLESIP_EXPORT void belle_sip_header_reason_set_cause(belle_sip_header_reason_t* reason,int cause);
+BELLESIP_EXPORT void belle_sip_header_reason_set_text(belle_sip_header_reason_t* reason,const char* text);
+BELLESIP_EXPORT const char*	belle_sip_header_reason_get_text(const belle_sip_header_reason_t* reason);
+
+#define BELLE_SIP_HEADER_REASON(t) BELLE_SIP_CAST(t,belle_sip_header_reason_t)
+#define BELLE_SIP_REASON "Reason"
+
+
+/******************************
+ * Authentication-Info header inherit from header
+ *
+ ******************************/
+typedef struct _belle_sip_header_authentication_info belle_sip_header_authentication_info_t;
+
+BELLESIP_EXPORT belle_sip_header_authentication_info_t* belle_sip_header_authentication_info_new(void);
+BELLESIP_EXPORT belle_sip_header_authentication_info_t* belle_sip_header_authentication_info_parse(const char* authentication_info );
+BELLESIP_EXPORT const char*	belle_sip_header_authentication_info_get_next_nonce(const belle_sip_header_authentication_info_t* authentication_info );
+BELLESIP_EXPORT void belle_sip_header_authentication_info_set_next_nonce(belle_sip_header_authentication_info_t* authentication_info, const char* next_nonce);
+
+/*limited to a sinle value*/
+BELLESIP_EXPORT const char*	belle_sip_header_authentication_info_get_qop(const belle_sip_header_authentication_info_t* authentication_info);
+BELLESIP_EXPORT void belle_sip_header_authentication_info_set_qop(belle_sip_header_authentication_info_t* authentication_info, const char* qop);
+
+BELLESIP_EXPORT const char*	belle_sip_header_authentication_info_get_rsp_auth(const belle_sip_header_authentication_info_t* rsp_auth);
+BELLESIP_EXPORT void belle_sip_header_authentication_info_set_rsp_auth(belle_sip_header_authentication_info_t* authentication_info, const char* rsp_auth);
+
+BELLESIP_EXPORT const char* belle_sip_header_authentication_info_get_cnonce(const belle_sip_header_authentication_info_t* authentication_info);
+BELLESIP_EXPORT void belle_sip_header_authentication_info_set_cnonce(belle_sip_header_authentication_info_t* authentication_info, const char* cNonce);
+
+BELLESIP_EXPORT void belle_sip_header_authentication_info_set_nonce_count(belle_sip_header_authentication_info_t* authentication_info, int nonceCount);
+BELLESIP_EXPORT int	belle_sip_header_authentication_info_get_nonce_count(const belle_sip_header_authentication_info_t* authentication_info);
+
+
+#define BELLE_SIP_HEADER_AUTHENTICATION_INFO(t) BELLE_SIP_CAST(t,belle_sip_header_authentication_info_t)
+#define BELLE_SIP_AUTHENTICATION_INFO "Authentication-Info"
+
 BELLE_SIP_END_DECLS
 
 
