@@ -39,7 +39,7 @@ BELLESIP_EXPORT belle_sip_message_t* belle_sip_message_parse_raw (const char* bu
 
 
 BELLESIP_EXPORT int belle_sip_message_is_request(belle_sip_message_t *msg);
-BELLESIP_EXPORT belle_sip_request_t* belle_sip_request_new();
+BELLESIP_EXPORT belle_sip_request_t* belle_sip_request_new(void);
 BELLESIP_EXPORT belle_sip_request_t* belle_sip_request_parse(const char* raw);
 
 BELLESIP_EXPORT belle_sip_request_t* belle_sip_request_create(belle_sip_uri_t *requri, const char* method,
@@ -120,6 +120,9 @@ BELLESIP_EXPORT void belle_sip_message_remove_last(belle_sip_message_t *msg, con
 
 BELLESIP_EXPORT void belle_sip_message_remove_header(belle_sip_message_t *msg, const char *header_name);
 
+BELLESIP_EXPORT void belle_sip_message_remove_header_from_ptr(belle_sip_message_t *msg, belle_sip_header_t* header);
+
+
 BELLESIP_EXPORT char *belle_sip_message_to_string(belle_sip_message_t *msg);
 
 BELLESIP_EXPORT belle_sip_body_handler_t *belle_sip_message_get_body_handler(const belle_sip_message_t *msg);
@@ -130,9 +133,9 @@ BELLESIP_EXPORT const char* belle_sip_message_get_body(belle_sip_message_t *msg)
 
 BELLESIP_EXPORT size_t belle_sip_message_get_body_size(const belle_sip_message_t *msg);
 
-BELLESIP_EXPORT void belle_sip_message_set_body(belle_sip_message_t *msg,const char* body,unsigned int size);
+BELLESIP_EXPORT void belle_sip_message_set_body(belle_sip_message_t *msg,const char* body,size_t size);
 
-BELLESIP_EXPORT void belle_sip_message_assign_body(belle_sip_message_t *msg, char* body, unsigned int size);
+BELLESIP_EXPORT void belle_sip_message_assign_body(belle_sip_message_t *msg, char* body, size_t size);
 
 BELLESIP_EXPORT int belle_sip_response_get_status_code(const belle_sip_response_t *response);
 BELLESIP_EXPORT void belle_sip_response_set_status_code(belle_sip_response_t *response,int status);
