@@ -21,11 +21,14 @@
 
 #include "belle-sip/defs.h"
 #include "belle-sip/utils.h"
+#include "belle-sip/list.h"
 
 /*
  * typedefs, macros and functions for object definition and manipulation.
  */
 
+#define BELLE_SIP_DEFAULT_BUFSIZE_HINT 0
+
 #define BELLE_SIP_TYPE_ID(_type) _type##_id
 
 typedef unsigned int belle_sip_type_id_t;
@@ -52,6 +55,16 @@ typedef unsigned int belle_sip_type_id_t;
 	struct object_type##_vptr_struct{\
 		BELLE_SIP_OBJECT_VPTR_TYPE(parent_type) base;
 
+#define BELLE_SIP_DECLARE_VPTR_NO_EXPORT(object_type) \
+	typedef belle_sip_object_vptr_t BELLE_SIP_OBJECT_VPTR_TYPE(object_type);\
+	BELLE_SIP_OBJECT_VPTR_TYPE(object_type) * BELLE_SIP_OBJECT_GET_VPTR_FUNC(object_type)(void);
+
+#define BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN_NO_EXPORT(object_type, parent_type) \
+	typedef struct object_type##_vptr_struct BELLE_SIP_OBJECT_VPTR_TYPE(object_type);\
+	BELLE_SIP_OBJECT_VPTR_TYPE(object_type) * BELLE_SIP_OBJECT_GET_VPTR_FUNC(object_type)(void); \
+	struct object_type##_vptr_struct{\
+		BELLE_SIP_OBJECT_VPTR_TYPE(parent_type) base;
+
 #define BELLE_SIP_DECLARE_CUSTOM_VPTR_END };
 
 #define BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(object_type) \
@@ -77,7 +90,8 @@ typedef unsigned int belle_sip_type_id_t;
 		BELLE_SIP_VPTR_INIT(object_type,parent_type,unowned), \
 		(belle_sip_object_destroy_t)destroy,	\
 		(belle_sip_object_clone_t)clone,	\
-		(belle_sip_object_marshal_t)marshal\
+		(belle_sip_object_marshal_t)marshal,\
+		BELLE_SIP_DEFAULT_BUFSIZE_HINT\
 		}; \
 		BELLE_SIP_OBJECT_VPTR_TYPE(object_type) * BELLE_SIP_OBJECT_GET_VPTR_FUNC(object_type)(void){\
 			return &BELLE_SIP_OBJECT_VPTR_NAME(object_type); \
@@ -127,7 +141,7 @@ typedef struct _belle_sip_object belle_sip_object_t;
 
 typedef void (*belle_sip_object_destroy_t)(belle_sip_object_t*);
 typedef void (*belle_sip_object_clone_t)(belle_sip_object_t* obj, const belle_sip_object_t *orig);
-typedef int (*belle_sip_object_marshal_t)(belle_sip_object_t* obj, char* buff, size_t buff_size, size_t *offset);
+typedef belle_sip_error_code (*belle_sip_object_marshal_t)(belle_sip_object_t* obj, char* buff, size_t buff_size, size_t *offset);
 typedef struct _belle_sip_object_vptr *(*belle_sip_object_get_vptr_t)(void);
 
 struct _belle_sip_object_vptr{
@@ -151,8 +165,8 @@ struct _belle_sip_object{
 	char* name;
 	struct weak_ref *weak_refs;
 	struct belle_sip_object_pool *pool;
-	struct _belle_sip_list *pool_iterator;
-	struct _belle_sip_list *data_store;
+	belle_sip_list_t *pool_iterator;
+	belle_sip_list_t *data_store;
 };
 
 
@@ -183,6 +197,8 @@ BELLESIP_EXPORT void belle_sip_object_enable_leak_detector(int enable);
 
 BELLESIP_EXPORT int belle_sip_object_get_object_count(void);
 
+BELLESIP_EXPORT void belle_sip_object_flush_active_objects(void);
+
 BELLESIP_EXPORT void belle_sip_object_dump_active_objects(void);
 
 /**
@@ -223,11 +239,12 @@ BELLESIP_EXPORT void belle_sip_object_weak_unref(void *obj, belle_sip_object_des
 /**
  * Set object name.
 **/
-void belle_sip_object_set_name(belle_sip_object_t *obj,const char* name);
+BELLESIP_EXPORT void belle_sip_object_set_name(belle_sip_object_t *obj,const char* name);
+
 /**
  * Get object name.
 **/
-const char* belle_sip_object_get_name(belle_sip_object_t *obj);
+BELLESIP_EXPORT const char* belle_sip_object_get_name(belle_sip_object_t *obj);
 
 /*copy the content of ref object to new object, for the part they have in common in their inheritence diagram*/
 void _belle_sip_object_copy(belle_sip_object_t *newobj, const belle_sip_object_t *ref);
@@ -436,7 +453,7 @@ BELLE_SIP_BEGIN_DECLS
 **/
 BELLESIP_EXPORT belle_sip_object_pool_t * belle_sip_object_pool_push(void);
 
-belle_sip_object_pool_t * belle_sip_object_pool_get_current();
+belle_sip_object_pool_t * belle_sip_object_pool_get_current(void);
 int belle_sip_object_pool_cleanable(belle_sip_object_pool_t *pool);
 void belle_sip_object_pool_clean(belle_sip_object_pool_t *obj);
 
