@@ -18,6 +18,7 @@
 
 #ifndef REFRESHER_HELPER_H_
 #define REFRESHER_HELPER_H_
+
 #define BELLE_SIP_REFRESHER_REUSE_EXPIRES -1
 
 BELLE_SIP_BEGIN_DECLS
@@ -29,11 +30,12 @@ typedef struct belle_sip_refresher belle_sip_refresher_t;
  * @param user_pointer user pointer
  * @param status_code status code for the last refresh action
  * @param reason_phrase
+ * @param will_retry a boolean indicating wether the refresher is going to retry the request automatically.
  * */
 typedef void (*belle_sip_refresher_listener_t) (belle_sip_refresher_t* refresher
-												,void* user_pointer
-												,unsigned int status_code
-												,const char* reason_phrase);
+								,void* user_pointer
+								,unsigned int status_code
+								,const char* reason_phrase, int will_retry);
 
 /**
  * add a refresher listener
@@ -102,9 +104,18 @@ BELLESIP_EXPORT const belle_sip_list_t* belle_sip_refresher_get_auth_events(cons
 BELLESIP_EXPORT void belle_sip_refresher_enable_manual_mode(belle_sip_refresher_t *refresher, int enabled);
 
 /**
- * Retrieve last know public ip:port contact used.
+ * Retrieve current local address used by the underlying refresher's channel.
 **/
-BELLESIP_EXPORT char* belle_sip_refresher_get_public_uri(belle_sip_refresher_t* refresher);
+BELLESIP_EXPORT const char * belle_sip_refresher_get_local_address(belle_sip_refresher_t* refresher, int *port);
+
+/**
+ * Retrieve current public address used by the underlying refresher's channel.
+**/
+BELLESIP_EXPORT const char * belle_sip_refresher_get_public_address(belle_sip_refresher_t* refresher, int *port);
+/**
+ * Retrieve last know contact header if known. Only available after a successful registration.
+**/
+BELLESIP_EXPORT belle_sip_header_contact_t* belle_sip_refresher_get_contact(const belle_sip_refresher_t* refresher);
 
 BELLE_SIP_END_DECLS
 
