@@ -75,7 +75,7 @@ belle_sip_error_code belle_sdp_attribute_marshal(belle_sdp_attribute_t* attribut
 }
 belle_sdp_attribute_t* belle_sdp_attribute_create(const char* name, const char* value) {
 	belle_sdp_attribute_t* ret;
-	int i;
+	size_t i;
 	size_t elements = sizeof(attribute_table) / sizeof(attribute_table[0]);
 
 	if (!name || name[0] == '\0') {
@@ -516,7 +516,7 @@ belle_sip_error_code belle_sdp_media_marshal(belle_sdp_media_t* media, char* buf
 	error=belle_sip_snprintf(buff,buff_size,offset," %s",media->protocol);
 	if (error!=BELLE_SIP_OK) return error;
 	for(;list!=NULL;list=list->next){
-		error=belle_sip_snprintf(buff,buff_size,offset," %li",(long)list->data);
+		error=belle_sip_snprintf(buff,buff_size,offset," %li",(long)(intptr_t)list->data);
 		if (error!=BELLE_SIP_OK) return error;
 	}
 	return error;
@@ -874,7 +874,7 @@ static const size_t payload_list_elements=sizeof(static_payload_list)/sizeof(str
 
 static int mime_parameter_is_static(const belle_sdp_mime_parameter_t *param){
 	const struct static_payload* iterator;
-	int i;
+	size_t i;
 
 	for (iterator = static_payload_list,i=0;i<payload_list_elements;i++,iterator++) {
 		if (iterator->number == param->media_format &&
@@ -889,7 +889,7 @@ static int mime_parameter_is_static(const belle_sdp_mime_parameter_t *param){
 
 static int mime_parameter_fill_from_static(belle_sdp_mime_parameter_t *mime_parameter,int format) {
 	const struct static_payload* iterator;
-	int i;
+	size_t i;
 
 	for (iterator = static_payload_list,i=0;i<payload_list_elements;i++,iterator++) {
 		if (iterator->number == format) {
@@ -977,7 +977,7 @@ belle_sip_list_t* belle_sdp_media_description_build_mime_parameters(const belle_
 		mime_parameter = belle_sdp_mime_parameter_new();
 		belle_sdp_mime_parameter_set_ptime(mime_parameter,ptime_as_int);
 		belle_sdp_mime_parameter_set_max_ptime(mime_parameter,max_ptime_as_int);
-		belle_sdp_mime_parameter_set_media_format(mime_parameter,(int)(long)media_formats->data);
+		belle_sdp_mime_parameter_set_media_format(mime_parameter,(int)(intptr_t)media_formats->data);
 
 		/*get rtpmap*/
 		rtpmap = belle_sdp_media_description_a_attr_value_get_with_pt(media_description
@@ -1008,7 +1008,7 @@ void belle_sdp_media_description_append_values_from_mime_parameter(belle_sdp_med
 	int current_max_ptime=0;
 
 	belle_sdp_media_set_media_formats(media,belle_sip_list_append(belle_sdp_media_get_media_formats(media)
-																,(void*)(long)(belle_sdp_mime_parameter_get_media_format(mime_parameter))));
+																,(void*)(intptr_t)(belle_sdp_mime_parameter_get_media_format(mime_parameter))));
 
 	if (belle_sdp_media_description_get_attribute_value(media_description,"ptime")) {
 		current_ptime=atoi(belle_sdp_media_description_get_attribute_value(media_description,"ptime"));
@@ -1152,7 +1152,7 @@ belle_sip_error_code belle_sdp_origin_marshal(belle_sdp_origin_t* origin, char*
 	return belle_sip_snprintf(	buff
 								,buff_size
 								,offset
-								,"o=%s %i %i %s %s %s"
+								,"o=%s %u %u %s %s %s"
 								,origin->username
 								,origin->session_id
 								,origin->session_version
@@ -1276,20 +1276,26 @@ belle_sip_error_code belle_sdp_session_description_marshal(belle_sdp_session_des
 	belle_sip_list_t* times;
 	belle_sip_list_t* attributes;
 
-	error=belle_sip_object_marshal(BELLE_SIP_OBJECT(session_description->version),buff,buff_size,offset);
-	if (error!=BELLE_SIP_OK) return error;
-	error=belle_sip_snprintf(buff, buff_size, offset, "\r\n");
-	if (error!=BELLE_SIP_OK) return error;
+	if (session_description->version) {
+		error=belle_sip_object_marshal(BELLE_SIP_OBJECT(session_description->version),buff,buff_size,offset);
+		if (error!=BELLE_SIP_OK) return error;
+		error=belle_sip_snprintf(buff, buff_size, offset, "\r\n");
+		if (error!=BELLE_SIP_OK) return error;
+	}
 
-	error=belle_sip_object_marshal(BELLE_SIP_OBJECT(session_description->origin),buff,buff_size,offset);
-	if (error!=BELLE_SIP_OK) return error;
-	error=belle_sip_snprintf(buff, buff_size, offset, "\r\n");
-	if (error!=BELLE_SIP_OK) return error;
+	if (session_description->origin) {
+		error=belle_sip_object_marshal(BELLE_SIP_OBJECT(session_description->origin),buff,buff_size,offset);
+		if (error!=BELLE_SIP_OK) return error;
+		error=belle_sip_snprintf(buff, buff_size, offset, "\r\n");
+		if (error!=BELLE_SIP_OK) return error;
+	}
 
-	error=belle_sip_object_marshal(BELLE_SIP_OBJECT(session_description->session_name),buff,buff_size,offset);
-	if (error!=BELLE_SIP_OK) return error;
-	error=belle_sip_snprintf(buff, buff_size, offset, "\r\n");
-	if (error!=BELLE_SIP_OK) return error;
+	if (session_description->session_name) {
+		error=belle_sip_object_marshal(BELLE_SIP_OBJECT(session_description->session_name),buff,buff_size,offset);
+		if (error!=BELLE_SIP_OK) return error;
+		error=belle_sip_snprintf(buff, buff_size, offset, "\r\n");
+		if (error!=BELLE_SIP_OK) return error;
+	}
 
 	error=belle_sdp_base_description_marshal((belle_sdp_base_description_t*)(&session_description->base_description),buff,buff_size, offset);
 	if (error!=BELLE_SIP_OK) return error;
@@ -1321,6 +1327,10 @@ BELLE_SDP_NEW(session_description,belle_sdp_base_description)
 BELLE_SDP_PARSE(session_description)
 
 
+belle_sip_list_t * belle_sdp_session_description_get_attributes(const belle_sdp_session_description_t *session_description) {
+	return belle_sdp_base_description_get_attributes(BELLE_SIP_CAST(session_description, belle_sdp_base_description_t));
+}
+
 const char*	belle_sdp_session_description_get_attribute_value(const belle_sdp_session_description_t* session_description, const char* name) {
 	return belle_sdp_base_description_get_attribute_value(BELLE_SIP_CAST(session_description,belle_sdp_base_description_t),name);
 }
