@@ -49,6 +49,8 @@ static struct header_name_func_pair  header_table[] = {
 	,{PROTO_SIP, 			BELLE_SIP_FROM,					(header_parse_func)belle_sip_header_from_parse}
 	,{PROTO_SIP, 			"t",							(header_parse_func)belle_sip_header_to_parse}
 	,{PROTO_SIP, 			BELLE_SIP_TO,					(header_parse_func)belle_sip_header_to_parse}
+	,{PROTO_SIP, 			"d",							(header_parse_func)belle_sip_header_diversion_parse}
+	,{PROTO_SIP, 			BELLE_SIP_DIVERSION,			(header_parse_func)belle_sip_header_diversion_parse}
 	,{PROTO_SIP, 			"i",							(header_parse_func)belle_sip_header_call_id_parse}
 	,{PROTO_SIP, 			BELLE_SIP_CALL_ID,				(header_parse_func)belle_sip_header_call_id_parse}
 	,{PROTO_SIP, 			"l",							(header_parse_func)belle_sip_header_content_length_parse}
@@ -81,6 +83,8 @@ static struct header_name_func_pair  header_table[] = {
 	,{PROTO_SIP, 			"k",							(header_parse_func)belle_sip_header_supported_parse}
 	,{PROTO_SIP, 			BELLE_SIP_CONTENT_DISPOSITION,	(header_parse_func)belle_sip_header_content_disposition_parse}
 	,{PROTO_SIP, 			BELLE_SIP_ACCEPT,				(header_parse_func)belle_sip_header_accept_parse}
+	,{PROTO_SIP, 			BELLE_SIP_REASON,				(header_parse_func)belle_sip_header_reason_parse}
+	,{PROTO_SIP,			BELLE_SIP_AUTHENTICATION_INFO,	(header_parse_func)belle_sip_header_authentication_info_parse}
 };
 
 static belle_sip_header_t* belle_header_create(const char* name,const char* value,int protocol) {
@@ -308,6 +312,13 @@ belle_sip_header_address_t* belle_sip_header_address_create2(const char* display
 	return address;
 }
 
+/*fast header address implemenation*/
+typedef belle_sip_header_address_t belle_sip_fast_header_address_t;
+#define belle_sip_fast_header_address_parse belle_sip_header_address_fast_parse  
+BELLE_SIP_PARSE(fast_header_address)
+
+
+
 /******************************
  * Extension header inherits from header
  *
@@ -568,6 +579,71 @@ const char *belle_sip_header_to_get_tag(const belle_sip_header_to_t *obj){
 	return belle_sip_header_to_get_raw_tag(obj);
 }
 
+/**************************
+* Diversion header object inherits from header_address
+****************************
+*/
+struct _belle_sip_header_diversion  {
+	belle_sip_header_address_t address;
+};
+
+static void belle_sip_header_diversion_destroy(belle_sip_header_diversion_t* diversion) {
+}
+
+void belle_sip_header_diversion_clone(belle_sip_header_diversion_t *contact, const belle_sip_header_diversion_t *orig){
+}
+
+belle_sip_error_code belle_sip_header_diversion_marshal(belle_sip_header_diversion_t* diversion, char* buff, size_t buff_size, size_t *offset) {
+	BELLE_SIP_FROM_LIKE_MARSHAL(diversion,FALSE)
+}
+
+BELLE_SIP_NEW_HEADER(header_diversion,header_address,BELLE_SIP_DIVERSION)
+BELLE_SIP_PARSE(header_diversion)
+GET_SET_STRING_PARAM2(belle_sip_header_diversion,tag,raw_tag);
+
+belle_sip_header_diversion_t* belle_sip_header_diversion_create2(const char *uri, const char *tag){
+	belle_sip_header_address_t* address = belle_sip_header_address_parse(uri);
+	if (address) {
+		belle_sip_header_diversion_t* diversion = belle_sip_header_diversion_create(address,tag);
+		belle_sip_object_unref(address);
+		return diversion;
+	} else
+		return NULL;
+}
+belle_sip_header_diversion_t* belle_sip_header_diversion_create(const belle_sip_header_address_t* address, const char *tag) {
+	belle_sip_header_diversion_t* header= belle_sip_header_diversion_new();
+	belle_sip_uri_t* uri;
+	_belle_sip_object_copy((belle_sip_object_t*)header,(belle_sip_object_t*)address);
+	/*clear unwanted uri components*/
+	if ((uri=belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(header)))) {
+		belle_sip_parameters_t* params=BELLE_SIP_PARAMETERS(uri);
+		belle_sip_parameters_remove_parameter(params,"lr");
+		belle_sip_parameters_remove_parameter(params,"ttl");
+		belle_sip_parameters_remove_parameter(params,"method");
+		belle_sip_parameters_remove_parameter(params,"maddr");
+		belle_sip_parameters_remove_parameter(params,"transport");
+		belle_sip_uri_set_port(uri,0);
+		belle_sip_uri_headers_clean(uri);
+	}
+	belle_sip_header_set_name(BELLE_SIP_HEADER(header),BELLE_SIP_DIVERSION); /*restaure header name*/
+	if (tag) belle_sip_header_diversion_set_tag(header,tag);
+	return header;
+}
+void belle_sip_header_diversion_set_random_tag(belle_sip_header_diversion_t *obj){
+	char tmp[8];
+	/*not less than 32bit */
+	belle_sip_header_diversion_set_tag(obj,belle_sip_random_token(tmp,sizeof(tmp)));
+}
+
+void belle_sip_header_diversion_set_tag(belle_sip_header_diversion_t *obj, const char *tag){
+	if (tag==BELLE_SIP_RANDOM_TAG) belle_sip_header_diversion_set_random_tag(obj);
+	else belle_sip_header_diversion_set_raw_tag(obj,tag);
+}
+
+const char *belle_sip_header_diversion_get_tag(const belle_sip_header_diversion_t *obj){
+	return belle_sip_header_diversion_get_raw_tag(obj);
+}
+
 
 /******************************
  * User-Agent header inherits from header
@@ -1594,7 +1670,7 @@ static void escaped_to_ascii(const char*a,char*b,size_t n) {
 	size_t index_a=0,index_b=0;
 
 	while (a[index_a]!='\0'&& index_a<n)
-		index_a+=belle_sip_get_char(a+index_a,n-index_a,b+index_b++);
+		index_a+=belle_sip_get_char(a+index_a,b+index_b++);
 }
 
 #define REPLACES_PREF_OFFSET (strlen(BELLE_SIP_REPLACES)+2)
@@ -1954,4 +2030,136 @@ belle_sip_header_accept_t* belle_sip_header_accept_create (const char* type,cons
 GET_SET_STRING(belle_sip_header_accept,type);
 GET_SET_STRING(belle_sip_header_accept,subtype);
 
+/******************************
+ * Reason header object inherent from parameters
+ *
+ ******************************/
+struct _belle_sip_header_reason  {
+	belle_sip_parameters_t params_list;
+	const char* protocol;
+	const char* unquoted_text;
+};
 
+static void belle_sip_header_reason_destroy(belle_sip_header_reason_t* reason) {
+	DESTROY_STRING(reason,protocol);
+	DESTROY_STRING(reason,unquoted_text);
+}
+
+static void belle_sip_header_reason_clone(belle_sip_header_reason_t* reason, const belle_sip_header_reason_t* orig){
+	CLONE_STRING(belle_sip_header_reason,protocol,reason,orig)
+}
+
+belle_sip_error_code belle_sip_header_reason_marshal(belle_sip_header_reason_t* reason, char* buff, size_t buff_size, size_t *offset) {
+	belle_sip_error_code error=belle_sip_header_marshal(BELLE_SIP_HEADER(reason), buff, buff_size, offset);
+	if (error!=BELLE_SIP_OK) return error;
+		error=belle_sip_snprintf(buff,buff_size,offset,"%s ",reason->protocol);
+	if (error!=BELLE_SIP_OK) return error;
+		error=belle_sip_parameters_marshal(BELLE_SIP_PARAMETERS(reason), buff, buff_size, offset);
+	if (error!=BELLE_SIP_OK) return error;
+	if (reason->unquoted_text)
+		error=belle_sip_snprintf(buff,buff_size,offset,"; text=\"%s\"",reason->unquoted_text);
+	return error;
+}
+
+GET_SET_STRING(belle_sip_header_reason,unquoted_text);
+
+void belle_sip_header_reason_set_text(belle_sip_header_reason_t* reason,const char* text) {
+	belle_sip_parameters_remove_parameter(BELLE_SIP_PARAMETERS(reason),"text"); /*just in case*/
+	belle_sip_header_reason_set_unquoted_text((belle_sip_header_reason_t*)reason, text);
+}
+BELLESIP_EXPORT const char*	belle_sip_header_reason_get_text(const belle_sip_header_reason_t* reason) {
+	if (!reason->unquoted_text) {
+		/*try from params*/
+		const char * quoted = belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(reason), "text");
+		if (quoted) {
+			char* unquoted = _belle_sip_str_dup_and_unquote_string(quoted);
+			belle_sip_header_reason_set_unquoted_text((belle_sip_header_reason_t*)reason, unquoted); /*change internal param, ,even if reason is const*/
+			belle_sip_parameters_remove_parameter(BELLE_SIP_PARAMETERS(reason),"text");
+			belle_sip_free(unquoted);
+		}
+
+	}
+	return reason->unquoted_text;
+}
+
+GET_SET_STRING(belle_sip_header_reason,protocol);
+
+GET_SET_INT_PARAM(belle_sip_header_reason,cause,int);
+BELLE_SIP_PARSE(header_reason)
+BELLE_SIP_NEW_HEADER(header_reason,parameters,BELLE_SIP_REASON)
+
+/******************************
+ * AuthenticationInfo header hinerite from header
+ *
+ ******************************/
+
+struct _belle_sip_header_authentication_info  {
+	belle_sip_header_t header;
+	const char* rsp_auth;
+	const char* cnonce;
+	int nonce_count;
+	const char* qop;
+	const char* next_nonce;
+};
+static void belle_sip_header_authentication_info_destroy(belle_sip_header_authentication_info_t* authentication_info) {
+	DESTROY_STRING(authentication_info,rsp_auth);
+	DESTROY_STRING(authentication_info,cnonce);
+	DESTROY_STRING(authentication_info,qop);
+	DESTROY_STRING(authentication_info,next_nonce);
+	
+}
+
+static void belle_sip_header_authentication_info_clone(	belle_sip_header_authentication_info_t* authentication_info
+													   , const belle_sip_header_authentication_info_t* orig){
+	CLONE_STRING(belle_sip_header_authentication_info,rsp_auth,authentication_info,orig)
+	CLONE_STRING(belle_sip_header_authentication_info,cnonce,authentication_info,orig)
+	CLONE_STRING(belle_sip_header_authentication_info,qop,authentication_info,orig)
+	CLONE_STRING(belle_sip_header_authentication_info,next_nonce,authentication_info,orig)
+}
+
+belle_sip_error_code belle_sip_header_authentication_info_marshal(belle_sip_header_authentication_info_t* authentication_info, char* buff, size_t buff_size, size_t *offset) {
+	char* border="";
+	belle_sip_error_code error=belle_sip_header_marshal(BELLE_SIP_HEADER(authentication_info), buff, buff_size, offset);
+	if (error!=BELLE_SIP_OK) return error;
+	
+	if (authentication_info->rsp_auth) {
+		error=belle_sip_snprintf(buff,buff_size,offset,"%srspauth=\"%s\"", border,authentication_info->rsp_auth);
+		border=", ";
+	}
+	if (error!=BELLE_SIP_OK) return error;
+
+	if (authentication_info->cnonce) {
+		error=belle_sip_snprintf(buff,buff_size,offset,"%scnonce=\"%s\"", border, authentication_info->cnonce);
+		border=", ";
+	}
+	if (error!=BELLE_SIP_OK) return error;
+
+	if (authentication_info->nonce_count >= 0) {
+		error=belle_sip_snprintf(buff,buff_size,offset,"%snc=%08x", border, authentication_info->nonce_count);
+		border=", ";
+	}
+	if (error!=BELLE_SIP_OK) return error;
+
+	if (authentication_info->qop) {
+		error=belle_sip_snprintf(buff,buff_size,offset,"%sqop=%s", border, authentication_info->qop);
+		border=", ";
+	}
+	if (error!=BELLE_SIP_OK) return error;
+	
+	if (authentication_info->next_nonce) {
+		error=belle_sip_snprintf(buff,buff_size,offset,"%snextnonce=\"%s\"", border, authentication_info->next_nonce);
+	}
+	return error;
+
+
+}
+void belle_sip_header_authentication_info_init(belle_sip_header_authentication_info_t* header_authentication) {
+	header_authentication->nonce_count=-1;
+}
+BELLE_SIP_NEW_HEADER_INIT(header_authentication_info,header,BELLE_SIP_AUTHENTICATION_INFO,header_authentication_info)
+BELLE_SIP_PARSE(header_authentication_info)
+GET_SET_STRING(belle_sip_header_authentication_info,rsp_auth);
+GET_SET_STRING(belle_sip_header_authentication_info,qop);
+GET_SET_STRING(belle_sip_header_authentication_info,next_nonce);
+GET_SET_STRING(belle_sip_header_authentication_info,cnonce);
+GET_SET_INT(belle_sip_header_authentication_info,nonce_count,int);
