@@ -26,9 +26,11 @@
 
 /* include all public headers*/
 #include "belle-sip/belle-sip.h"
+#include "bctoolbox/map.h"
 
 #include "port.h"
 #include <bctoolbox/port.h>
+#include <bctoolbox/vconnect.h>
 
 #ifdef HAVE_CONFIG_H
 
@@ -80,6 +82,16 @@
 			__method=BELLE_SIP_INTERFACE_GET_METHODS(__obj,interface_name)->method;\
 			if (__method) BELLE_SIP_INTERFACE_GET_METHODS(__obj,interface_name)->
 
+#define __BELLE_SIP_INVOKE_LISTENER_REVERSE_BEGIN(list,interface_name,method) \
+	if (list!=NULL) {\
+		belle_sip_list_t *__copy=belle_sip_list_copy_reverse_with_data((list), (void* (*)(void*))belle_sip_object_ref);\
+		const belle_sip_list_t *__elem=__copy;\
+		do{\
+			void *__method;\
+			interface_name *__obj=(interface_name*)__elem->data;\
+			__method=BELLE_SIP_INTERFACE_GET_METHODS(__obj,interface_name)->method;\
+			if (__method) BELLE_SIP_INTERFACE_GET_METHODS(__obj,interface_name)->
+
 #define __BELLE_SIP_INVOKE_LISTENER_END \
 			__elem=__elem->next;\
 		}while(__elem!=NULL);\
@@ -96,12 +108,16 @@
 	method(__obj,arg);\
 	__BELLE_SIP_INVOKE_LISTENER_END
 
-
 #define BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(list,interface_name,method,arg1,arg2) \
 			__BELLE_SIP_INVOKE_LISTENER_BEGIN(list,interface_name,method)\
 			method(__obj,arg1,arg2);\
 			__BELLE_SIP_INVOKE_LISTENER_END
 
+#define BELLE_SIP_INVOKE_LISTENERS_REVERSE_ARG1_ARG2(list,interface_name,method,arg1,arg2) \
+			__BELLE_SIP_INVOKE_LISTENER_REVERSE_BEGIN(list,interface_name,method)\
+			method(__obj,arg1,arg2);\
+			__BELLE_SIP_INVOKE_LISTENER_END
+
 #define BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2_ARG3(list,interface_name,method,arg1,arg2,arg3) \
 			__BELLE_SIP_INVOKE_LISTENER_BEGIN(list,interface_name)\
 			method(__obj,arg1,arg2,arg3);\
@@ -139,6 +155,7 @@ BELLE_SIP_DECLARE_VPTR(belle_sip_header_contact_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_from_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_to_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_via_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_header_diversion_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_uri_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_message_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_request_t);
@@ -209,9 +226,13 @@ BELLE_SIP_DECLARE_VPTR(belle_sip_header_event_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_supported_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_content_disposition_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_accept_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_header_reason_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_header_authentication_info_t);
+
 
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_resolver_context_t,belle_sip_source_t)
 	void (*cancel)(belle_sip_resolver_context_t *);
+	void (*notify)(belle_sip_resolver_context_t *);
 BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_simple_resolver_context_t,belle_sip_resolver_context_t)
@@ -223,6 +244,8 @@ BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_dual_resolver_context_t,belle_sip_resolver_context_t)
 BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 
+void belle_sip_resolver_context_notify(belle_sip_resolver_context_t *ctx);
+
 struct belle_sip_source{
 	belle_sip_object_t base;
 	belle_sip_list_t node;
@@ -244,6 +267,8 @@ struct belle_sip_source{
 	unsigned char expired;
 	unsigned char oneshot;
 	unsigned char notify_required; /*for testing purpose, use to ask for being scheduled*/
+	bctbx_iterator_t *it; /*for fast removal*/
+	belle_sip_main_loop_t *ml; 
 };
 
 void belle_sip_socket_source_init(belle_sip_source_t *s, belle_sip_source_func_t func, void *data, belle_sip_socket_t fd, unsigned int events, unsigned int timeout_value_ms);
@@ -528,11 +553,11 @@ struct belle_sip_stack{
 	unsigned char dns_search_enabled;
 };
 
-belle_sip_hop_t* belle_sip_hop_new(const char* transport, const char *cname, const char* host,int port);
-belle_sip_hop_t* belle_sip_hop_new_from_uri(const belle_sip_uri_t *uri);
-belle_sip_hop_t* belle_sip_hop_new_from_generic_uri(const belle_generic_uri_t *uri);
+BELLESIP_EXPORT belle_sip_hop_t* belle_sip_hop_new(const char* transport, const char *cname, const char* host,int port);
+BELLESIP_EXPORT belle_sip_hop_t* belle_sip_hop_new_from_uri(const belle_sip_uri_t *uri);
+BELLESIP_EXPORT belle_sip_hop_t* belle_sip_hop_new_from_generic_uri(const belle_generic_uri_t *uri);
 
-belle_sip_hop_t * belle_sip_stack_get_next_hop(belle_sip_stack_t *stack, belle_sip_request_t *req);
+BELLESIP_EXPORT belle_sip_hop_t * belle_sip_stack_get_next_hop(belle_sip_stack_t *stack, belle_sip_request_t *req);
 
 /*
  belle_sip_provider_t
@@ -554,12 +579,12 @@ struct belle_sip_provider{
 	unsigned short unconditional_answer;
 };
 
-BELLESIP_INTERNAL_EXPORT belle_sip_provider_t *belle_sip_provider_new(belle_sip_stack_t *s, belle_sip_listening_point_t *lp);
+BELLESIP_EXPORT belle_sip_provider_t *belle_sip_provider_new(belle_sip_stack_t *s, belle_sip_listening_point_t *lp);
 void belle_sip_provider_add_client_transaction(belle_sip_provider_t *prov, belle_sip_client_transaction_t *t);
 belle_sip_client_transaction_t *belle_sip_provider_find_matching_client_transaction(belle_sip_provider_t *prov, belle_sip_response_t *resp);
 void belle_sip_provider_remove_client_transaction(belle_sip_provider_t *prov, belle_sip_client_transaction_t *t);
 void belle_sip_provider_add_server_transaction(belle_sip_provider_t *prov, belle_sip_server_transaction_t *t);
-BELLESIP_INTERNAL_EXPORT belle_sip_server_transaction_t * belle_sip_provider_find_matching_server_transaction(belle_sip_provider_t *prov,belle_sip_request_t *req);
+BELLESIP_EXPORT belle_sip_server_transaction_t * belle_sip_provider_find_matching_server_transaction(belle_sip_provider_t *prov,belle_sip_request_t *req);
 void belle_sip_provider_remove_server_transaction(belle_sip_provider_t *prov, belle_sip_server_transaction_t *t);
 void belle_sip_provider_set_transaction_terminated(belle_sip_provider_t *p, belle_sip_transaction_t *t);
 void *belle_sip_transaction_get_application_data_internal(const belle_sip_transaction_t *t);
@@ -572,7 +597,7 @@ void belle_sip_provider_remove_internal_sip_listener(belle_sip_provider_t *p, be
 belle_sip_client_transaction_t * belle_sip_provider_find_matching_client_transaction_from_req(belle_sip_provider_t *prov, belle_sip_request_t *req);
 belle_sip_dialog_t *belle_sip_provider_find_dialog_from_message(belle_sip_provider_t *prov, belle_sip_message_t *msg, int as_uas);
 /*for testing purpose only:*/
-BELLESIP_INTERNAL_EXPORT void belle_sip_provider_dispatch_message(belle_sip_provider_t *prov, belle_sip_message_t *msg);
+BELLESIP_EXPORT void belle_sip_provider_dispatch_message(belle_sip_provider_t *prov, belle_sip_message_t *msg);
 
 typedef struct listener_ctx{
 	belle_sip_listener_t *listener;
@@ -647,12 +672,14 @@ struct belle_sip_transaction{
 	belle_sip_response_t *last_response;
 	belle_sip_channel_t *channel;
 	belle_sip_dialog_t *dialog;
+	belle_sip_source_t *call_repair_timer;
 	char *branch_id;
 	belle_sip_transaction_state_t state;
 	void *appdata;
 	unsigned char is_internal;
 	unsigned char timed_out;
 	unsigned char sent_by_dialog_queue;
+	unsigned long bg_task_id;
 };
 
 
@@ -786,12 +813,17 @@ BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 belle_sip_nist_t * belle_sip_nist_new(belle_sip_provider_t *prov, belle_sip_request_t *req);
 
 
+typedef enum belle_sip_dialog_type{
+	BELLE_SIP_DIALOG_INVITE,
+	BELLE_SIP_DIALOG_SUBSCRIBE_NOTIFY
+}belle_sip_dialog_type_t;
 /*
  * Dialogs
  */
 struct belle_sip_dialog{
 	belle_sip_object_t base;
 	void *appdata;
+	belle_sip_dialog_type_t type;
 	belle_sip_provider_t *provider;
 	belle_sip_request_t *last_out_invite;
 	belle_sip_request_t *last_out_ack; /*so that it can be retransmitted when needed*/
@@ -834,6 +866,8 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj,belle_sip_transaction_t* tra
 void belle_sip_dialog_check_ack_sent(belle_sip_dialog_t*obj);
 int belle_sip_dialog_handle_ack(belle_sip_dialog_t *obj, belle_sip_request_t *ack);
 void belle_sip_dialog_queue_client_transaction(belle_sip_dialog_t *dialog, belle_sip_client_transaction_t *tr);
+void belle_sip_dialog_stop_200Ok_retrans(belle_sip_dialog_t *obj);
+
 
 /*
  belle_sip_response_t
@@ -956,18 +990,18 @@ belle_sip_refresher_t* belle_sip_refresher_new(belle_sip_client_transaction_t* t
 
 /*
  * returns a char, even if entry is escaped*/
-size_t belle_sip_get_char (const char*a,size_t n,char*out);
+size_t belle_sip_get_char (const char*a,char*out);
 /*return an escaped string*/
-BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_username(const char* buff) ;
-BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_userpasswd(const char* buff) ;
-BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_parameter(const char* buff) ;
-BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_header(const char* buff) ;
+BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_username(const char* buff) ;
+BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_userpasswd(const char* buff) ;
+BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_parameter(const char* buff) ;
+BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_header(const char* buff) ;
 
 
 /*(uri RFC 2396)*/
 
-BELLESIP_INTERNAL_EXPORT char* belle_generic_uri_to_escaped_query(const char* buff);
-BELLESIP_INTERNAL_EXPORT char* belle_generic_uri_to_escaped_path(const char* buff);
+BELLESIP_EXPORT char* belle_generic_uri_to_escaped_query(const char* buff);
+BELLESIP_EXPORT char* belle_generic_uri_to_escaped_path(const char* buff);
 
 #define BELLE_SIP_SOCKET_TIMEOUT 30000
 
@@ -1003,7 +1037,7 @@ belle_sip_header_extension_t* belle_sip_header_extension_new(void);
 
 belle_sip_header_extension_t* belle_sip_header_extension_parse (const char* extension) ;
 belle_sip_header_extension_t* belle_sip_header_extension_create (const char* name,const char* value);
-BELLESIP_INTERNAL_EXPORT const char* belle_sip_header_extension_get_value(const belle_sip_header_extension_t* extension);
+BELLESIP_EXPORT const char* belle_sip_header_extension_get_value(const belle_sip_header_extension_t* extension);
 void belle_sip_header_extension_set_value(belle_sip_header_extension_t* extension,const char* value);
 #define BELLE_SIP_HEADER_EXTENSION(t) BELLE_SIP_CAST(t,belle_sip_header_extension_t)
 
@@ -1014,14 +1048,16 @@ void belle_sip_header_extension_set_value(belle_sip_header_extension_t* extensio
 
 
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_body_handler_t,belle_sip_object_t)
-	void (*begin_transfer)(belle_sip_body_handler_t *obj);
+	void (*begin_recv_transfer)(belle_sip_body_handler_t *obj);
+	void (*begin_send_transfer)(belle_sip_body_handler_t *obj);
 	void (*end_transfer)(belle_sip_body_handler_t *obj);
-	void (*chunk_recv)(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset, const uint8_t *buf, size_t size);
-	int (*chunk_send)(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset, uint8_t *buf, size_t * size);
+	void (*chunk_recv)(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t size);
+	int (*chunk_send)(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t * size);
 BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 
-void belle_sip_body_handler_begin_transfer(belle_sip_body_handler_t *obj);
-void belle_sip_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, const uint8_t *buf, size_t size);
+void belle_sip_body_handler_begin_recv_transfer(belle_sip_body_handler_t *obj);
+void belle_sip_body_handler_begin_send_transfer(belle_sip_body_handler_t *obj);
+void belle_sip_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, uint8_t *buf, size_t size);
 int belle_sip_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, uint8_t *buf, size_t *size);
 void belle_sip_body_handler_end_transfer(belle_sip_body_handler_t *obj);
 
@@ -1052,4 +1088,8 @@ void belle_sip_multipart_body_handler_progress_cb(belle_sip_body_handler_t *obj,
  */
 belle_sip_list_t *belle_sip_parse_directory(const char *path, const char *file_type);
 
+typedef struct authorization_context authorization_context_t;
+BELLESIP_EXPORT void belle_sip_authorization_destroy(authorization_context_t* object);
+
+
 #endif
