@@ -18,7 +18,8 @@
 
 #include "belle-sip/belle-sip.h"
 #include "belle_sip_internal.h"
-
+#include "bctoolbox/map.h"
+#include <limits.h>
 
 #ifndef _WIN32
 #include <unistd.h>
@@ -177,6 +178,10 @@ void belle_sip_source_uninit(belle_sip_source_t *obj){
 #endif
 	obj->fd=(belle_sip_fd_t)-1;
 	obj->sock=(belle_sip_socket_t)-1;
+/*	if (obj->it) {
+		bctbx_iterator_delete(obj->it);
+		obj->it=NULL;
+	}*/
 }
 
 void belle_sip_source_set_notify(belle_sip_source_t *s, belle_sip_source_func_t func) {
@@ -245,32 +250,51 @@ belle_sip_socket_t belle_sip_source_get_socket(const belle_sip_source_t* source)
 
 struct belle_sip_main_loop{
 	belle_sip_object_t base;
-	belle_sip_list_t *sources;
+	belle_sip_list_t *fd_sources;
+	bctbx_map_t *timer_sources;
 	belle_sip_object_pool_t *pool;
 	int nsources;
 	int run;
 	int in_loop;
+	bctbx_mutex_t timer_sources_mutex;
 };
 
 void belle_sip_main_loop_remove_source(belle_sip_main_loop_t *ml, belle_sip_source_t *source){
-	if (!source->node.next && !source->node.prev && &source->node!=ml->sources) return; /*nothing to do*/
-	source->cancelled=TRUE;
-	ml->sources=belle_sip_list_remove_link(ml->sources,&source->node);
-	ml->nsources--;
-	
-	if (source->on_remove)
-		source->on_remove(source);
-	belle_sip_object_unref(source);
+	bool_t elem_removed = FALSE;
+	if (source->node.next || source->node.prev || &source->node==ml->fd_sources)  {
+		ml->fd_sources=belle_sip_list_remove_link(ml->fd_sources,&source->node);
+		belle_sip_object_unref(source);
+		elem_removed = TRUE;
+	}
+	if (source->it) {
+		bctbx_mutex_lock(&ml->timer_sources_mutex);
+		bctbx_map_erase(ml->timer_sources, source->it);
+		bctbx_iterator_delete(source->it);
+		bctbx_mutex_unlock(&ml->timer_sources_mutex);
+
+		source->it=NULL;
+		belle_sip_object_unref(source);
+		elem_removed = TRUE;
+	}
+	if (elem_removed) {
+		source->cancelled=TRUE;
+		ml->nsources--;
+		if (source->on_remove)
+			source->on_remove(source);
+		
+	}
 }
 
 
 static void belle_sip_main_loop_destroy(belle_sip_main_loop_t *ml){
-	while (ml->sources){
-		belle_sip_main_loop_remove_source(ml,(belle_sip_source_t*)ml->sources->data);
+	while (ml->fd_sources){
+		belle_sip_main_loop_remove_source(ml,(belle_sip_source_t*)ml->fd_sources->data);
 	}
 	if (belle_sip_object_pool_cleanable(ml->pool)){
 		belle_sip_object_unref(ml->pool);
 	}
+	bctbx_mmap_ullong_delete(ml->timer_sources);
+	bctbx_mutex_destroy(&ml->timer_sources_mutex);
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_main_loop_t);
@@ -279,6 +303,8 @@ BELLE_SIP_INSTANCIATE_VPTR(belle_sip_main_loop_t,belle_sip_object_t,belle_sip_ma
 belle_sip_main_loop_t *belle_sip_main_loop_new(void){
 	belle_sip_main_loop_t*m=belle_sip_object_new(belle_sip_main_loop_t);
 	m->pool=belle_sip_object_pool_push();
+	m->timer_sources = bctbx_mmap_ullong_new();
+	bctbx_mutex_init(&m->timer_sources_mutex,NULL);
 	return m;
 }
 
@@ -291,25 +317,49 @@ void belle_sip_main_loop_add_source(belle_sip_main_loop_t *ml, belle_sip_source_
 		belle_sip_fatal("Insane source passed to belle_sip_main_loop_add_source() !");
 		return;
 	}
-	belle_sip_object_ref(source);
+	
+	source->ml=ml;
+	
 	if (source->timeout>=0){
+		belle_sip_object_ref(source);
 		source->expire_ms=belle_sip_time_ms()+source->timeout;
+		bctbx_mutex_lock(&ml->timer_sources_mutex);
+		source->it = bctbx_map_insert_and_delete_with_returned_it(ml->timer_sources
+																	  , (bctbx_pair_t*)bctbx_pair_ullong_new(source->expire_ms, source));
+		bctbx_mutex_unlock(&ml->timer_sources_mutex);
+
 	}
 	source->cancelled=FALSE;
-	ml->sources=belle_sip_list_prepend_link(ml->sources,&source->node);
+	if (source->fd != (belle_sip_fd_t)-1 ) {
+		belle_sip_object_ref(source);
+		ml->fd_sources=belle_sip_list_prepend_link(ml->fd_sources,&source->node);
+	}
+
 	ml->nsources++;
 }
 
-belle_sip_source_t* belle_sip_main_loop_create_timeout(belle_sip_main_loop_t *ml
-														, belle_sip_source_func_t func
-														, void *data
-														, unsigned int timeout_value_ms
-														,const char* timer_name) {
+belle_sip_source_t* belle_sip_main_loop_create_timeout_with_remove_cb(  belle_sip_main_loop_t *ml
+																	  , belle_sip_source_func_t func
+																	  , void *data
+																	  , unsigned int timeout_value_ms
+																	  , const char* timer_name
+																	  , belle_sip_source_remove_callback_t remove_func) {
 	belle_sip_source_t * s=belle_sip_timeout_source_new(func,data,timeout_value_ms);
 	belle_sip_object_set_name((belle_sip_object_t*)s,timer_name);
+	if (remove_func) {
+		belle_sip_source_set_remove_cb(s, remove_func);
+	}
 	belle_sip_main_loop_add_source(ml,s);
 	return s;
 }
+belle_sip_source_t* belle_sip_main_loop_create_timeout(belle_sip_main_loop_t *ml
+																	  , belle_sip_source_func_t func
+																	  , void *data
+																	  , unsigned int timeout_value_ms
+																	  ,const char* timer_name) {
+	return belle_sip_main_loop_create_timeout_with_remove_cb(ml, func, data, timeout_value_ms,timer_name,NULL);
+	
+}
 
 unsigned long belle_sip_main_loop_add_timeout(belle_sip_main_loop_t *ml, belle_sip_source_func_t func, void *data, unsigned int timeout_value_ms){
 	belle_sip_source_t * s=belle_sip_main_loop_create_timeout(ml,func,data,timeout_value_ms,"Timer");
@@ -326,18 +376,40 @@ void belle_sip_main_loop_do_later(belle_sip_main_loop_t *ml, belle_sip_callback_
 
 void belle_sip_source_set_timeout(belle_sip_source_t *s, unsigned int value_ms){
 	if (!s->expired){
+		belle_sip_main_loop_t *ml = s->ml;
 		s->expire_ms=belle_sip_time_ms()+value_ms;
+		if (s->it){
+			/*this timeout is already sorted in the timer_sources map, we need to move it to its new place*/
+			bctbx_mutex_lock(&ml->timer_sources_mutex);
+			bctbx_map_erase(ml->timer_sources, s->it);
+			bctbx_iterator_delete(s->it);
+			s->it = bctbx_map_insert_and_delete_with_returned_it(ml->timer_sources,
+				(bctbx_pair_t*)bctbx_pair_ullong_new(s->expire_ms, s));
+			bctbx_mutex_unlock(&ml->timer_sources_mutex);
+		}
 	}
 	s->timeout=value_ms;
 }
+
 void belle_sip_source_set_remove_cb(belle_sip_source_t *s, belle_sip_source_remove_callback_t func) {
 	s->on_remove=func;
 }
+
 unsigned int belle_sip_source_get_timeout(const belle_sip_source_t *s){
 	return s->timeout;
 }
+
 void belle_sip_source_cancel(belle_sip_source_t *s){
-	if (s) s->cancelled=TRUE;
+	s->cancelled=TRUE;
+	if (s->it) {
+		bctbx_mutex_lock(&s->ml->timer_sources_mutex);
+		bctbx_map_erase(s->ml->timer_sources, s->it);
+		bctbx_iterator_delete(s->it);
+		/*put on front*/
+		s->it = bctbx_map_insert_and_delete_with_returned_it(s->ml->timer_sources, (bctbx_pair_t*)bctbx_pair_ullong_new(0, s));
+		
+		bctbx_mutex_unlock(&s->ml->timer_sources_mutex);
+	}
 }
 
 static int match_source_id(const void *s, const void *pid){
@@ -348,16 +420,24 @@ static int match_source_id(const void *s, const void *pid){
 }
 
 belle_sip_source_t *belle_sip_main_loop_find_source(belle_sip_main_loop_t *ml, unsigned long id){
-	belle_sip_list_t *elem=belle_sip_list_find_custom(ml->sources,match_source_id,(const void*)(intptr_t)id);
-	if (elem!=NULL){
-		return (belle_sip_source_t*)elem->data;
-	}
-	return NULL;
+	bctbx_iterator_t *it;
+	belle_sip_source_t *ret=NULL;
+	belle_sip_list_t *elem=belle_sip_list_find_custom(ml->fd_sources,match_source_id,(const void*)(intptr_t)id);
+	if (elem!=NULL) {
+		ret = (belle_sip_source_t*)elem->data;
+	} else if ((it = bctbx_map_find_custom(ml->timer_sources, match_source_id, (const void*)(intptr_t)id))) {
+		ret = (belle_sip_source_t*)bctbx_pair_get_second(bctbx_iterator_get_pair(it));
+		bctbx_iterator_delete(it);
+	} /*else
+		ret = NULL;*/
+	
+	return ret;
+	
 }
 
 void belle_sip_main_loop_cancel_source(belle_sip_main_loop_t *ml, unsigned long id){
 	belle_sip_source_t *s=belle_sip_main_loop_find_source(ml,id);
-	belle_sip_source_cancel(s);
+	if (s) belle_sip_source_cancel(s);
 }
 
 static void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
@@ -366,13 +446,13 @@ static void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 	int i=0;
 	belle_sip_source_t *s;
 	belle_sip_list_t *elem,*next;
-	uint64_t min_time_ms=(uint64_t)-1;
 	int duration=-1;
 	int ret;
 	uint64_t cur;
 	belle_sip_list_t *to_be_notified=NULL;
 	int can_clean=belle_sip_object_pool_cleanable(ml->pool); /*iterate might not be called by the thread that created the main loop*/ 
 	belle_sip_object_pool_t *tmp_pool=NULL;
+	bctbx_iterator_t *it,*end;
 	
 	if (!can_clean){
 		/*Push a temporary pool for the time of the iterate loop*/
@@ -380,7 +460,7 @@ static void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 	}
 	
 	/*Step 1: prepare the pollfd table and get the next timeout value */
-	for(elem=ml->sources;elem!=NULL;elem=next){
+	for(elem=ml->fd_sources;elem!=NULL;elem=next) {
 		next=elem->next;
 		s=(belle_sip_source_t*)elem->data;
 		if (!s->cancelled){
@@ -388,24 +468,26 @@ static void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 				belle_sip_source_to_poll(s,pfd,i);
 				++i;
 			}
-			if (s->timeout>=0){
-				if (min_time_ms>s->expire_ms){
-					min_time_ms=s->expire_ms;
-				}
-			}
 		}
 	}
-	
-	if (min_time_ms!=(uint64_t)-1 ){
+	/*all source with timeout are in ml->timer_sources*/
+	if (bctbx_map_size(ml->timer_sources) >0) {
 		int64_t diff;
+		uint64_t next_wakeup_time;
+		it = bctbx_map_begin(ml->timer_sources);
+		/*use first because in case of canceled timer, key ==0 , key != s->expire_ms */
+		next_wakeup_time = bctbx_pair_ullong_get_first((const bctbx_pair_ullong_t *)bctbx_iterator_get_pair(it));
 		/* compute the amount of time to wait for shortest timeout*/
 		cur=belle_sip_time_ms();
-		diff=min_time_ms-cur;
+		diff=next_wakeup_time-cur;
 		if (diff>0)
-			duration=(int)diff;
+			duration=MIN((unsigned int)diff,INT_MAX);
 		else 
 			duration=0;
+		bctbx_iterator_delete(it);
+		it = NULL;
 	}
+	
 	/* do the poll */
 	ret=belle_sip_poll(pfd,i,duration);
 	if (ret==-1){
@@ -414,10 +496,9 @@ static void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 	
 	/* Step 2: examine poll results and determine the list of source to be notified */
 	cur=belle_sip_time_ms();
-	for(elem=ml->sources;elem!=NULL;elem=elem->next){
+	for(elem=ml->fd_sources;elem!=NULL;elem=elem->next){
 		unsigned revents=0;
 		s=(belle_sip_source_t*)elem->data;
-
 		if (!s->cancelled){
 			if (s->fd!=(belle_sip_fd_t)-1){
 				if (s->notify_required) { /*for testing purpose to force channel to read*/
@@ -427,17 +508,42 @@ static void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 					revents=belle_sip_source_get_revents(s,pfd);
 				}
 				s->revents=revents;
+			} else {
+				belle_sip_error("Source [%p] does not contains any fd !",s);
 			}
-			if (revents!=0 || (s->timeout>=0 && cur>=s->expire_ms)){
+			if (revents!=0){
 				to_be_notified=belle_sip_list_append(to_be_notified,belle_sip_object_ref(s));
-				s->expired=TRUE;
-				if (s->revents==0)
-					s->revents=BELLE_SIP_EVENT_TIMEOUT;
 			}
 		}else to_be_notified=belle_sip_list_append(to_be_notified,belle_sip_object_ref(s));
 	}
+
+	/* Step 3: find timeouted sources */
 	
-	/* Step 3: notify those to be notified */
+	bctbx_mutex_lock(&ml->timer_sources_mutex); /*iterator chain might be alterated by element insertion*/
+	it = bctbx_map_begin(ml->timer_sources);
+	end = bctbx_map_end(ml->timer_sources);
+	while (!bctbx_iterator_equals(it,end)) {
+		/*use first because in case of canceled timer, key != s->expire_ms*/
+		uint64_t expire = bctbx_pair_ullong_get_first((const bctbx_pair_ullong_t *)bctbx_iterator_get_pair(it));
+		s = (belle_sip_source_t*)bctbx_pair_get_second(bctbx_iterator_get_pair(it));
+		if (expire > cur) {
+			/* no need to continue looping because map is ordered*/
+			break;
+		} else {
+			if (s->revents==0) {
+				s->expired=TRUE;
+				to_be_notified=belle_sip_list_append(to_be_notified,belle_sip_object_ref(s));
+			} /*else already in to_be_notified by Step 2*/
+			
+			s->revents|=BELLE_SIP_EVENT_TIMEOUT;
+			it=bctbx_iterator_get_next(it);
+		}
+	}
+	bctbx_iterator_delete(it);
+	bctbx_iterator_delete(end);
+	bctbx_mutex_unlock(&ml->timer_sources_mutex);
+	
+	/* Step 4: notify those to be notified */
 	for(elem=to_be_notified;elem!=NULL;){
 		s=(belle_sip_source_t*)elem->data;
 		next=elem->next;
@@ -454,24 +560,46 @@ static void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 			if (ret==BELLE_SIP_STOP || s->oneshot){
 				/*this source needs to be removed*/
 				belle_sip_main_loop_remove_source(ml,s);
-			}else if (s->revents==BELLE_SIP_EVENT_TIMEOUT){
-				/*timeout needs to be started again */
-				if (ret==BELLE_SIP_CONTINUE_WITHOUT_CATCHUP){
-					s->expire_ms=cur+s->timeout;
-				}else{
-					s->expire_ms+=s->timeout;
+			} else  {
+				if (s->expired && s->it) {
+					bctbx_mutex_lock(&ml->timer_sources_mutex);
+					bctbx_map_erase(ml->timer_sources, s->it);
+					bctbx_iterator_delete(s->it);
+					bctbx_mutex_unlock(&ml->timer_sources_mutex);
+					s->it=NULL;
+					belle_sip_object_unref(s);
+				}
+				if (!s->it && s->timeout >= 0){
+					/*timeout needs to be started again */
+					if (ret==BELLE_SIP_CONTINUE_WITHOUT_CATCHUP){
+						s->expire_ms=cur+s->timeout;
+					}else{
+						s->expire_ms+=s->timeout;
+					}
+					s->expired=FALSE;
+					bctbx_mutex_lock(&ml->timer_sources_mutex);
+					s->it = bctbx_map_insert_and_delete_with_returned_it(ml->timer_sources,
+							(bctbx_pair_t*)bctbx_pair_ullong_new(s->expire_ms, s));
+					bctbx_mutex_unlock(&ml->timer_sources_mutex);
+					belle_sip_object_ref(s);
 				}
-				s->expired=FALSE;
 			}
-		}else belle_sip_main_loop_remove_source(ml,s);
+		} else {
+			belle_sip_main_loop_remove_source(ml,s);
+		}
+		s->revents=0;
 		belle_sip_object_unref(s);
 		belle_sip_free(elem); /*free just the element*/
 		elem=next;
 	}
 	
 	if (can_clean) belle_sip_object_pool_clean(ml->pool);
-	else if (tmp_pool) belle_sip_object_unref(tmp_pool);
+	else if (tmp_pool) {
+		belle_sip_object_unref(tmp_pool);
+		tmp_pool=NULL;
+	}
 end:
+	
 	belle_sip_free(pfd);
 }
 
