@@ -18,8 +18,10 @@
 
 #include "belle_sip_internal.h"
 #include "dns.h"
+#include <bctoolbox/defs.h>
 
 #include <stdlib.h>
+#include <stdint.h>
 #ifdef __APPLE__
 #include "TargetConditionals.h"
 #endif
@@ -51,7 +53,7 @@ struct belle_sip_dns_srv{
 	belle_sip_combined_resolver_context_t *root_resolver;/* used internally to combine SRV and A queries*/
 	belle_sip_resolver_context_t *a_resolver; /* used internally to combine SRV and A queries*/
 	struct addrinfo *a_results; /* used internally to combine SRV and A queries*/
-	
+
 };
 
 static void belle_sip_dns_srv_destroy(belle_sip_dns_srv_t *obj){
@@ -61,6 +63,7 @@ static void belle_sip_dns_srv_destroy(belle_sip_dns_srv_t *obj){
 	}
 	if (obj->a_resolver){
 		belle_sip_resolver_context_cancel(obj->a_resolver);
+		belle_sip_object_unref(obj->a_resolver);
 		obj->a_resolver=NULL;
 	}
 	if (obj->a_results){
@@ -101,8 +104,10 @@ BELLE_SIP_INSTANCIATE_VPTR(belle_sip_dns_srv_t, belle_sip_object_t,belle_sip_dns
 struct belle_sip_resolver_context{
 	belle_sip_source_t source;
 	belle_sip_stack_t *stack;
-	uint8_t done;
-	uint8_t pad[3];
+	uint32_t min_ttl;
+	uint8_t notified;
+	uint8_t cancelled;
+	uint8_t pad[2];
 };
 
 struct belle_sip_simple_resolver_context{
@@ -122,7 +127,6 @@ struct belle_sip_simple_resolver_context{
 	int family;
 	int flags;
 	uint64_t start_time;
-	unsigned char results_notified;
 #ifdef USE_GETADDRINFO_FALLBACK
 	struct addrinfo *getaddrinfo_ai_list;
 	belle_sip_source_t *getaddrinfo_source;
@@ -159,12 +163,14 @@ struct belle_sip_dual_resolver_context{
 	belle_sip_resolver_context_t *aaaa_ctx;
 	struct addrinfo *a_results;
 	struct addrinfo *aaaa_results;
-	unsigned char a_done;
-	unsigned char aaaa_done;
+	uint8_t a_notified;
+	uint8_t aaaa_notified;
+	uint8_t pad[2];
 };
 
 void belle_sip_resolver_context_init(belle_sip_resolver_context_t *obj, belle_sip_stack_t *stack){
 	obj->stack=stack;
+	obj->min_ttl = UINT32_MAX;
 	belle_sip_init_sockets(); /* Need to be called for DNS resolution to work on Windows platform. */
 }
 
@@ -193,10 +199,10 @@ static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx)
 		belle_sip_error("%s dns_resconf_open error: %s", __FUNCTION__, dns_strerror(error));
 		return NULL;
 	}
-	
+
 	path = belle_sip_stack_get_dns_resolv_conf_file(ctx->base.stack);
 	servers = ctx->base.stack->dns_servers;
-	
+
 	if (servers){
 		belle_sip_message("%s using application supplied dns server list.", __FUNCTION__);
 		error = dns_resconf_nameservers_from_list(ctx->resconf, servers);
@@ -216,7 +222,7 @@ static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx)
 		if (error) {
 			belle_sip_error("%s dns_resconf_loadwin error", __FUNCTION__);
 		}
-#elif defined(ANDROID)
+#elif defined(__ANDROID__)
 		error = dns_resconf_loadandroid(ctx->resconf);
 		if (error) {
 			belle_sip_error("%s dns_resconf_loadandroid error", __FUNCTION__);
@@ -248,18 +254,18 @@ static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx)
 			return NULL;
 		}
 	}
-	
+
 	if (error==0){
 		char ip[64];
 		char serv[10];
 		int using_ipv6=FALSE;
-		int i;
-		
+		size_t i;
+
 		belle_sip_message("Resolver is using DNS server(s):");
 		for(i=0;i<sizeof(ctx->resconf->nameserver)/sizeof(ctx->resconf->nameserver[0]);++i){
 			struct sockaddr *ns_addr=(struct sockaddr*)&ctx->resconf->nameserver[i];
 			if (ns_addr->sa_family==AF_UNSPEC) break;
-			getnameinfo(ns_addr,ns_addr->sa_family==AF_INET6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr)
+			bctbx_getnameinfo(ns_addr,ns_addr->sa_family==AF_INET6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr)
 					,ip,sizeof(ip),serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
 			belle_sip_message("\t%s",ip);
 			if (ns_addr->sa_family==AF_INET6) using_ipv6=TRUE;
@@ -337,7 +343,7 @@ static belle_sip_dns_srv_t *srv_elect_one(belle_sip_list_t *srv_list){
 	belle_sip_list_t *elem;
 	belle_sip_dns_srv_t *srv;
 	int rand_number;
-	
+
 	for(elem=srv_list;elem!=NULL;elem=elem->next){
 		srv=(belle_sip_dns_srv_t*)elem->data;
 		sum+=srv->weight;
@@ -408,30 +414,56 @@ static belle_sip_list_t *srv_select_by_weight(belle_sip_list_t *srv_list){
 	return srv_list;/*no weight election was necessary, return original list*/
 }
 
-static void notify_results(belle_sip_simple_resolver_context_t *ctx){
-	ctx->base.done=TRUE;
-	if (!ctx->results_notified) {
-		if ((ctx->type == DNS_T_A) || (ctx->type == DNS_T_AAAA)) {
-			struct addrinfo **ai_list = &ctx->ai_list;
+static void simple_resolver_context_notify(belle_sip_resolver_context_t *obj) {
+	belle_sip_simple_resolver_context_t *ctx = BELLE_SIP_SIMPLE_RESOLVER_CONTEXT(obj);
+	if ((ctx->type == DNS_T_A) || (ctx->type == DNS_T_AAAA)) {
+		struct addrinfo **ai_list = &ctx->ai_list;
 #if USE_GETADDRINFO_FALLBACK
-			if (ctx->getaddrinfo_ai_list != NULL) ai_list = &ctx->getaddrinfo_ai_list;
+		if (ctx->getaddrinfo_ai_list != NULL) ai_list = &ctx->getaddrinfo_ai_list;
 #endif
-			ctx->cb(ctx->cb_data, ctx->name, *ai_list);
-			*ai_list = NULL;
-			ctx->results_notified = TRUE;
-		} else if (ctx->type == DNS_T_SRV) {
-			ctx->srv_list = srv_select_by_weight(ctx->srv_list);
-			ctx->srv_cb(ctx->srv_cb_data, ctx->name, ctx->srv_list);
-		}
+		ctx->cb(ctx->cb_data, ctx->name, *ai_list, BELLE_SIP_RESOLVER_CONTEXT(obj)->min_ttl);
+		*ai_list = NULL;
+	} else if (ctx->type == DNS_T_SRV) {
+		ctx->srv_list = srv_select_by_weight(ctx->srv_list);
+		ctx->srv_cb(ctx->srv_cb_data, ctx->name, ctx->srv_list, BELLE_SIP_RESOLVER_CONTEXT(obj)->min_ttl);
 	}
 }
 
+static void dual_resolver_context_notify(belle_sip_resolver_context_t *obj) {
+	belle_sip_dual_resolver_context_t *ctx = BELLE_SIP_DUAL_RESOLVER_CONTEXT(obj);
+	struct addrinfo *results = ctx->aaaa_results;
+	results = ai_list_append(results, ctx->a_results);
+	ctx->a_results = NULL;
+	ctx->aaaa_results = NULL;
+	ctx->cb(ctx->cb_data, ctx->name, results, BELLE_SIP_RESOLVER_CONTEXT(obj)->min_ttl);
+}
+
+static void combined_resolver_context_cleanup(belle_sip_combined_resolver_context_t *ctx) {
+	if (ctx->srv_ctx) {
+		belle_sip_object_unref(ctx->srv_ctx);
+		ctx->srv_ctx = NULL;
+	}
+	if (ctx->a_fallback_ctx) {
+		belle_sip_object_unref(ctx->a_fallback_ctx);
+		ctx->a_fallback_ctx = NULL;
+	}
+	belle_sip_list_free_with_data(ctx->srv_results, belle_sip_object_unref);
+	ctx->srv_results = NULL;
+}
+
+static void combined_resolver_context_notify(belle_sip_resolver_context_t *obj) {
+	belle_sip_combined_resolver_context_t *ctx = BELLE_SIP_COMBINED_RESOLVER_CONTEXT(obj);
+	ctx->cb(ctx->cb_data, ctx->name, ctx->final_results, BELLE_SIP_RESOLVER_CONTEXT(obj)->min_ttl);
+	ctx->final_results = NULL;
+	combined_resolver_context_cleanup(ctx);
+}
+
 static void append_dns_result(belle_sip_simple_resolver_context_t *ctx, struct addrinfo **ai_list, struct sockaddr *addr, socklen_t addrlen){
 	char host[NI_MAXHOST + 1];
 	int gai_err;
 	int family=ctx->family;
-	
-	if ((gai_err=getnameinfo(addr, addrlen, host, sizeof(host), NULL, 0, NI_NUMERICHOST)) != 0){
+
+	if ((gai_err=bctbx_getnameinfo(addr, addrlen, host, sizeof(host), NULL, 0, NI_NUMERICHOST)) != 0){
 		belle_sip_error("append_dns_result(): getnameinfo() failed: %s",gai_strerror(gai_err));
 		return;
 	}
@@ -454,10 +486,10 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 		belle_sip_warning("Simulating DNS timeout");
 		simulated_timeout=1;
 	}
-	
-	if (simulated_timeout || ((revents & BELLE_SIP_EVENT_TIMEOUT) && (belle_sip_time_ms()-ctx->start_time>=timeout))) {
+
+	if (simulated_timeout || ((revents & BELLE_SIP_EVENT_TIMEOUT) && ((int)(belle_sip_time_ms()-ctx->start_time)>=timeout))) {
 		belle_sip_error("%s timed-out", __FUNCTION__);
-		notify_results(ctx);
+		belle_sip_resolver_context_notify(BELLE_SIP_RESOLVER_CONTEXT(ctx));
 		return BELLE_SIP_STOP;
 	}
 	dns_res_enable_search(ctx->R, search_enabled);
@@ -486,6 +518,7 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 					sin6.sin6_family = AF_INET6;
 					sin6.sin6_port = ctx->port;
 					append_dns_result(ctx,&ctx->ai_list,(struct sockaddr*)&sin6,sizeof(sin6));
+					if (rr.ttl < BELLE_SIP_RESOLVER_CONTEXT(ctx)->min_ttl) BELLE_SIP_RESOLVER_CONTEXT(ctx)->min_ttl = rr.ttl;
 				} else if ((ctx->type == DNS_T_A) && (rr.class == DNS_C_IN) && (rr.type == DNS_T_A)) {
 					struct dns_a *a = &any.a;
 					struct sockaddr_in sin;
@@ -494,6 +527,7 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 					sin.sin_family = AF_INET;
 					sin.sin_port = ctx->port;
 					append_dns_result(ctx,&ctx->ai_list,(struct sockaddr*)&sin,sizeof(sin));
+					if (rr.ttl < BELLE_SIP_RESOLVER_CONTEXT(ctx)->min_ttl) BELLE_SIP_RESOLVER_CONTEXT(ctx)->min_ttl = rr.ttl;
 				} else if ((ctx->type == DNS_T_SRV) && (rr.class == DNS_C_IN) && (rr.type == DNS_T_SRV)) {
 					char host[NI_MAXHOST + 1];
 					struct dns_srv *srv = &any.srv;
@@ -501,6 +535,7 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 					snprintf(host, sizeof(host), "[target:%s port:%d prio:%d weight:%d]", srv->target, srv->port, srv->priority, srv->weight);
 					ctx->srv_list = belle_sip_list_insert_sorted(ctx->srv_list, belle_sip_object_ref(b_srv), srv_compare_prio);
 					belle_sip_message("SRV %s resolved to %s", ctx->name, host);
+					if (rr.ttl < BELLE_SIP_RESOLVER_CONTEXT(ctx)->min_ttl) BELLE_SIP_RESOLVER_CONTEXT(ctx)->min_ttl = rr.ttl;
 				}
 			}
 		}
@@ -508,7 +543,10 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 #ifdef USE_GETADDRINFO_FALLBACK
 		ctx->getaddrinfo_cancelled = TRUE;
 #endif
-		notify_results(ctx);
+		if (dns_res_was_asymetric(ctx->R)){
+			belle_sip_warning("DNS answer was not received from the DNS server IP address the request was sent to. This seems to be a known issue with NAT64 networks created by Apple computers.");
+		}
+		belle_sip_resolver_context_notify(BELLE_SIP_RESOLVER_CONTEXT(ctx));
 		return BELLE_SIP_STOP;
 	}
 	if (error != DNS_EAGAIN) {
@@ -521,7 +559,7 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 			return BELLE_SIP_CONTINUE;
 		}
 #else
-		notify_results(ctx);
+		belle_sip_resolver_context_notify(BELLE_SIP_RESOLVER_CONTEXT(ctx));
 		return BELLE_SIP_STOP;
 #endif
 	} else {
@@ -572,7 +610,7 @@ static void * _resolver_getaddrinfo_thread(void *ptr) {
 
 static int _resolver_getaddrinfo_callback(belle_sip_simple_resolver_context_t *ctx, unsigned int revents) {
 	if (!ctx->getaddrinfo_cancelled) {
-		notify_results(ctx);
+		belle_sip_resolver_context_notify(BELLE_SIP_RESOLVER_CONTEXT(ctx));
 	}
 	belle_sip_object_unref(ctx);
 	return BELLE_SIP_STOP;
@@ -634,7 +672,7 @@ static int _resolver_send_query(belle_sip_simple_resolver_context_t *ctx) {
 static int resolver_process_data_delayed(belle_sip_simple_resolver_context_t *ctx, unsigned int revents) {
 	int err=_resolver_send_query(ctx);
 	if (err==0) return BELLE_SIP_CONTINUE;
-	
+
 	return BELLE_SIP_STOP;
 }
 
@@ -656,6 +694,11 @@ static int _resolver_start_query(belle_sip_simple_resolver_context_t *ctx) {
 		return -1;
 
 	memset(&opts, 0, sizeof opts);
+	
+	/* When there are IPv6 nameservers, allow responses to arrive from an IP address that is not the IP address to which the request was sent originally.
+		* Mac' NAT64 network tend to do this sometimes.*/
+	opts.udp_uses_connect = ctx->resconf->iface.ss_family != AF_INET6;
+	if (!opts.udp_uses_connect) belle_sip_message("Resolver is not using connect().");
 
 	if (!(ctx->R = dns_res_open(ctx->resconf, ctx->hosts, dns_hints_mortal(dns_hints_local(ctx->resconf, &error)), cache(ctx), &opts, &error))) {
 		belle_sip_error("%s dns_res_open error [%s]: %s", __FUNCTION__, ctx->name, dns_strerror(error));
@@ -668,23 +711,27 @@ static int _resolver_start_query(belle_sip_simple_resolver_context_t *ctx) {
 	} else {
 		error=_resolver_send_query(ctx);
 	}
-	if (error==0 && !ctx->base.done) belle_sip_main_loop_add_source(ctx->base.stack->ml,(belle_sip_source_t*)ctx);
+	if (error==0 && !ctx->base.notified) belle_sip_main_loop_add_source(ctx->base.stack->ml,(belle_sip_source_t*)ctx);
 	return error;
 }
 
-static belle_sip_simple_resolver_context_t * resolver_start_query(belle_sip_simple_resolver_context_t *ctx){
-	int error=_resolver_start_query(ctx);
-	if (error==0){
-		if (!ctx->base.done){
-			/* the resolution could not be done synchronously, return the context*/
+static belle_sip_simple_resolver_context_t * resolver_start_query(belle_sip_simple_resolver_context_t *ctx) {
+	int error;
+
+	/* Take a ref for this part of code because _resolver_start_query() can notify the results and free the ctx if this is not the case. */
+	belle_sip_object_ref(ctx);
+	error = _resolver_start_query(ctx);
+	if (error == 0) {
+		if (!ctx->base.notified) {
+			/* The resolution could not be done synchronously, return the context */
+			belle_sip_object_unref(ctx);
 			return ctx;
 		}
-		/*else resolution could be done synchronously*/
-	}else{
-		/*An error occured. We must notify the app.*/
-		notify_results(ctx);
+		/* Otherwise, resolution could be done synchronously */
+	} else {
+		/* An error occured. We must notify the app. */
+		belle_sip_resolver_context_notify(BELLE_SIP_RESOLVER_CONTEXT(ctx));
 	}
-	/*If it is failed or result could be found immediately, the context is now useless, we don't have to return it.*/
 	belle_sip_object_unref(ctx);
 	return NULL;
 }
@@ -712,6 +759,7 @@ static void belle_sip_simple_resolver_context_destroy(belle_sip_simple_resolver_
 	if (ctx->getaddrinfo_thread != 0) {
 		belle_sip_thread_join(ctx->getaddrinfo_thread, NULL);
 	}
+	if (ctx->getaddrinfo_source) belle_sip_object_unref(ctx->getaddrinfo_source);
 #ifdef _WIN32
 	if (ctx->ctlevent != (belle_sip_fd_t)-1)
 		CloseHandle(ctx->ctlevent);
@@ -772,38 +820,37 @@ static void belle_sip_dual_resolver_context_destroy(belle_sip_dual_resolver_cont
 	}
 }
 
-static void simple_resolver_cancel(belle_sip_resolver_context_t *obj){
-	belle_sip_main_loop_remove_source(obj->stack->ml,(belle_sip_source_t*)obj);
-	belle_sip_object_unref(obj);
+static void simple_resolver_context_cancel(belle_sip_resolver_context_t *obj) {
+	belle_sip_main_loop_remove_source(obj->stack->ml, (belle_sip_source_t *)obj);
 }
 
-static void combined_resolver_cancel(belle_sip_resolver_context_t *p){
-	belle_sip_combined_resolver_context_t *obj=(belle_sip_combined_resolver_context_t*)p;
-	
-	if (obj->srv_ctx){
-		belle_sip_resolver_context_cancel(obj->srv_ctx);
-		obj->srv_ctx=NULL;
+static void combined_resolver_context_cancel(belle_sip_resolver_context_t *obj) {
+	belle_sip_combined_resolver_context_t *ctx = BELLE_SIP_COMBINED_RESOLVER_CONTEXT(obj);
+	if (ctx->srv_ctx) {
+		belle_sip_resolver_context_cancel(ctx->srv_ctx);
+		belle_sip_object_unref(ctx->srv_ctx);
+		ctx->srv_ctx = NULL;
 	}
-	if (obj->a_fallback_ctx){
-		belle_sip_resolver_context_cancel(obj->a_fallback_ctx);
-		obj->a_fallback_ctx=NULL;
+	if (ctx->a_fallback_ctx) {
+		belle_sip_resolver_context_cancel(ctx->a_fallback_ctx);
+		belle_sip_object_unref(ctx->a_fallback_ctx);
+		ctx->a_fallback_ctx = NULL;
 	}
-	belle_sip_list_free_with_data(obj->srv_results,belle_sip_object_unref);
-	obj->srv_results=NULL;
-	belle_sip_object_unref(obj);
+	combined_resolver_context_cleanup(ctx);
 }
 
-static void dual_resolver_cancel(belle_sip_resolver_context_t *p){
-	belle_sip_dual_resolver_context_t *obj=(belle_sip_dual_resolver_context_t*)p;
-	if (obj->a_ctx){
-		belle_sip_resolver_context_cancel(obj->a_ctx);
-		obj->a_ctx=NULL;
+static void dual_resolver_context_cancel(belle_sip_resolver_context_t *obj) {
+	belle_sip_dual_resolver_context_t *ctx= BELLE_SIP_DUAL_RESOLVER_CONTEXT(obj);
+	if (ctx->a_ctx) {
+		belle_sip_resolver_context_cancel(ctx->a_ctx);
+		belle_sip_object_unref(ctx->a_ctx);
+		ctx->a_ctx = NULL;
 	}
-	if (obj->aaaa_ctx){
-		belle_sip_resolver_context_cancel(obj->aaaa_ctx);
-		obj->aaaa_ctx=NULL;
+	if (ctx->aaaa_ctx) {
+		belle_sip_resolver_context_cancel(ctx->aaaa_ctx);
+		belle_sip_object_unref(ctx->aaaa_ctx);
+		ctx->aaaa_ctx = NULL;
 	}
-	belle_sip_object_unref(p);
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_resolver_context_t);
@@ -812,7 +859,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_resolver_context_t)
 		BELLE_SIP_VPTR_INIT(belle_sip_resolver_context_t,belle_sip_source_t,TRUE),
 		(belle_sip_object_destroy_t) NULL,
 		NULL,
-		NULL
+		NULL,
+		BELLE_SIP_DEFAULT_BUFSIZE_HINT
 	},
 	NULL
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
@@ -825,9 +873,11 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_simple_resolver_context_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_simple_resolver_context_t,belle_sip_resolver_context_t,TRUE),
 			(belle_sip_object_destroy_t) belle_sip_simple_resolver_context_destroy,
 			NULL,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
-		simple_resolver_cancel
+		simple_resolver_context_cancel,
+		simple_resolver_context_notify
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
@@ -838,9 +888,11 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_dual_resolver_context_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_dual_resolver_context_t,belle_sip_resolver_context_t,TRUE),
 			(belle_sip_object_destroy_t) belle_sip_dual_resolver_context_destroy,
 			NULL,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
-		dual_resolver_cancel
+		dual_resolver_context_cancel,
+		dual_resolver_context_notify
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
@@ -851,9 +903,11 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_combined_resolver_context_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_combined_resolver_context_t,belle_sip_resolver_context_t,TRUE),
 			(belle_sip_object_destroy_t) belle_sip_combined_resolver_context_destroy,
 			NULL,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
-		combined_resolver_cancel
+		combined_resolver_context_cancel,
+		combined_resolver_context_notify
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
@@ -869,25 +923,18 @@ static char * srv_prefix_from_service_and_transport(const char *service, const c
 	return belle_sip_strdup_printf("_%s._udp.", service);
 }
 
-static void combined_notify_results(belle_sip_combined_resolver_context_t *obj){
-	obj->base.done=TRUE;
-	obj->cb(obj->cb_data,obj->name,obj->final_results);
-	obj->final_results=NULL;
-	/*clean everything (cancel() does it very well)*/
-	combined_resolver_cancel((belle_sip_resolver_context_t*)obj);
-}
-
-static void process_a_fallback_result(void *data, const char *name, struct addrinfo *ai_list){
+static void process_a_fallback_result(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl){
 	belle_sip_combined_resolver_context_t *ctx=(belle_sip_combined_resolver_context_t *)data;
 	ctx->final_results=ai_list;
-	combined_notify_results(ctx);
+	belle_sip_resolver_context_notify(BELLE_SIP_RESOLVER_CONTEXT(ctx));
 }
 
-static void combined_resolver_context_check_finished(belle_sip_combined_resolver_context_t *obj){
+static void combined_resolver_context_check_finished(belle_sip_combined_resolver_context_t *obj, uint32_t ttl){
 	belle_sip_list_t *elem;
 	struct addrinfo *final=NULL;
 	unsigned char finished=TRUE;
-	
+
+	if (ttl < BELLE_SIP_RESOLVER_CONTEXT(obj)->min_ttl) BELLE_SIP_RESOLVER_CONTEXT(obj)->min_ttl = ttl;
 	for(elem=obj->srv_results;elem!=NULL;elem=elem->next){
 		belle_sip_dns_srv_t *srv=(belle_sip_dns_srv_t*)elem->data;
 		if (!srv->a_done) {
@@ -905,23 +952,24 @@ static void combined_resolver_context_check_finished(belle_sip_combined_resolver
 		belle_sip_list_free_with_data(obj->srv_results,belle_sip_object_unref);
 		obj->srv_results=NULL;
 		obj->final_results=final;
-		combined_notify_results(obj);
+		belle_sip_resolver_context_notify(BELLE_SIP_RESOLVER_CONTEXT(obj));
 	}
 }
 
-static void process_a_from_srv(void *data, const char *name, struct addrinfo *ai_list){
+static void process_a_from_srv(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl){
 	belle_sip_dns_srv_t *srv=(belle_sip_dns_srv_t*)data;
 	srv->a_results=ai_list;
 	srv->a_done=TRUE;
 	belle_sip_message("A query finished for srv result [%s]",srv->target);
-	combined_resolver_context_check_finished(srv->root_resolver);
+	if (ttl < BELLE_SIP_RESOLVER_CONTEXT(srv->root_resolver)->min_ttl) BELLE_SIP_RESOLVER_CONTEXT(srv->root_resolver)->min_ttl = ttl;
+	combined_resolver_context_check_finished(srv->root_resolver, ttl);
 }
 
 static void srv_resolve_a(belle_sip_combined_resolver_context_t *obj, belle_sip_dns_srv_t *srv){
 	belle_sip_message("Starting A/AAAA query for srv result [%s]",srv->target);
 	srv->root_resolver=obj;
 	/* take a ref of the srv object because the A resolution may terminate synchronously and destroy the srv
-	 object before to store the returned value of belle_sip_stack_resole_a(). That would lead to an invalid write */
+	 object before to store the returned value of belle_sip_stack_resolve_a(). That would lead to an invalid write */
 	belle_sip_object_ref(srv);
 	srv->a_resolver=belle_sip_stack_resolve_a(obj->base.stack,srv->target,srv->port,obj->family,process_a_from_srv,srv);
 	if (srv->a_resolver){
@@ -930,10 +978,11 @@ static void srv_resolve_a(belle_sip_combined_resolver_context_t *obj, belle_sip_
 	belle_sip_object_unref(srv);
 }
 
-static void process_srv_results(void *data, const char *name, belle_sip_list_t *srv_results){
+static void process_srv_results(void *data, const char *name, belle_sip_list_t *srv_results, uint32_t ttl){
 	belle_sip_combined_resolver_context_t *ctx=(belle_sip_combined_resolver_context_t *)data;
 	/*take a ref here, because the A resolution might succeed synchronously and terminate the context before exiting this function*/
 	belle_sip_object_ref(ctx);
+	if (ttl < BELLE_SIP_RESOLVER_CONTEXT(data)->min_ttl) BELLE_SIP_RESOLVER_CONTEXT(data)->min_ttl = ttl;
 	if (srv_results){
 		belle_sip_list_t *elem;
 		/* take a ref of each srv_results because the last A resolution may terminate synchronously
@@ -968,13 +1017,14 @@ belle_sip_resolver_context_t * belle_sip_stack_resolve(belle_sip_stack_t *stack,
 		ctx->cb_data = data;
 		ctx->name = belle_sip_strdup(name);
 		ctx->port=port;
+		belle_sip_object_set_name((belle_sip_object_t*)ctx, ctx->name);
 		if (family == 0) family = AF_UNSPEC;
 		ctx->family = family;
-		/*take a ref for the entire duration of the DNS procedure, it will be released when it is finished*/
+		/* Take a ref for the entire duration of the DNS procedure, it will be released when it is finished */
 		belle_sip_object_ref(ctx);
 		ctx->srv_ctx=belle_sip_stack_resolve_srv(stack,service,transport,name,process_srv_results,ctx);
 		if (ctx->srv_ctx) belle_sip_object_ref(ctx->srv_ctx);
-		if (ctx->base.done){
+		if (ctx->base.notified) {
 			belle_sip_object_unref(ctx);
 			return NULL;
 		}
@@ -982,7 +1032,7 @@ belle_sip_resolver_context_t * belle_sip_stack_resolve(belle_sip_stack_t *stack,
 		return BELLE_SIP_RESOLVER_CONTEXT(ctx);
 	} else {
 		/* There is no resolve to be done */
-		cb(data, name, res);
+		cb(data, name, res, UINT32_MAX);
 		return NULL;
 	}
 }
@@ -991,12 +1041,14 @@ static belle_sip_resolver_context_t * belle_sip_stack_resolve_single(belle_sip_s
 	/* Then perform asynchronous DNS A or AAAA query */
 	belle_sip_simple_resolver_context_t *ctx = belle_sip_object_new(belle_sip_simple_resolver_context_t);
 	belle_sip_resolver_context_init((belle_sip_resolver_context_t*)ctx,stack);
-	
 	ctx->cb_data = data;
 	ctx->cb = cb;
 	ctx->name = belle_sip_strdup(name);
 	ctx->port = port;
 	ctx->flags = flags;
+	belle_sip_object_set_name((belle_sip_object_t*)ctx, ctx->name);
+	/* Take a ref for the entire duration of the DNS procedure, it will be released when it is finished */
+	belle_sip_object_ref(ctx);
 	if (family == 0) family = AF_UNSPEC;
 	ctx->family = family;
 	ctx->type = (ctx->family == AF_INET6) ? DNS_T_AAAA : DNS_T_A;
@@ -1006,30 +1058,30 @@ static belle_sip_resolver_context_t * belle_sip_stack_resolve_single(belle_sip_s
 	return (belle_sip_resolver_context_t*)resolver_start_query(ctx);
 }
 
-static void notify_dual_results(belle_sip_dual_resolver_context_t *ctx){
-	if (ctx->a_done && ctx->aaaa_done){
-		struct addrinfo *results=ctx->aaaa_results;
-		results=ai_list_append(results,ctx->a_results);
-		ctx->a_results=NULL;
-		ctx->aaaa_results=NULL;
-		ctx->base.done=TRUE;
-		ctx->cb(ctx->cb_data,ctx->name,results);
-		belle_sip_object_unref(ctx);
+static uint8_t belle_sip_resolver_context_can_be_cancelled(belle_sip_resolver_context_t *obj) {
+	return ((obj->cancelled == TRUE) || (obj->notified == TRUE)) ? FALSE : TRUE;
+}
+
+#define belle_sip_resolver_context_can_be_notified(obj) belle_sip_resolver_context_can_be_cancelled(obj)
+
+static void dual_resolver_context_check_finished(belle_sip_dual_resolver_context_t *ctx) {
+	if (belle_sip_resolver_context_can_be_notified(BELLE_SIP_RESOLVER_CONTEXT(ctx)) && (ctx->a_notified == TRUE) && (ctx->aaaa_notified == TRUE)) {
+		belle_sip_resolver_context_notify(BELLE_SIP_RESOLVER_CONTEXT(ctx));
 	}
 }
 
-static void on_ipv4_results(void *data, const char *name, struct addrinfo *ai_list){
-	belle_sip_dual_resolver_context_t *ctx=(belle_sip_dual_resolver_context_t *)data;
-	ctx->a_done=TRUE;
-	ctx->a_results=ai_list;
-	notify_dual_results(ctx);
+static void on_ipv4_results(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl) {
+	belle_sip_dual_resolver_context_t *ctx = BELLE_SIP_DUAL_RESOLVER_CONTEXT(data);
+	ctx->a_results = ai_list;
+	ctx->a_notified = TRUE;
+	dual_resolver_context_check_finished(ctx);
 }
 
-static void on_ipv6_results(void *data, const char *name, struct addrinfo *ai_list){
-	belle_sip_dual_resolver_context_t *ctx=(belle_sip_dual_resolver_context_t *)data;
-	ctx->aaaa_done=TRUE;
-	ctx->aaaa_results=ai_list;
-	notify_dual_results(ctx);
+static void on_ipv6_results(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl) {
+	belle_sip_dual_resolver_context_t *ctx = BELLE_SIP_DUAL_RESOLVER_CONTEXT(data);
+	ctx->aaaa_results = ai_list;
+	ctx->aaaa_notified = TRUE;
+	dual_resolver_context_check_finished(ctx);
 }
 
 static belle_sip_resolver_context_t * belle_sip_stack_resolve_dual(belle_sip_stack_t *stack, const char *name, int port, belle_sip_resolver_callback_t cb , void *data){
@@ -1040,17 +1092,18 @@ static belle_sip_resolver_context_t * belle_sip_stack_resolve_dual(belle_sip_sta
 	ctx->cb_data = data;
 	ctx->cb = cb;
 	ctx->name = belle_sip_strdup(name);
-	/*take a ref for the entire duration of the DNS procedure, it will be released when it is finished*/
+	belle_sip_object_set_name((belle_sip_object_t*)ctx, ctx->name);
+	/* Take a ref for the entire duration of the DNS procedure, it will be released when it is finished */
 	belle_sip_object_ref(ctx);
 	ctx->a_ctx=belle_sip_stack_resolve_single(stack,name,port,AF_INET, AI_V4MAPPED, on_ipv4_results,ctx);
 	if (ctx->a_ctx) belle_sip_object_ref(ctx->a_ctx);
 	ctx->aaaa_ctx=belle_sip_stack_resolve_single(stack, name, port, AF_INET6, 0, on_ipv6_results, ctx);
 	if (ctx->aaaa_ctx) belle_sip_object_ref(ctx->aaaa_ctx);
-	if (ctx->base.done){
-		/*all results were found synchronously*/
+	if (ctx->base.notified){
+		/* All results were found synchronously */
 		belle_sip_object_unref(ctx);
-		ctx=NULL;
-	}else belle_sip_object_unref(ctx);
+		ctx = NULL;
+	} else belle_sip_object_unref(ctx);
 	return BELLE_SIP_RESOLVER_CONTEXT(ctx);
 }
 
@@ -1060,6 +1113,7 @@ belle_sip_resolver_context_t * belle_sip_stack_resolve_a(belle_sip_stack_t *stac
 		switch(family){
 			case AF_UNSPEC:
 				family=AF_INET6;
+				BCTBX_NO_BREAK; /*intentionally no break*/
 			case AF_INET6:
 				return belle_sip_stack_resolve_dual(stack,name,port,cb,data);
 				break;
@@ -1071,7 +1125,7 @@ belle_sip_resolver_context_t * belle_sip_stack_resolve_a(belle_sip_stack_t *stac
 		}
 	} else {
 		/* There is no resolve to be done */
-		cb(data, name, res);
+		cb(data, name, res, UINT32_MAX);
 	}
 	return NULL;
 }
@@ -1084,45 +1138,62 @@ belle_sip_resolver_context_t * belle_sip_stack_resolve_srv(belle_sip_stack_t *st
 	ctx->srv_cb = cb;
 	ctx->name = belle_sip_concat(srv_prefix, name, NULL);
 	ctx->type = DNS_T_SRV;
+	belle_sip_object_set_name((belle_sip_object_t*)ctx, ctx->name);
+	/* Take a ref for the entire duration of the DNS procedure, it will be released when it is finished */
+	belle_sip_object_ref(ctx);
 	belle_sip_free(srv_prefix);
 	return (belle_sip_resolver_context_t*)resolver_start_query(ctx);
 }
 
-void belle_sip_resolver_context_cancel(belle_sip_resolver_context_t *obj){
-	BELLE_SIP_OBJECT_VPTR(obj,belle_sip_resolver_context_t)->cancel(obj);
+void belle_sip_resolver_context_cancel(belle_sip_resolver_context_t *obj) {
+	if (belle_sip_resolver_context_can_be_cancelled(obj)) {
+		obj->cancelled = TRUE;
+		BELLE_SIP_OBJECT_VPTR(obj, belle_sip_resolver_context_t)->cancel(obj);
+		belle_sip_object_unref(obj);
+	}
+}
+
+void belle_sip_resolver_context_notify(belle_sip_resolver_context_t *obj) {
+	if (belle_sip_resolver_context_can_be_notified(obj)) {
+		obj->notified = TRUE;
+		BELLE_SIP_OBJECT_VPTR(obj, belle_sip_resolver_context_t)->notify(obj);
+		belle_sip_object_unref(obj);
+	}
 }
 
 /*
 This function does the connect() method to get local ip address suitable to reach a given destination.
 It works on all platform except for windows using ipv6 sockets. TODO: find a workaround for win32+ipv6 socket
 */
-void belle_sip_get_src_addr_for(const struct sockaddr *dest, socklen_t destlen, struct sockaddr *src, socklen_t *srclen, int local_port){
+int belle_sip_get_src_addr_for(const struct sockaddr *dest, socklen_t destlen, struct sockaddr *src, socklen_t *srclen, int local_port){
 	int af_type=dest->sa_family;
-	int sock=(int)socket(af_type,SOCK_DGRAM,IPPROTO_UDP);
-	
-	belle_sip_message("belle_sip_get_src_addr_for(): af_inet6=%i",af_type==AF_INET6);
+	int sock=(int)bctbx_socket(af_type,SOCK_DGRAM,IPPROTO_UDP);
+	int ret = 0;
+
 	if (sock==(belle_sip_socket_t)-1){
 		if (af_type == AF_INET){
 			belle_sip_fatal("Could not create socket: %s",belle_sip_get_socket_error_string());
 		}
 		goto fail;
 	}
-	
+
 	if (af_type==AF_INET6 && (IN6_IS_ADDR_V4MAPPED(&((struct sockaddr_in6*)dest)->sin6_addr))){
-		/*this is actually required only for windows, who is enable to provide an ipv4 mapped local address if the remote is ipv4 mapped,
+		/*this is actually required only for windows, who is unable to provide an ipv4 mapped local address if the remote is ipv4 mapped,
 		and unable to provide a correct local address if the remote address is true ipv6 address when in dual stack mode*/
 		belle_sip_socket_enable_dual_stack(sock);
 	}
-	
-	if (connect(sock,dest,destlen)==-1){
-		belle_sip_error("belle_sip_get_src_addr_for: connect() failed: %s",belle_sip_get_socket_error_string());
+	if (bctbx_connect(sock,dest,destlen)==-1){
+	//if (connect(sock,dest,destlen)==-1){
+		ret = -get_socket_error();
+		belle_sip_error("belle_sip_get_src_addr_for: bctbx_connect() failed: %s",belle_sip_get_socket_error_string_from_code(-ret));
 		goto fail;
 	}
-	if (getsockname(sock,src,srclen)==-1){
-		belle_sip_error("belle_sip_get_src_addr_for: getsockname() failed: %s",belle_sip_get_socket_error_string());
+	if (bctbx_getsockname(sock,src,srclen)==-1){
+		ret = -get_socket_error();
+		belle_sip_error("belle_sip_get_src_addr_for: bctbx_getsockname() failed: %s",belle_sip_get_socket_error_string_from_code(-ret));
 		goto fail;
 	}
-	
+
 	if (af_type==AF_INET6){
 		struct sockaddr_in6 *sin6=(struct sockaddr_in6*)src;
 		sin6->sin6_port=htons(local_port);
@@ -1130,9 +1201,9 @@ void belle_sip_get_src_addr_for(const struct sockaddr *dest, socklen_t destlen,
 		struct sockaddr_in *sin=(struct sockaddr_in*)src;
 		sin->sin_port=htons(local_port);
 	}
-	
-	close_socket(sock);
-	return;
+
+	belle_sip_close_socket(sock);
+	return ret;
 fail:
 	{
 		struct addrinfo *res = bctbx_ip_address_to_addrinfo(af_type, SOCK_STREAM, af_type == AF_INET ? "127.0.0.1" : "::1", local_port);
@@ -1144,5 +1215,6 @@ fail:
 			if (af_type == AF_INET) belle_sip_fatal("belle_sip_get_src_addr_for(): belle_sip_ip_address_to_addrinfo() failed");
 		}
 	}
-	if (sock!=(belle_sip_socket_t)-1) close_socket(sock);
+	if (sock!=(belle_sip_socket_t)-1) belle_sip_close_socket(sock);
+	return ret;
 }
