@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <sys/stat.h>
 #include "belle_sip_internal.h"
+#include "bctoolbox/parser.h"
 
 #include "clock_gettime.h" /*for apple*/
 
@@ -89,7 +90,7 @@ static int find_best_clock_id (void) {
 #if 0
 	struct timespec ts;
 	static int clock_id=-1;
-#ifndef ANDROID
+#ifndef __ANDROID__
 #define DEFAULT_CLOCK_MODE CLOCK_MONOTONIC
 #else
 #define DEFAULT_CLOCK_MODE CLOCK_REALTIME /*monotonic clock stop during sleep mode*/
@@ -112,10 +113,20 @@ static int find_best_clock_id (void) {
 	 * The only reason for seeing the register is if the network address has changed, in which case it will trigger a shutdown of all sockets.
 	 * As a result, we fallback to CLOCK_REALTIME until the OS correctly implement CLOCK_MONOTONIC according to POSIX specifications
 	 */
+#ifdef __APPLE__
+	#ifdef CLOCK_REALTIME
+		#undef CLOCK_REALTIME
+	#endif
+	#define CLOCK_REALTIME BC_CLOCK_REALTIME
+#endif
 	return CLOCK_REALTIME;
 #endif
 }
 uint64_t belle_sip_time_ms(void){
+#ifdef __APPLE__
+#define clock_gettime bc_clock_gettime
+#endif
+
 	struct timespec ts;
 	if (clock_gettime(find_best_clock_id(),&ts)==-1){
 		belle_sip_error("clock_gettime() error for clock_id=%i: %s",find_best_clock_id(),strerror(errno));
@@ -248,6 +259,12 @@ unsigned int belle_sip_random(void){
 #endif
 }
 
+
+void belle_sip_set_socket_api(bctbx_vsocket_api_t* my_api){
+
+	bctbx_vsocket_api_set_default(my_api);
+}
+
 static const char *symbols="aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ0123456789-~";
 
 /**
@@ -355,48 +372,14 @@ void belle_sip_util_copy_headers(belle_sip_message_t *orig, belle_sip_message_t
 	}
 }
 
-size_t belle_sip_get_char (const char*a,size_t n,char*out) {
-	if (*a=='%' && n>2) {
-		unsigned int tmp;
-		sscanf(a+1,"%02x",&tmp);
-		*out=(char)tmp;
-		return 3;
-	} else {
-		*out=*a;
-		return 1;
-	}
-}
 
 char* belle_sip_to_unescaped_string(const char* buff) {
-	char *output_buff=belle_sip_malloc(strlen(buff)+1);
-	size_t i;
-	size_t out_buff_index=0;
-
-	for(i=0; buff[i]!='\0'; out_buff_index++) {
-		i+=belle_sip_get_char(buff+i,3,output_buff+out_buff_index);
-	}
-	output_buff[out_buff_index]='\0';
-	return output_buff;
+	return bctbx_unescaped_string(buff);
 }
 
-#define BELLE_SIP_NO_ESCAPES_SIZE 257
-static void noescapes_add_list(char noescapes[BELLE_SIP_NO_ESCAPES_SIZE], const char *allowed) {
-	while (*allowed) {
-		noescapes[(unsigned int) *allowed] = 1;
-		++allowed;
-	}
+size_t belle_sip_get_char(const char* a, char *b) {
+	return bctbx_get_char(a,b);
 }
-
-static void noescapes_add_range(char noescapes[BELLE_SIP_NO_ESCAPES_SIZE], char first, char last) {
-	memset(noescapes + (unsigned int)first, 1, last-first+1);
-}
-
-static void noescapes_add_alfanums(char noescapes[BELLE_SIP_NO_ESCAPES_SIZE]) {
-	noescapes_add_range(noescapes, '0', '9');
-	noescapes_add_range(noescapes, 'A', 'Z');
-	noescapes_add_range(noescapes, 'a', 'z');
-}
-
 /*
 static void print_noescapes_map(char noescapes[BELLE_SIP_NO_ESCAPES_SIZE], const char *name) {
 	unsigned int i;
@@ -409,9 +392,9 @@ static void print_noescapes_map(char noescapes[BELLE_SIP_NO_ESCAPES_SIZE], const
 }
 */
 
-static const char *get_sip_uri_username_noescapes(void) {
-	static char noescapes[BELLE_SIP_NO_ESCAPES_SIZE] = {0};
-	if (noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] == 0) {
+static const bctbx_noescape_rules_t* get_sip_uri_username_noescapes(void) {
+	static bctbx_noescape_rules_t noescapes = {0};
+	if (noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] == 0) {
 		// concurrent initialization should not be an issue
 		/*user             =  1*( unreserved / escaped / user-unreserved )
 		 unreserved  =  alphanum / mark
@@ -419,39 +402,39 @@ static const char *get_sip_uri_username_noescapes(void) {
 		 / "(" / ")"
 		user-unreserved  =  "&" / "=" / "+" / "$" / "," / ";" / "?" / "/"
 		*/
-		noescapes_add_alfanums(noescapes);
+		bctbx_noescape_rules_add_alfanums(noescapes);
 		/*mark*/
-		noescapes_add_list(noescapes, "-_.!~*'()");
+		bctbx_noescape_rules_add_list(noescapes, "-_.!~*'()");
 		/*user-unreserved*/
-		noescapes_add_list(noescapes, "&=+$,;?/");
+		bctbx_noescape_rules_add_list(noescapes, "&=+$,;?/");
 
-		noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] = 1; // initialized
+		noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] = 1; // initialized
 //		print_noescapes_map(noescapes, "uri_username");
 	}
-	return noescapes;
+	return (const bctbx_noescape_rules_t*)&noescapes;/*gcc asks for a cast, clang not*/
 }
 /*
  *
  * password         =  *( unreserved / escaped /
                     "&" / "=" / "+" / "$" / "," )
  * */
-static const char *get_sip_uri_userpasswd_noescapes(void) {
-	static char noescapes[BELLE_SIP_NO_ESCAPES_SIZE] = {0};
-	if (noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] == 0) {
+static const bctbx_noescape_rules_t* get_sip_uri_userpasswd_noescapes(void) {
+	static bctbx_noescape_rules_t noescapes = {0};
+	if (noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] == 0) {
 		// unreserved
-		noescapes_add_alfanums(noescapes);
-		noescapes_add_list(noescapes, "-_.!~*'()");
-		noescapes_add_list(noescapes, "&=+$,");
+		bctbx_noescape_rules_add_alfanums(noescapes);
+		bctbx_noescape_rules_add_list(noescapes, "-_.!~*'()");
+		bctbx_noescape_rules_add_list(noescapes, "&=+$,");
 
-		noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] = 1; // initialized
+		noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] = 1; // initialized
 
 	}
-	return noescapes;
+	return (const bctbx_noescape_rules_t*)&noescapes;/*gcc asks for a cast, clang not*/
 }
 
-static const char *get_sip_uri_parameter_noescapes(void) {
-	static char noescapes[BELLE_SIP_NO_ESCAPES_SIZE] = {0};
-	if (noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] == 0) {
+static  const bctbx_noescape_rules_t* get_sip_uri_parameter_noescapes(void) {
+	static bctbx_noescape_rules_t  noescapes= {0};
+	if (noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] == 0) {
 		/*
 		 other-param       =  pname [ "=" pvalue ]
 		 pname             =  1*paramchar
@@ -467,23 +450,23 @@ static const char *get_sip_uri_parameter_noescapes(void) {
 		 */
 		//param-unreserved  =
 
-		noescapes_add_list(noescapes,"[]/:&+$");
+		bctbx_noescape_rules_add_list(noescapes,"[]/:&+$");
 
 		// token
-		noescapes_add_alfanums(noescapes);
-		noescapes_add_list(noescapes, "-.!%*_+`'~");
+		bctbx_noescape_rules_add_alfanums(noescapes);
+		bctbx_noescape_rules_add_list(noescapes, "-.!%*_+`'~");
 
 		// unreserved
-		noescapes_add_list(noescapes, "-_.!~*'()");
+		bctbx_noescape_rules_add_list(noescapes, "-_.!~*'()");
 
-		noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] = 1; // initialized
+		noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] = 1; // initialized
 //		print_noescapes_map(noescapes, "uri_parameter");
 	}
-	return noescapes;
+	return (const bctbx_noescape_rules_t*)&noescapes;/*gcc asks for a cast, clang not*/
 }
-static const char *get_sip_uri_header_noescapes(void) {
-	static char noescapes[BELLE_SIP_NO_ESCAPES_SIZE] = {0};
-	if (noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] == 0) {
+static const bctbx_noescape_rules_t* get_sip_uri_header_noescapes(void) {
+	static bctbx_noescape_rules_t  noescapes= {0};
+	if (noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] == 0) {
 		/*
 		 unreserved  =  alphanum / mark
 		 mark        =  "-" / "_" / "." / "!" / "~" / "*" / "'"
@@ -500,61 +483,37 @@ static const char *get_sip_uri_header_noescapes(void) {
 
 		// unreserved
 		//alphanum
-		noescapes_add_alfanums(noescapes);
+		bctbx_noescape_rules_add_alfanums(noescapes);
 		//mark
-		noescapes_add_list(noescapes, "-_.!~*'()");
+		bctbx_noescape_rules_add_list(noescapes, "-_.!~*'()");
 
-		noescapes_add_list(noescapes, "[]/?:+$");
+		bctbx_noescape_rules_add_list(noescapes, "[]/?:+$");
 		//hnv-unreserved
-		noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] = 1; // initialized
+		noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] = 1; // initialized
 //		print_noescapes_map(noescapes, "uri_parameter");
 	}
-	return noescapes;
+	return (const bctbx_noescape_rules_t*)&noescapes;/*gcc asks for a cast, clang not*/
 }
 
-static char* belle_sip_escape(const char* buff, const char *noescapes) {
-	size_t outbuf_size=strlen(buff);
-	size_t orig_size=outbuf_size;
-	char *output_buff=(char*)belle_sip_malloc(outbuf_size + 1);
-	int i;
-	size_t out_buff_index=0;
-
-	for(i=0; buff[i] != '\0'; i++) {
-		int c = ((unsigned char*)buff)[i];
-		if (outbuf_size < out_buff_index + 3){
-			// we will possibly add 3 chars
-			outbuf_size += MAX(orig_size/2,3);
-			output_buff = belle_sip_realloc(output_buff, outbuf_size + 1);
-		}
-		if (noescapes[c] == 1) {
-			output_buff[out_buff_index++]=c;
-		} else {
-			// this will write 3 characters
-			out_buff_index+=snprintf(output_buff+out_buff_index, outbuf_size +1 - out_buff_index, "%%%02x", c);
-		}
-	}
-	output_buff[out_buff_index]='\0';
-	return output_buff;
-}
 
 char* belle_sip_uri_to_escaped_username(const char* buff) {
-	return belle_sip_escape(buff, get_sip_uri_username_noescapes());
+	return bctbx_escape(buff, *get_sip_uri_username_noescapes());
 }
 char* belle_sip_uri_to_escaped_userpasswd(const char* buff) {
-	return belle_sip_escape(buff, get_sip_uri_userpasswd_noescapes());
+	return bctbx_escape(buff, *get_sip_uri_userpasswd_noescapes());
 }
 char* belle_sip_uri_to_escaped_parameter(const char* buff) {
-	return belle_sip_escape(buff, get_sip_uri_parameter_noescapes());
+	return bctbx_escape(buff, *get_sip_uri_parameter_noescapes());
 }
 char* belle_sip_uri_to_escaped_header(const char* buff) {
-	return belle_sip_escape(buff, get_sip_uri_header_noescapes());
+	return bctbx_escape(buff, *get_sip_uri_header_noescapes());
 }
 
 
 /*uri (I.E RFC 2396)*/
-static const char *get_generic_uri_query_noescapes(void) {
-	static char noescapes[BELLE_SIP_NO_ESCAPES_SIZE] = {0};
-	if (noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] == 0) {
+static const bctbx_noescape_rules_t *get_generic_uri_query_noescapes(void) {
+	static bctbx_noescape_rules_t  noescapes= {0};
+	if (noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] == 0) {
 		/*
 	    uric          = reserved | unreserved | escaped
 		reserved      = ";" | "/" | "?" | ":" | "@" | "&" | "=" | "+" |
@@ -570,18 +529,18 @@ static const char *get_generic_uri_query_noescapes(void) {
 
 		*/
 		/*unreserved*/
-		noescapes_add_alfanums(noescapes);
+		bctbx_noescape_rules_add_alfanums(noescapes);
 		/*mark*/
-		noescapes_add_list(noescapes, "-_.!~*'()");
-		noescapes_add_list(noescapes, "=&"); // otherwise how to pass parameters?
-		noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] = 1; // initialized
+		bctbx_noescape_rules_add_list(noescapes, "-_.!~*'()");
+		bctbx_noescape_rules_add_list(noescapes, "=&"); // otherwise how to pass parameters?
+		noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] = 1; // initialized
 	}
-	return noescapes;
+	return (const bctbx_noescape_rules_t*)&noescapes;/*gcc asks for a cast, clang not*/
 }
 
-static const char *get_generic_uri_path_noescapes(void) {
-	static char noescapes[BELLE_SIP_NO_ESCAPES_SIZE] = {0};
-	if (noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] == 0) {
+static const bctbx_noescape_rules_t *get_generic_uri_path_noescapes(void) {
+	static bctbx_noescape_rules_t  noescapes= {0};
+	if (noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] == 0) {
 		/*
 	    3.3. Path Component
 
@@ -607,25 +566,25 @@ static const char *get_generic_uri_path_noescapes(void) {
 
 		*/
 		/*unreserved*/
-		noescapes_add_alfanums(noescapes);
+		bctbx_noescape_rules_add_alfanums(noescapes);
 		/*mark*/
-		noescapes_add_list(noescapes, "-_.!~*'()");
+		bctbx_noescape_rules_add_list(noescapes, "-_.!~*'()");
 		/*pchar*/
-		noescapes_add_list(noescapes, ":@&=+$,");
+		bctbx_noescape_rules_add_list(noescapes, ":@&=+$,");
 		/*;*/
-		noescapes_add_list(noescapes, ";");
-		noescapes_add_list(noescapes, "/");
+		bctbx_noescape_rules_add_list(noescapes, ";");
+		bctbx_noescape_rules_add_list(noescapes, "/");
 
-		noescapes[BELLE_SIP_NO_ESCAPES_SIZE-1] = 1; // initialized
+		noescapes[BCTBX_NOESCAPE_RULES_USER_INDEX] = 1; // initialized
 	}
-	return noescapes;
+	return (const bctbx_noescape_rules_t*)&noescapes;/*gcc asks for a cast, clang not*/
 }
 
 char* belle_generic_uri_to_escaped_query(const char* buff) {
-	return belle_sip_escape(buff, get_generic_uri_query_noescapes());
+	return bctbx_escape(buff, *get_generic_uri_query_noescapes());
 }
 char* belle_generic_uri_to_escaped_path(const char* buff) {
-	return belle_sip_escape(buff, get_generic_uri_path_noescapes());
+	return bctbx_escape(buff, *get_generic_uri_path_noescapes());
 }
 
 char* belle_sip_string_to_backslash_less_unescaped_string(const char* buff) {
@@ -710,35 +669,27 @@ belle_sip_list_t *belle_sip_parse_directory(const char *path, const char *file_t
 #else
 	DIR *dir;
 	struct dirent *ent;
-	struct dirent *result;
-	long int name_max;
-	int res;
 
 	if ((dir = opendir(path)) == NULL) {
 		belle_sip_error("Could't open [%s] directory.", path);
 		return NULL;
 	}
 
-	/* allocate the directory entry structure */
-	name_max = pathconf(path, _PC_NAME_MAX);
-	if (name_max == -1) name_max = 255;
-	ent = malloc(offsetof(struct dirent, d_name) + name_max + 1);
-
 	/* loop on all directory files */
-	res = readdir_r(dir, ent, &result);
-	while ((res == 0) && (result != NULL)) {
+	errno = 0;
+	ent = readdir(dir);
+	while (ent != NULL) {
 		/* filter on file type if given */
 		if (file_type==NULL
 			|| (strncmp(ent->d_name+strlen(ent->d_name)-strlen(file_type), file_type, strlen(file_type))==0) ) {
 			char *name_with_path=belle_sip_strdup_printf("%s/%s",path,ent->d_name);
 			file_list = belle_sip_list_append(file_list, name_with_path);
 		}
-		res = readdir_r(dir, ent, &result);
+		ent = readdir(dir);
 	}
-	if (res != 0) {
+	if (errno != 0) {
 		belle_sip_error("Error while reading the [%s] directory: %s.", path, strerror(errno));
 	}
-	free(ent);
 	closedir(dir);
 #endif
 	return file_list;
