@@ -17,6 +17,8 @@
 */
 
 #include "belle_sip_internal.h"
+#include "bctoolbox/vfs.h"
+
 #ifdef HAVE_ZLIB
 #include "zlib.h"
 #endif
@@ -54,7 +56,7 @@ static void belle_sip_body_handler_destroy(belle_sip_body_handler_t *obj){
 static belle_sip_error_code belle_sip_body_handler_marshal(belle_sip_body_handler_t *obj, char *buff, size_t buff_size, size_t *offset) {
 	int ret;
 	size_t len;
-	if (*offset == 0) belle_sip_body_handler_begin_transfer(obj);
+	if (*offset == 0) belle_sip_body_handler_begin_send_transfer(obj);
 	do {
 		len = buff_size - *offset;
 		ret = belle_sip_body_handler_send_chunk(obj, NULL, (uint8_t*)buff + *offset, &len);
@@ -73,8 +75,10 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_body_handler_t)
 		(belle_sip_object_destroy_t) belle_sip_body_handler_destroy,
 		(belle_sip_object_clone_t) belle_sip_body_handler_clone,
 		(belle_sip_object_marshal_t) belle_sip_body_handler_marshal,
+		BELLE_SIP_DEFAULT_BUFSIZE_HINT
 	},
-	NULL, /* begin_transfer */
+	NULL, /* begin_recv_transfer */
+	NULL, /* begin_send_transfer */
 	NULL, /* end_transfer */
 	NULL, /*chunk_recv*/
 	NULL /*chunk_send*/
@@ -122,16 +126,24 @@ static void update_progress(belle_sip_body_handler_t *obj, belle_sip_message_t *
 		obj->progress_cb(obj,msg,obj->user_data,obj->transfered_size,obj->expected_size);
 }
 
-void belle_sip_body_handler_begin_transfer(belle_sip_body_handler_t *obj){
+void belle_sip_body_handler_begin_recv_transfer(belle_sip_body_handler_t *obj){
 	BELLE_SIP_OBJECT_VPTR_TYPE(belle_sip_body_handler_t) *vptr = BELLE_SIP_OBJECT_VPTR(obj, belle_sip_body_handler_t);
-	if (vptr->begin_transfer != NULL) {
-		vptr->begin_transfer(obj);
+	if (vptr->begin_recv_transfer != NULL) {
+		vptr->begin_recv_transfer(obj);
 	}
 	obj->transfered_size=0;
 }
 
-void belle_sip_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, const uint8_t *buf, size_t size){
-	BELLE_SIP_OBJECT_VPTR(obj,belle_sip_body_handler_t)->chunk_recv(obj,msg,obj->transfered_size,buf,size);
+void belle_sip_body_handler_begin_send_transfer(belle_sip_body_handler_t *obj) {
+	BELLE_SIP_OBJECT_VPTR_TYPE(belle_sip_body_handler_t) *vptr = BELLE_SIP_OBJECT_VPTR(obj, belle_sip_body_handler_t);
+	if (vptr->begin_send_transfer != NULL) {
+		vptr->begin_send_transfer(obj);
+	}
+	obj->transfered_size = 0;
+}
+
+void belle_sip_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, uint8_t *buf, size_t size){
+	BELLE_SIP_OBJECT_VPTR(obj,belle_sip_body_handler_t)->chunk_recv(obj,msg,(off_t)obj->transfered_size,buf,size);
 	obj->transfered_size+=size;
 	update_progress(obj,msg);
 }
@@ -142,13 +154,21 @@ int belle_sip_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_m
 	if (obj->expected_size!=0){
 		to_send=MIN(*size,obj->expected_size-obj->transfered_size);
 	}
-	ret=BELLE_SIP_OBJECT_VPTR(obj,belle_sip_body_handler_t)->chunk_send(obj,msg,obj->transfered_size,buf,&to_send);
+	if (to_send==0 && obj->transfered_size==obj->expected_size) {
+		// An eWouldBlock error added a call to the function, nothing to send so return
+		// In some case to_send=0 because not buffer is available but sendings not finished.
+		belle_sip_message("body handler [%p] : Nothing to send",obj);
+		*size=0;
+		return BELLE_SIP_STOP;
+	}
+	ret=BELLE_SIP_OBJECT_VPTR(obj,belle_sip_body_handler_t)->chunk_send(obj,msg,(off_t)obj->transfered_size,buf,&to_send);
 	obj->transfered_size+=to_send;
 	*size=to_send;
 	update_progress(obj,msg);
 	if (obj->expected_size!=0){
-		if (obj->transfered_size==obj->expected_size)
+		if (obj->transfered_size==obj->expected_size) {
 			return BELLE_SIP_STOP;
+		}
 		if (ret==BELLE_SIP_STOP && obj->transfered_size<obj->expected_size){
 			belle_sip_error("body handler [%p] transfered only [%i] bytes while [%i] were expected",obj,
 					(int)obj->transfered_size,(int)obj->expected_size);
@@ -189,14 +209,14 @@ static void belle_sip_memory_body_handler_clone(belle_sip_memory_body_handler_t
 	obj->encoding_applied = orig->encoding_applied;
 }
 
-static void belle_sip_memory_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, const uint8_t *buf, size_t size){
+static void belle_sip_memory_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t size){
 	belle_sip_memory_body_handler_t *obj=(belle_sip_memory_body_handler_t*)base;
 	obj->buffer=belle_sip_realloc(obj->buffer,offset+size+1);
 	memcpy(obj->buffer+offset,buf,size);
 	obj->buffer[offset+size]='\0';
 }
 
-static int belle_sip_memory_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, uint8_t *buf, size_t *size){
+static int belle_sip_memory_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t *size){
 	belle_sip_memory_body_handler_t *obj=(belle_sip_memory_body_handler_t*)base;
 	size_t to_send=MIN(*size,obj->base.expected_size-offset);
 	if (obj->buffer == NULL) return BELLE_SIP_STOP;
@@ -212,10 +232,12 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_memory_body_handler_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_memory_body_handler_t,belle_sip_body_handler_t,TRUE),
 			(belle_sip_object_destroy_t) belle_sip_memory_body_handler_destroy,
 			(belle_sip_object_clone_t)belle_sip_memory_body_handler_clone,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
 		NULL,
 		NULL,
+		NULL,
 		belle_sip_memory_body_handler_recv_chunk,
 		belle_sip_memory_body_handler_send_chunk
 	}
@@ -292,6 +314,7 @@ int belle_sip_memory_body_handler_unapply_encoding(belle_sip_memory_body_handler
 		unsigned int outbuf_size = avail_out;
 		unsigned char *outbuf = belle_sip_malloc(outbuf_size);
 		unsigned char *outbuf_ptr = outbuf;
+		bool_t outbuf_too_small = FALSE;
 		int ret;
 
 		strm.zalloc = Z_NULL;
@@ -304,20 +327,38 @@ int belle_sip_memory_body_handler_unapply_encoding(belle_sip_memory_body_handler
 		strm.avail_in = (uInt)initial_size;
 		strm.next_in = obj->buffer;
 		do {
-			if (avail_out < BELLE_SIP_MEMORY_BODY_HANDLER_ZLIB_INITIAL_SIZE) {
+			if ((avail_out < BELLE_SIP_MEMORY_BODY_HANDLER_ZLIB_INITIAL_SIZE) || (outbuf_too_small == TRUE)) {
 				unsigned int cursize = (unsigned int)(outbuf_ptr - outbuf);
+				unsigned int increase = outbuf_size;
 				outbuf_size *= 2;
 				outbuf = belle_sip_realloc(outbuf, outbuf_size);
 				outbuf_ptr = outbuf + cursize;
+				avail_out += increase;
 			}
+			outbuf_too_small = FALSE;
 			strm.avail_out = avail_out;
 			strm.next_out = outbuf_ptr;
 			ret = inflate(&strm, Z_NO_FLUSH);
 			switch (ret) {
+				case Z_OK:
+					/* Error handling for truncated input buffer. Z_STREAM_END is not returned but there is no further input content */
+					if (strm.avail_out > 0) {
+						belle_sip_error("%s: truncated compressed body. Cannot uncompress it...", __FUNCTION__);
+						return -1;
+					}
+					break;
+				case Z_STREAM_END:
+					// Everything is ok, continue
+					break;
+				case Z_BUF_ERROR:
+					// Ask for more output space
+					outbuf_too_small = TRUE;
+					break;
 				case Z_NEED_DICT:
-					ret = Z_DATA_ERROR;
 				case Z_DATA_ERROR:
+				case Z_STREAM_ERROR:
 				case Z_MEM_ERROR:
+				default:
 					inflateEnd(&strm);
 					belle_sip_free(outbuf);
 					return -1;
@@ -376,18 +417,32 @@ belle_sip_memory_body_handler_t *belle_sip_memory_body_handler_new_copy_from_buf
 
 struct belle_sip_user_body_handler{
 	belle_sip_body_handler_t base;
+	belle_sip_user_body_handler_start_callback_t start_cb;
 	belle_sip_user_body_handler_recv_callback_t recv_cb;
 	belle_sip_user_body_handler_send_callback_t send_cb;
+	belle_sip_user_body_handler_stop_callback_t stop_cb;
 };
 
-static void belle_sip_user_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, const uint8_t *buf, size_t size){
+static void belle_sip_user_body_handler_begin_transfer(belle_sip_body_handler_t *base) {
+	belle_sip_user_body_handler_t *obj = (belle_sip_user_body_handler_t *)base;
+	if (obj->start_cb)
+		obj->start_cb((belle_sip_user_body_handler_t*)base, base->user_data);
+}
+
+static void belle_sip_user_body_handler_end_transfer(belle_sip_body_handler_t *base) {
+	belle_sip_user_body_handler_t *obj = (belle_sip_user_body_handler_t *)base;
+	if (obj->stop_cb)
+		obj->stop_cb((belle_sip_user_body_handler_t*)base, base->user_data);
+}
+
+static void belle_sip_user_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t size){
 	belle_sip_user_body_handler_t *obj=(belle_sip_user_body_handler_t*)base;
 	if (obj->recv_cb)
 		obj->recv_cb((belle_sip_user_body_handler_t*)base, msg, base->user_data, offset, buf, size);
 	else belle_sip_warning("belle_sip_user_body_handler_t ignoring received chunk.");
 }
 
-static int belle_sip_user_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, uint8_t *buf, size_t *size){
+static int belle_sip_user_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t *size){
 	belle_sip_user_body_handler_t *obj=(belle_sip_user_body_handler_t*)base;
 	if (obj->send_cb)
 		return obj->send_cb((belle_sip_user_body_handler_t*)base, msg, base->user_data, offset, buf, size);
@@ -397,8 +452,10 @@ static int belle_sip_user_body_handler_send_chunk(belle_sip_body_handler_t *base
 }
 
 static void belle_sip_user_body_handler_clone(belle_sip_user_body_handler_t *obj, const belle_sip_user_body_handler_t *orig){
+	obj->start_cb=orig->start_cb;
 	obj->recv_cb=orig->recv_cb;
 	obj->send_cb=orig->send_cb;
+	obj->stop_cb=orig->stop_cb;
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_user_body_handler_t);
@@ -408,10 +465,12 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_user_body_handler_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_user_body_handler_t,belle_sip_body_handler_t,TRUE),
 			(belle_sip_object_destroy_t) NULL,
 			(belle_sip_object_clone_t)belle_sip_user_body_handler_clone,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
-		NULL,
-		NULL,
+		belle_sip_user_body_handler_begin_transfer,
+		belle_sip_user_body_handler_begin_transfer,
+		belle_sip_user_body_handler_end_transfer,
 		belle_sip_user_body_handler_recv_chunk,
 		belle_sip_user_body_handler_send_chunk
 	}
@@ -420,14 +479,18 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 belle_sip_user_body_handler_t *belle_sip_user_body_handler_new(
 	size_t total_size,
 	belle_sip_body_handler_progress_callback_t progress_cb,
+	belle_sip_user_body_handler_start_callback_t start_cb,
 	belle_sip_user_body_handler_recv_callback_t recv_cb,
 	belle_sip_user_body_handler_send_callback_t send_cb,
+	belle_sip_user_body_handler_stop_callback_t stop_cb,
 	void *data){
 	belle_sip_user_body_handler_t * obj=belle_sip_object_new(belle_sip_user_body_handler_t);
 	belle_sip_body_handler_init((belle_sip_body_handler_t*)obj,progress_cb,data);
 	obj->base.expected_size=total_size;
+	obj->start_cb=start_cb;
 	obj->recv_cb=recv_cb;
 	obj->send_cb=send_cb;
+	obj->stop_cb=stop_cb;
 	return obj;
 }
 
@@ -439,53 +502,117 @@ belle_sip_user_body_handler_t *belle_sip_user_body_handler_new(
 struct belle_sip_file_body_handler{
 	belle_sip_body_handler_t base;
 	char *filepath;
+	bctbx_vfs_file_t *file;
+	belle_sip_user_body_handler_t *user_bh;
 };
 
 static void belle_sip_file_body_handler_destroy(belle_sip_file_body_handler_t *obj) {
 	if (obj->filepath) belle_sip_free(obj->filepath);
+	if (obj->file) {
+		ssize_t ret;
+		ret = bctbx_file_close(obj->file);
+		if (ret == BCTBX_VFS_ERROR) {
+			bctbx_error("Can't close file %s", obj->filepath);
+		}
+		obj->file = NULL;
+	}
+	if (obj->user_bh) {
+		belle_sip_object_unref(obj->user_bh);
+		obj->user_bh = NULL;
+	}
 }
 
 static void belle_sip_file_body_handler_clone(belle_sip_file_body_handler_t *obj, const belle_sip_file_body_handler_t *orig) {
 	obj->filepath = belle_sip_strdup(orig->filepath);
+	obj->file = orig->file;
+	obj->user_bh = orig->user_bh;
+	if (obj->user_bh) {
+		belle_sip_object_ref(obj->user_bh);
+	}
 }
 
-static void belle_sip_file_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, const uint8_t *buf, size_t size) {
-	FILE *f;
-	size_t ret;
+static void belle_sip_file_body_handler_begin_recv_transfer(belle_sip_body_handler_t *base) {
 	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
+	bctbx_vfs_t *vfs = bctbx_vfs_get_default();
+
 	if (obj->filepath == NULL) return;
-	f = fopen(obj->filepath, "ab");
-	if (f == NULL) return;
-	ret = fwrite(buf, 1, size, f);
-	if (ret != size) {
-		fclose(f);
-		return;
+	obj->file = bctbx_file_open(vfs, obj->filepath, "r+");
+	if (!obj->file) {
+		bctbx_error("Can't open file %s", obj->filepath);
+	}
+	
+	if (obj->user_bh && obj->user_bh->start_cb) {
+		obj->user_bh->start_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), obj->user_bh->base.user_data);
 	}
-	fclose(f);
 }
 
-static int belle_sip_file_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, uint8_t *buf, size_t *size) {
-	FILE *f;
-	int int_ret;
-	size_t size_t_ret;
+static void belle_sip_file_body_handler_begin_send_transfer(belle_sip_body_handler_t *base) {
 	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
+	bctbx_vfs_t *vfs = bctbx_vfs_get_default();
+
+	if (obj->filepath == NULL) return;
+	obj->file = bctbx_file_open(vfs, obj->filepath, "r");
+	if (!obj->file) {
+		bctbx_error("Can't open file %s", obj->filepath);
+	}
+
+	if (obj->user_bh && obj->user_bh->start_cb) {
+		obj->user_bh->start_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), obj->user_bh->base.user_data);
+	}
+}
+
+static void belle_sip_file_body_handler_end_transfer(belle_sip_body_handler_t *base) {
+	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
+	
+	if (obj->user_bh && obj->user_bh->stop_cb) {
+		obj->user_bh->stop_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), obj->user_bh->base.user_data);
+	}
+	
+	if (obj->file) {
+		ssize_t ret;
+		ret = bctbx_file_close(obj->file);
+		if (ret == BCTBX_VFS_ERROR) {
+			bctbx_error("Can't close file %s", obj->filepath);
+		}
+		obj->file = NULL;
+	}
+}
+
+static void belle_sip_file_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t size) {
+	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
+	ssize_t ret;
+
+	if (obj->file == NULL) return;
+	
+	if (obj->user_bh && obj->user_bh->recv_cb) {
+		obj->user_bh->recv_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), msg, obj->user_bh->base.user_data, offset, buf, size);
+	}
+	
+	ret = bctbx_file_write(obj->file, buf, size, offset);
+	if (ret == BCTBX_VFS_ERROR) {
+		bctbx_error("File body handler recv write error at offset %lu", (unsigned long)offset);
+	}
+}
+
+static int belle_sip_file_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t *size) {
+	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
+	ssize_t size_t_ret;
 	size_t to_send = MIN(*size, obj->base.expected_size - offset);
-	if (obj->filepath == NULL) return BELLE_SIP_STOP;
-	f = fopen(obj->filepath, "rb");
-	if (f == NULL) return BELLE_SIP_STOP;
-	int_ret = fseek(f, (long)offset, SEEK_SET);
-	if (int_ret < 0) {
-		fclose(f);
+
+	if (obj->file == NULL) return BELLE_SIP_STOP;
+	size_t_ret = bctbx_file_read(obj->file, buf, to_send, offset);
+	if (size_t_ret == BCTBX_VFS_ERROR) {
+		bctbx_error("File body handler send read error at offset %lu", (unsigned long)offset);
 		return BELLE_SIP_STOP;
 	}
-	size_t_ret = fread(buf, 1, to_send, f);
-	if (size_t_ret < 0) {
-		fclose(f);
-		return BELLE_SIP_STOP;
+	*size = (size_t)size_t_ret;
+	
+	if (obj->user_bh && obj->user_bh->send_cb) {
+		int result = obj->user_bh->send_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), msg, obj->user_bh->base.user_data, offset, buf, size);
+		if (result == BELLE_SIP_STOP) return result;
 	}
-	*size = size_t_ret;
-	fclose(f);
-	return (((obj->base.expected_size - offset) == *size) || (*size == 0)) ? BELLE_SIP_STOP : BELLE_SIP_CONTINUE;
+	
+	return (((obj->base.expected_size - offset) == (size_t)size_t_ret) || (*size == 0)) ? BELLE_SIP_STOP : BELLE_SIP_CONTINUE;
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_file_body_handler_t);
@@ -495,10 +622,12 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_file_body_handler_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_file_body_handler_t,belle_sip_body_handler_t,TRUE),
 			(belle_sip_object_destroy_t) belle_sip_file_body_handler_destroy,
 			(belle_sip_object_clone_t)belle_sip_file_body_handler_clone,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
-		NULL,
-		NULL,
+		belle_sip_file_body_handler_begin_recv_transfer,
+		belle_sip_file_body_handler_begin_send_transfer,
+		belle_sip_file_body_handler_end_transfer,
 		belle_sip_file_body_handler_recv_chunk,
 		belle_sip_file_body_handler_send_chunk
 	}
@@ -509,12 +638,23 @@ belle_sip_file_body_handler_t *belle_sip_file_body_handler_new(const char *filep
 	belle_sip_file_body_handler_t *obj = belle_sip_object_new(belle_sip_file_body_handler_t);
 	belle_sip_body_handler_init((belle_sip_body_handler_t*)obj, progress_cb, data);
 	obj->filepath = belle_sip_strdup(filepath);
+	obj->user_bh = NULL;
 	if (stat(obj->filepath, &statbuf) == 0) {
 		obj->base.expected_size = statbuf.st_size;
 	}
 	return obj;
 }
 
+size_t belle_sip_file_body_handler_get_file_size(belle_sip_file_body_handler_t *file_bh) {
+	return file_bh->base.expected_size;
+}
+
+void belle_sip_file_body_handler_set_user_body_handler(belle_sip_file_body_handler_t *file_bh, belle_sip_user_body_handler_t *user_bh) {
+	if (file_bh) {
+		file_bh->user_bh = user_bh;
+		belle_sip_object_ref(file_bh->user_bh);
+	}
+}
 
 /*
  * Multipart body handler implementation
@@ -541,12 +681,22 @@ static void belle_sip_multipart_body_handler_clone(belle_sip_multipart_body_hand
 	obj->parts=belle_sip_list_copy_with_data(obj->parts,(void *(*)(void*))belle_sip_object_clone_and_ref);
 }
 
-static void belle_sip_multipart_body_handler_begin_transfer(belle_sip_body_handler_t *obj) {
+static void belle_sip_multipart_body_handler_begin_recv_transfer(belle_sip_body_handler_t *obj) {
 	const belle_sip_list_t *it;
 	belle_sip_multipart_body_handler_t *obj_multipart = (belle_sip_multipart_body_handler_t *)obj;
 	for (it = obj_multipart->parts; it != NULL; it = it->next) {
 		belle_sip_body_handler_t *bh = BELLE_SIP_BODY_HANDLER(it->data);
-		belle_sip_body_handler_begin_transfer(bh);
+		belle_sip_body_handler_begin_recv_transfer(bh);
+	}
+	obj_multipart->transfer_current_part = obj_multipart->parts;
+}
+
+static void belle_sip_multipart_body_handler_begin_send_transfer(belle_sip_body_handler_t *obj) {
+	const belle_sip_list_t *it;
+	belle_sip_multipart_body_handler_t *obj_multipart = (belle_sip_multipart_body_handler_t *)obj;
+	for (it = obj_multipart->parts; it != NULL; it = it->next) {
+		belle_sip_body_handler_t *bh = BELLE_SIP_BODY_HANDLER(it->data);
+		belle_sip_body_handler_begin_send_transfer(bh);
 	}
 	obj_multipart->transfer_current_part = obj_multipart->parts;
 }
@@ -561,8 +711,8 @@ static void belle_sip_multipart_body_handler_end_transfer(belle_sip_body_handler
 	obj_multipart->transfer_current_part = NULL;
 }
 
-static void belle_sip_multipart_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset,
-							const uint8_t *buffer, size_t size){
+static void belle_sip_multipart_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, off_t offset,
+							uint8_t *buffer, size_t size){
 	/* Store the whole buffer, the parts will be split when belle_sip_multipart_body_handler_progress_cb() is called with transfered size equal to expected size. */
 	belle_sip_multipart_body_handler_t *obj_multipart = (belle_sip_multipart_body_handler_t *)obj;
 	obj_multipart->buffer = belle_sip_realloc(obj_multipart->buffer,offset + size + 1);
@@ -570,7 +720,7 @@ static void belle_sip_multipart_body_handler_recv_chunk(belle_sip_body_handler_t
 	obj_multipart->buffer[offset + size] = '\0';
 }
 
-static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset,
+static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, off_t offset,
 							uint8_t *buffer, size_t *size){
 	belle_sip_multipart_body_handler_t *obj_multipart=(belle_sip_multipart_body_handler_t*)obj;
 
@@ -609,7 +759,6 @@ static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t
 		}
 
 		retval = belle_sip_body_handler_send_chunk(current_part, msg, buffer+offsetSize, size); /* add offsetSize to the buffer address in order to point at the begining of free space (after header if included) */
-
 		*size +=offsetSize; /* restore total of data given including potential separator and header */
 
 
@@ -639,9 +788,11 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_multipart_body_handler_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_multipart_body_handler_t,belle_sip_body_handler_t,TRUE),
 			(belle_sip_object_destroy_t) belle_sip_multipart_body_handler_destroy,
 			(belle_sip_object_clone_t)belle_sip_multipart_body_handler_clone,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
-		belle_sip_multipart_body_handler_begin_transfer,
+		belle_sip_multipart_body_handler_begin_recv_transfer,
+		belle_sip_multipart_body_handler_begin_send_transfer,
 		belle_sip_multipart_body_handler_end_transfer,
 		belle_sip_multipart_body_handler_recv_chunk,
 		belle_sip_multipart_body_handler_send_chunk
@@ -675,7 +826,7 @@ belle_sip_multipart_body_handler_t *belle_sip_multipart_body_handler_new_from_bu
 	belle_sip_body_handler_init((belle_sip_body_handler_t *)obj, belle_sip_multipart_body_handler_progress_cb, NULL);
 	belle_sip_multipart_body_handler_set_boundary(obj_multipart, boundary);
 	obj_multipart->base.expected_size = bufsize;
-	belle_sip_body_handler_begin_transfer(obj);
+	belle_sip_body_handler_begin_recv_transfer(obj);
 	belle_sip_body_handler_recv_chunk(obj, NULL, (uint8_t *)buffer, bufsize);
 	belle_sip_body_handler_end_transfer(obj);
 	return obj_multipart;
