@@ -22,10 +22,21 @@
 #include <ctype.h>
 #include <wchar.h>
 
-#ifdef ANDROID
+#ifdef __ANDROID__
 #include "wakelock_internal.h"
 #endif
 
+#define BELLE_SIP_CHANNEL_INVOKE_MESSAGE_HEADERS_LISTENERS(channel,msg) \
+	BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(channel->full_listeners, belle_sip_channel_listener_t, on_message_headers, channel, msg)
+
+#define BELLE_SIP_CHANNEL_INVOKE_SENDING_LISTENERS(channel,msg) \
+	BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(channel->full_listeners, belle_sip_channel_listener_t, on_sending, channel, msg)
+
+#define BELLE_SIP_CHANNEL_INVOKE_STATE_LISTENERS(channel,state) \
+	BELLE_SIP_INVOKE_LISTENERS_REVERSE_ARG1_ARG2(channel->full_listeners, belle_sip_channel_listener_t, on_state_changed, channel, state) \
+	BELLE_SIP_INVOKE_LISTENERS_REVERSE_ARG1_ARG2(channel->state_listeners, belle_sip_channel_listener_t, on_state_changed, channel, state) 
+
+
 static void channel_prepare_continue(belle_sip_channel_t *obj);
 static void channel_process_queue(belle_sip_channel_t *obj);
 static void channel_begin_send_background_task(belle_sip_channel_t *obj);
@@ -103,12 +114,21 @@ static void belle_sip_channel_destroy(belle_sip_channel_t *obj){
 	if (obj->peer_cname) belle_sip_free(obj->peer_cname);
 	belle_sip_free(obj->peer_name);
 	if (obj->local_ip) belle_sip_free(obj->local_ip);
-	obj->listeners=for_each_weak_unref_free(obj->listeners,(belle_sip_object_destroy_notify_t)channel_remove_listener,obj);
-	if (obj->resolver_ctx>0) belle_sip_resolver_context_cancel(obj->resolver_ctx);
+	obj->state_listeners=for_each_weak_unref_free(obj->state_listeners,(belle_sip_object_destroy_notify_t)channel_remove_listener,obj);
+	obj->full_listeners=for_each_weak_unref_free(obj->full_listeners,(belle_sip_object_destroy_notify_t)channel_remove_listener,obj);
+
+	if (obj->resolver_ctx != NULL) {
+		belle_sip_resolver_context_cancel(obj->resolver_ctx);
+		belle_sip_object_unref(obj->resolver_ctx);
+	}
 	if (obj->inactivity_timer){
 		belle_sip_main_loop_remove_source(obj->stack->ml,obj->inactivity_timer);
 		belle_sip_object_unref(obj->inactivity_timer);
 	}
+	if (obj->dns_ttl_timer) {
+		belle_sip_main_loop_remove_source(obj->stack->ml, obj->dns_ttl_timer);
+		belle_sip_object_unref(obj->dns_ttl_timer);
+	}
 	if (obj->public_ip) belle_sip_free(obj->public_ip);
 	if (obj->outgoing_messages) belle_sip_list_free_with_data(obj->outgoing_messages,belle_sip_object_unref);
 	if (obj->incoming_messages) belle_sip_list_free_with_data(obj->incoming_messages,belle_sip_object_unref);
@@ -132,7 +152,14 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_channel_t)
 		(belle_sip_object_destroy_t)belle_sip_channel_destroy,
 		NULL, /*clone*/
 		NULL, /*marshal*/
-	}
+		BELLE_SIP_DEFAULT_BUFSIZE_HINT
+	},
+	NULL, /* transport */
+	0, /* reliable */
+	NULL, /* connect */
+	NULL, /* channel_send */
+	NULL, /* channel_recv */
+	NULL /* close */
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
 static void fix_incoming_via(belle_sip_request_t *msg, const struct addrinfo* origin){
@@ -148,7 +175,7 @@ static void fix_incoming_via(belle_sip_request_t *msg, const struct addrinfo* or
 		return;
 	}
 	bctbx_sockaddr_remove_v4_mapping(origin->ai_addr, (struct sockaddr*)&saddr, &slen);
-	err=getnameinfo((struct sockaddr*)&saddr,slen,received,sizeof(received),
+	err=bctbx_getnameinfo((struct sockaddr*)&saddr,slen,received,sizeof(received),
 	                rport,sizeof(rport),NI_NUMERICHOST|NI_NUMERICSERV);
 	if (err!=0){
 		belle_sip_error("fix_via: getnameinfo() failed: %s",gai_strerror(errno));
@@ -236,13 +263,6 @@ static int get_message_start_pos(char *buff, size_t bufflen) {
 	return -1;
 }
 
-char *belle_sip_channel_get_public_ip_port(belle_sip_channel_t *obj){
-	if (obj->public_ip){
-		return belle_sip_strdup_printf("%s:%d", obj->public_ip, obj->public_port);
-	}
-	return NULL;
-}
-
 void belle_sip_channel_set_public_ip_port(belle_sip_channel_t *obj, const char *public_ip, int port){
 	if (obj->public_ip){
 		int ip_changed=0;
@@ -354,9 +374,10 @@ static int check_body(belle_sip_channel_t *obj){
 
 	obj->input_stream.content_length= content_length_header ? belle_sip_header_content_length_get_content_length(content_length_header) : 0;
 
-	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg,belle_sip_response_t) || BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg,belle_sip_request_t)){
-		expect_body=obj->input_stream.content_length>0;
-	}else{/*http*/
+	expect_body=obj->input_stream.content_length>0;
+	
+	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg,belle_http_response_t) || BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg,belle_http_request_t)){
+		/*http chunked mode handling*/
 		if (belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t)!=NULL){
 			belle_sip_header_t *transfer_encoding=belle_sip_message_get_header(msg,"Transfer-Encoding");
 
@@ -375,7 +396,7 @@ static int check_body(belle_sip_channel_t *obj){
 	if (expect_body){
 		belle_sip_body_handler_t *bh;
 		/*should notify the listeners*/
-		BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(obj->listeners,belle_sip_channel_listener_t,on_message_headers,obj,msg);
+		BELLE_SIP_CHANNEL_INVOKE_MESSAGE_HEADERS_LISTENERS(obj,msg);
 		/*check if the listener has setup a body handler, otherwise create a default one*/
 		if ((bh=belle_sip_message_get_body_handler(msg))==NULL){
 			belle_sip_header_t *content_encoding = belle_sip_message_get_header(msg, "Content-Encoding");
@@ -394,7 +415,7 @@ static int check_body(belle_sip_channel_t *obj){
 				belle_sip_message_set_body_handler(msg,(bh=(belle_sip_body_handler_t*)belle_sip_memory_body_handler_new(NULL,NULL)));
 			}
 		}
-		belle_sip_body_handler_begin_transfer(bh);
+		belle_sip_body_handler_begin_recv_transfer(bh);
 	}
 	return expect_body;
 }
@@ -492,7 +513,8 @@ static int acquire_body(belle_sip_channel_t *obj, int end_of_stream){
 
 static void notify_incoming_messages(belle_sip_channel_t *obj){
 	belle_sip_list_t *elem,*l_it;
-	belle_sip_list_t *listeners=belle_sip_list_copy_with_data(obj->listeners,(void *(*)(void*))belle_sip_object_ref);
+	
+	belle_sip_list_t *listeners=belle_sip_list_copy_with_data(obj->full_listeners,(void *(*)(void*))belle_sip_object_ref);
 
 	for(l_it=listeners;l_it!=NULL;l_it=l_it->next){
 		belle_sip_channel_listener_t *listener=(belle_sip_channel_listener_t*)l_it->data;
@@ -652,6 +674,7 @@ static int belle_sip_channel_process_read_data(belle_sip_channel_t *obj){
 	} else if (num == 0) {
 		/*before closing the channel, check if there was a pending message to receive, whose body acquisition is to be finished.*/
 		belle_sip_channel_process_stream(obj,TRUE);
+		obj->closed_by_remote = TRUE;
 		channel_set_state(obj,BELLE_SIP_CHANNEL_DISCONNECTED);
 		ret=BELLE_SIP_STOP;
 	} else if (belle_sip_error_code_is_would_block(-num)){
@@ -667,6 +690,7 @@ static int belle_sip_channel_process_read_data(belle_sip_channel_t *obj){
 
 int belle_sip_channel_process_data(belle_sip_channel_t *obj,unsigned int revents){
 	int ret=BELLE_SIP_CONTINUE;
+	belle_sip_object_ref(obj);
 	if (revents & BELLE_SIP_EVENT_READ) {
 		int rret=belle_sip_channel_process_read_data(obj);
 		if (rret==BELLE_SIP_STOP) ret=BELLE_SIP_STOP;
@@ -679,6 +703,7 @@ int belle_sip_channel_process_data(belle_sip_channel_t *obj,unsigned int revents
 			channel_process_queue(obj);
 		}
 	}
+	belle_sip_object_unref(obj);
 	return ret;
 }
 
@@ -769,14 +794,31 @@ void belle_sip_channel_set_socket(belle_sip_channel_t *obj, belle_sip_socket_t s
 									, -1);
 }
 
+static bool_t is_state_only_listener(const belle_sip_channel_listener_t *listener) {
+	BELLE_SIP_INTERFACE_METHODS_TYPE(belle_sip_channel_listener_t) *methods;
+	methods=BELLE_SIP_INTERFACE_GET_METHODS(listener,belle_sip_channel_listener_t);
+	return methods->on_state_changed && !(methods->on_message_headers || methods->on_message || methods->on_sending || methods->on_auth_requested);
+}
 static void channel_remove_listener(belle_sip_channel_t *obj, belle_sip_channel_listener_t *l){
-	obj->listeners=belle_sip_list_remove(obj->listeners,l);
+	if (is_state_only_listener(l))
+		obj->state_listeners=belle_sip_list_remove(obj->state_listeners,l);
+	else
+		obj->full_listeners=belle_sip_list_remove(obj->full_listeners,l);
+
 }
 
 void belle_sip_channel_add_listener(belle_sip_channel_t *obj, belle_sip_channel_listener_t *l){
-	obj->listeners=belle_sip_list_append(obj->listeners,
-	                belle_sip_object_weak_ref(l,
-	                (belle_sip_object_destroy_notify_t)channel_remove_listener,obj));
+	
+	if (is_state_only_listener(l)) {
+		obj->state_listeners=belle_sip_list_prepend(obj->state_listeners,
+												   belle_sip_object_weak_ref(l,
+																	   (belle_sip_object_destroy_notify_t)channel_remove_listener,obj));
+	} else {
+		obj->full_listeners=belle_sip_list_prepend(obj->full_listeners,
+													belle_sip_object_weak_ref(l,
+																		(belle_sip_object_destroy_notify_t)channel_remove_listener,obj));
+	}
+	
 }
 
 void belle_sip_channel_remove_listener(belle_sip_channel_t *obj, belle_sip_channel_listener_t *l){
@@ -800,6 +842,12 @@ const char *belle_sip_channel_get_local_address(belle_sip_channel_t *obj, int *p
 	return obj->local_ip;
 }
 
+const char *belle_sip_channel_get_public_address(belle_sip_channel_t *obj, int *port){
+	const char *ret = obj->public_ip ? obj->public_ip : obj->local_ip;
+	if (*port) *port= obj->public_port;
+	return ret;
+}
+
 belle_sip_uri_t *belle_sip_channel_create_routable_uri(belle_sip_channel_t *chan) {
 	const char *transport = belle_sip_channel_get_transport_name_lower_case(chan);
 	belle_sip_uri_t* uri = belle_sip_uri_new();
@@ -922,18 +970,25 @@ static void channel_invoke_state_listener(belle_sip_channel_t *obj){
 	}
 	/*Channel listeners may drop the last reference of the channel, so protect by ref/unref until we finish.*/
 	belle_sip_object_ref(obj);
-	BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(obj->listeners,belle_sip_channel_listener_t,on_state_changed,obj,obj->state);
+	BELLE_SIP_CHANNEL_INVOKE_STATE_LISTENERS(obj,obj->state);
 	if (close) belle_sip_channel_close(obj);
 	belle_sip_object_unref(obj);
 }
 
-static void channel_invoke_state_listener_defered(belle_sip_channel_t *obj){
-	channel_invoke_state_listener(obj);
+static void channel_notify_error_to_listeners(belle_sip_channel_t *obj){
+	/* The channel may have been passed to DISCONNECTED state due to _force_close() method.
+	 * Do not notify the error in this case, since the channel is already closed.
+	 */
+	if (obj->state == BELLE_SIP_CHANNEL_ERROR){
+		channel_invoke_state_listener(obj);
+	}
 	belle_sip_object_unref(obj);
 }
 
 static void channel_connect_next(belle_sip_channel_t *obj){
-	belle_sip_channel_connect(obj);
+	if (obj->state == BELLE_SIP_CHANNEL_RETRY){
+		belle_sip_channel_connect(obj);
+	}
 	belle_sip_object_unref(obj);
 }
 
@@ -956,12 +1011,18 @@ static void belle_sip_channel_handle_error(belle_sip_channel_t *obj){
 	* it is safer to invoke the listener outside the current call stack.
 	* Indeed the channel encounters network errors while being called for transmiting by a transaction.
 	*/
-	belle_sip_main_loop_do_later(obj->stack->ml,(belle_sip_callback_t)channel_invoke_state_listener_defered,belle_sip_object_ref(obj));
+	belle_sip_main_loop_do_later(obj->stack->ml,(belle_sip_callback_t)channel_notify_error_to_listeners,belle_sip_object_ref(obj));
 }
 
 int belle_sip_channel_notify_timeout(belle_sip_channel_t *obj){
 	const int too_long=60;
-	if (belle_sip_time_ms() - obj->last_recv_time>=(too_long * 1000)){
+	
+	if (obj->state != BELLE_SIP_CHANNEL_READY){
+		/*no need to notify the timeout if the channel is already in error or retry state*/
+		return FALSE;
+	}
+	
+	if ((int)(belle_sip_time_ms() - obj->last_recv_time) >= (too_long * 1000)){
 		belle_sip_message("A timeout related to this channel occured and no message received during last %i seconds. This channel is suspect, moving to error state",too_long);
 		channel_set_state(obj,BELLE_SIP_CHANNEL_ERROR);
 		return TRUE;
@@ -1041,26 +1102,25 @@ static char *make_logbuf(belle_sip_channel_t *obj, belle_sip_log_level level, co
 	if (obj->stop_logging_buffer == 1) {
 		return NULL;
 	}
-	limit=find_non_printable(buffer,MIN(size,limit));
-	if (limit != size) {
+	size = MIN(size,limit);
+	limit=find_non_printable(buffer, size);
+	if (limit < size) {
 		belle_sip_message("channel [%p]: found binary data in buffer, will stop logging it now.", obj);
 		obj->stop_logging_buffer = 1;
-		if (limit==0){
+		if (limit == 0){
 			snprintf(truncate_msg,sizeof(truncate_msg)-1,"... (binary data)");
 		} else {
-			size=limit;
 			snprintf(truncate_msg,sizeof(truncate_msg)-1,"... (first %u bytes shown)",(unsigned int)limit);
 		}
 	}
+	size = limit;
 
-	if (truncate_msg[0]!=0){
-		size+=sizeof(truncate_msg);
-	}
+	size += strlen(truncate_msg);
 
 	logbuf=belle_sip_malloc(size+1);
-	strncpy(logbuf,buffer,limit);
+	strncpy(logbuf, buffer, size);
 	if (truncate_msg[0]!=0){
-		strncpy(logbuf+limit,truncate_msg,sizeof(truncate_msg));
+		strcpy(logbuf+limit,truncate_msg);
 	}
 	logbuf[size]='\0';
 	return logbuf;
@@ -1199,7 +1259,7 @@ static void _send_message(belle_sip_channel_t *obj){
 	}
 
 	if (obj->out_state==OUTPUT_STREAM_SENDING_HEADERS){
-		BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(obj->listeners,belle_sip_channel_listener_t,on_sending,obj,msg);
+		BELLE_SIP_CHANNEL_INVOKE_SENDING_LISTENERS(obj,msg);
 		check_content_length(msg,body_len);
 		error=belle_sip_object_marshal((belle_sip_object_t*)msg,buffer,sizeof(buffer)-1,&len);
 		if (error!=BELLE_SIP_OK) {
@@ -1211,7 +1271,7 @@ static void _send_message(belle_sip_channel_t *obj){
 			size_t max_body_len=sizeof(buffer)-1-len;
 
 			if (body_len>0 && body_len<=max_body_len){ /*if size is known and fits into our buffer, send together with headers*/
-				belle_sip_body_handler_begin_transfer(bh);
+				belle_sip_body_handler_begin_send_transfer(bh);
 				do{
 					max_body_len=sizeof(buffer)-1-len;
 					ret=belle_sip_body_handler_send_chunk(bh,msg,(uint8_t*)buffer+len,&max_body_len);
@@ -1224,7 +1284,7 @@ static void _send_message(belle_sip_channel_t *obj){
 				if (body_len==0){
 					belle_sip_fatal("Sending bodies whose size is not known must be done in chunked mode, which is not supported yet.");
 				}
-				belle_sip_body_handler_begin_transfer(bh);
+				belle_sip_body_handler_begin_send_transfer(bh);
 				obj->out_state=OUTPUT_STREAM_SENDING_BODY;
 			}
 		}
@@ -1323,8 +1383,9 @@ static void channel_process_queue(belle_sip_channel_t *obj){
 	belle_sip_message_t *msg;
 	belle_sip_object_ref(obj);/* we need to ref ourself because code below may trigger our destruction*/
 
-	if (obj->out_state!=OUTPUT_STREAM_IDLE)
+	if (obj->out_state!=OUTPUT_STREAM_IDLE) {
 		_send_message(obj);
+	}
 
 	while((msg=channel_pop_outgoing(obj))!=NULL && obj->state==BELLE_SIP_CHANNEL_READY && obj->out_state==OUTPUT_STREAM_IDLE) {
 		send_message(obj, msg);
@@ -1348,7 +1409,7 @@ void belle_sip_channel_set_ready(belle_sip_channel_t *obj, const struct sockaddr
 
 		bctbx_sockaddr_remove_v4_mapping(addr,(struct sockaddr*) &saddr,&slen2);
 
-		err=getnameinfo((struct sockaddr*)&saddr,slen2,name,sizeof(name),serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
+		err=bctbx_getnameinfo((struct sockaddr*)&saddr,slen2,name,sizeof(name),serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
 		if (err!=0){
 			belle_sip_error("belle_sip_channel_set_ready(): getnameinfo() failed: %s",gai_strerror(err));
 		}else{
@@ -1361,16 +1422,57 @@ void belle_sip_channel_set_ready(belle_sip_channel_t *obj, const struct sockaddr
 	channel_process_queue(obj);
 }
 
-static void channel_res_done(void *data, const char *name, struct addrinfo *ai_list){
+static int channel_dns_ttl_timeout(void *data, unsigned int event) {
+	belle_sip_channel_t *obj = (belle_sip_channel_t *)data;
+	belle_sip_message("Channel [%p]: DNS TTL timeout reached.", obj);
+	obj->dns_ttl_timedout = TRUE;
+	return BELLE_SIP_STOP;
+}
+
+static bool_t addrinfo_in_list(const struct addrinfo *ai, const struct addrinfo *list) {
+	const struct addrinfo *item = list;
+	bool_t in_list = FALSE;
+	while (item != NULL) {
+		if ((ai->ai_family == item->ai_family) && (bctbx_sockaddr_equals(ai->ai_addr, item->ai_addr))) {
+			in_list = TRUE;
+			break;
+		}
+		item = item->ai_next;
+	}
+	return in_list;
+}
+
+static void channel_res_done(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl){
 	belle_sip_channel_t *obj=(belle_sip_channel_t*)data;
 	if (obj->resolver_ctx){
 		belle_sip_object_unref(obj->resolver_ctx);
 		obj->resolver_ctx=NULL;
 	}
 	if (ai_list){
-		obj->peer_list=obj->current_peer=ai_list;
-		channel_set_state(obj,BELLE_SIP_CHANNEL_RES_DONE);
+		if (!obj->current_peer) {
+			obj->peer_list=obj->current_peer=ai_list;
+			channel_set_state(obj,BELLE_SIP_CHANNEL_RES_DONE);
+		} else {
+			if (addrinfo_in_list(obj->current_peer, ai_list)) {
+				belle_sip_message("channel[%p]: DNS resolution returned the currently used address, continue using it", obj);
+				obj->peer_list = ai_list;
+				channel_set_state(obj, BELLE_SIP_CHANNEL_READY);
+			} else {
+				belle_sip_message("channel[%p]: DNS resolution returned an address different than the one being used, reconnect to the new address", obj);
+				obj->peer_list = obj->current_peer = ai_list;
+				belle_sip_channel_close(obj);
+				belle_sip_main_loop_do_later(obj->stack->ml, (belle_sip_callback_t)channel_connect_next, belle_sip_object_ref(obj));
+				channel_set_state(obj, BELLE_SIP_CHANNEL_RETRY);
+			}
+		}
 		channel_prepare_continue(obj);
+		if (!obj->dns_ttl_timer ) {
+			obj->dns_ttl_timer = belle_sip_main_loop_create_timeout(obj->stack->ml, channel_dns_ttl_timeout, obj, ttl * 1000, "Channel DNS TTL timer");
+		} else {
+			/* Restart the timer for new period. */
+			belle_sip_source_set_timeout(obj->dns_ttl_timer, ttl * 1000);
+			belle_sip_main_loop_add_source(obj->stack->ml, obj->dns_ttl_timer);
+		}
 	}else{
 		belle_sip_error("%s: DNS resolution failed for %s", __FUNCTION__, name);
 		channel_set_state(obj,BELLE_SIP_CHANNEL_ERROR);
@@ -1481,7 +1583,14 @@ belle_sip_channel_t *belle_sip_channel_find_from_list(belle_sip_list_t *l, int a
 	return chan;
 }
 
-#ifdef ANDROID
+void belle_sip_channel_check_dns_reusability(belle_sip_channel_t *obj) {
+	if (obj->dns_ttl_timedout) {
+		obj->dns_ttl_timedout = FALSE;
+		belle_sip_channel_resolve(obj);
+	}
+}
+
+#ifdef __ANDROID__
 
 unsigned long belle_sip_begin_background_task(const char *name, belle_sip_background_task_end_callback_t cb, void *data){
     return wake_lock_acquire(name);
