@@ -90,7 +90,8 @@ struct belle_sip_channel{
 	belle_sip_stack_t *stack;
 	belle_sip_listening_point_t *lp; /*the listening point that owns this channel*/
 	belle_sip_channel_state_t state;
-	belle_sip_list_t *listeners;
+	belle_sip_list_t *state_listeners;
+	belle_sip_list_t *full_listeners;
 	int ai_family;
 	char *peer_cname;
 	char *peer_name;
@@ -111,6 +112,7 @@ struct belle_sip_channel{
 	belle_sip_channel_input_stream_t input_stream;
 	belle_sip_list_t* incoming_messages;
 	belle_sip_source_t *inactivity_timer;
+	belle_sip_source_t *dns_ttl_timer;
 	uint64_t last_recv_time;
 	int simulated_recv_return; /* used to simulate network error. 0= no data (disconnected) >0= do nothing -1= network error, 1500 special number to silently discard incoming buffer*/
 	unsigned long bg_task_id;
@@ -122,6 +124,8 @@ struct belle_sip_channel{
 	unsigned char srv_overrides_port; /*set when this channel was connected to destination port provided by SRV resolution*/
 	unsigned char soft_error; /*set when this channel enters ERROR state because of error detected in upper layer */
 	int stop_logging_buffer; /*log buffer content only if this is non binary data, and stop it at the first occurence*/
+	bool_t closed_by_remote; /*If the channel has been remotely closed*/
+	bool_t dns_ttl_timedout;
 };
 
 #define BELLE_SIP_CHANNEL(obj)		BELLE_SIP_CAST(obj,belle_sip_channel_t)
@@ -166,11 +170,11 @@ int belle_sip_channel_is_reliable(const belle_sip_channel_t *obj);
 
 const char * belle_sip_channel_get_transport_name(const belle_sip_channel_t *obj);
 const char * belle_sip_channel_get_transport_name_lower_case(const belle_sip_channel_t *obj);
-char *belle_sip_channel_get_public_ip_port(belle_sip_channel_t *obj);
 
 const struct addrinfo * belle_sip_channel_get_peer(belle_sip_channel_t *obj);
 
 const char *belle_sip_channel_get_local_address(belle_sip_channel_t *obj, int *port);
+const char *belle_sip_channel_get_public_address(belle_sip_channel_t *obj, int *port);
 
 /*
  * Returns a sip-uri suitable for using in record-route.
@@ -183,7 +187,14 @@ belle_sip_uri_t *belle_sip_channel_create_routable_uri(belle_sip_channel_t *chan
 
 void channel_set_state(belle_sip_channel_t *obj, belle_sip_channel_state_t state);
 
-/*just invokes the listeners to process data*/
+/*
+ * Process incoming data and synchronously invoke the listeners if
+ * complete message are received. The invocation of the listeners may
+ * result in the channel being destroyed (ex: calling belle_sip_listening_point_clean_channels() within 
+ * a transaction completed notification).
+ * WARNING: As a result, the caller of this function must be take into account that the channel no longer exists
+ * in return from this function.
+ */
 int belle_sip_channel_process_data(belle_sip_channel_t *obj,unsigned int revents);
 
 /*this function is to be used only in belle_sip_listening_point_clean_channels()*/
@@ -196,7 +207,12 @@ void belle_sip_channel_force_close(belle_sip_channel_t *obj);
 int belle_sip_channel_notify_timeout(belle_sip_channel_t *obj);
 
 /*Used by transaction layer to report a server having internal errors, so that we can retry with another IP (in case of DNS SRV)*/
-void belle_sip_channel_notify_server_error(belle_sip_channel_t *obj);
+BELLESIP_EXPORT void belle_sip_channel_notify_server_error(belle_sip_channel_t *obj);
+
+/*
+ * Check if the DNS TTL has expired. If this is the case, set the channel status to RES_IN_PROGRESS.
+ */
+void belle_sip_channel_check_dns_reusability(belle_sip_channel_t *obj);
 
 BELLE_SIP_END_DECLS
 
@@ -224,6 +240,7 @@ belle_sip_channel_t *belle_sip_channel_find_from_list(belle_sip_list_t *l, int a
 struct belle_tls_crypto_config{
 	belle_sip_object_t base;
 	char *root_ca; /**< path to the trusted certificate chain used when verifiying peer certificate */
+	char *root_ca_data; /**< content of the trusted certificate chain used when verifiying peer certificate */
 	int exception_flags; /**< override some exception raised during certificate verification, can be:
 						   BELLE_TLS_VERIFY_NONE do not override any exception
 						   BELLE_TLS_VERIFY_CN_MISMATCH ignore Common Name mismatch exception
