@@ -960,7 +960,7 @@ int dns_resconf_load_struct_res_state_nameservers(struct dns_resolv_conf *rescon
 DNS_PUBLIC int dns_resconf_loadwin(struct dns_resolv_conf *);
 #endif
 
-#ifdef ANDROID
+#ifdef __ANDROID__
 DNS_PUBLIC int dns_resconf_loadandroid(struct dns_resolv_conf *resconf);
 #endif
 
@@ -1098,6 +1098,7 @@ struct dns_options {
 		DNS_SYSPOLL,
 		DNS_LIBEVENT,
 	} events;
+	int udp_uses_connect;
 }; /* struct dns_options */
 
 
@@ -1202,6 +1203,8 @@ DNS_PUBLIC void dns_res_sethints(struct dns_resolver *, struct dns_hints *);
 
 DNS_PUBLIC void dns_res_enable_search(struct dns_resolver *, unsigned char enable);
 
+DNS_PUBLIC int dns_res_was_asymetric(struct dns_resolver *);
+
 
 /*
  * A D D R I N F O  I N T E R F A C E
