@@ -16,10 +16,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/** 
+/**
  * INVITE client transaction implementation.
 **/
 
+#include <bctoolbox/defs.h>
+
 #include "belle_sip_internal.h"
 
 
@@ -78,6 +80,7 @@ static belle_sip_request_t *make_ack(belle_sip_ict_t *obj, belle_sip_response_t
 	return obj->ack;
 }
 
+/* Timer D: Wait time for response retransmits */
 static int ict_on_timer_D(belle_sip_ict_t *obj){
 	belle_sip_transaction_t *base=(belle_sip_transaction_t*)obj;
 	if (base->state==BELLE_SIP_TRANSACTION_COMPLETED){
@@ -86,6 +89,7 @@ static int ict_on_timer_D(belle_sip_ict_t *obj){
 	return BELLE_SIP_STOP;
 }
 
+/* Timer M: Wait time for retransmission of 2xx to INVITE or additional 2xx from other branches of a forked INVITE */
 static int ict_on_timer_M(belle_sip_ict_t *obj){
 	belle_sip_transaction_t *base=(belle_sip_transaction_t*)obj;
 	if (base->state==BELLE_SIP_TRANSACTION_ACCEPTED){
@@ -102,7 +106,7 @@ static void ict_on_response(belle_sip_ict_t *obj, belle_sip_response_t *resp){
 	switch (base->state){
 		case BELLE_SIP_TRANSACTION_CALLING:
 			belle_sip_transaction_set_state(base,BELLE_SIP_TRANSACTION_PROCEEDING);
-			/* no break*/
+			BCTBX_NO_BREAK; /*intentionally no break*/
 		case BELLE_SIP_TRANSACTION_PROCEEDING:
 			if (code>=300){
 				belle_sip_transaction_set_state(base,BELLE_SIP_TRANSACTION_COMPLETED);
@@ -134,9 +138,12 @@ static void ict_on_response(belle_sip_ict_t *obj, belle_sip_response_t *resp){
 	}
 }
 
+/* Timer A: INVITE request retransmit interval, for UDP only */
 static int ict_on_timer_A(belle_sip_ict_t *obj){
 	belle_sip_transaction_t *base=(belle_sip_transaction_t*)obj;
 
+	if (!base->channel) return BELLE_SIP_STOP;
+
 	switch(base->state){
 		case BELLE_SIP_TRANSACTION_CALLING:
 		{
@@ -149,10 +156,11 @@ static int ict_on_timer_A(belle_sip_ict_t *obj){
 		default:
 		break;
 	}
-	
+
 	return BELLE_SIP_CONTINUE;
 }
 
+/* Timer B: INVITE transaction timeout timer */
 static int ict_on_timer_B(belle_sip_ict_t *obj){
 	belle_sip_transaction_t *base=(belle_sip_transaction_t*)obj;
 	switch (base->state){
@@ -171,7 +179,7 @@ static void ict_send_request(belle_sip_ict_t *obj){
 	const belle_sip_timer_config_t *cfg=belle_sip_transaction_get_timer_config(base);
 
 	belle_sip_transaction_set_state(base,BELLE_SIP_TRANSACTION_CALLING);
-	
+
 	if (!belle_sip_channel_is_reliable(base->channel)){
 		obj->timer_A=belle_sip_timeout_source_new((belle_sip_source_func_t)ict_on_timer_A,obj,cfg->T1);
 		belle_sip_transaction_start_timer(base,obj->timer_A);
@@ -179,7 +187,7 @@ static void ict_send_request(belle_sip_ict_t *obj){
 
 	obj->timer_B=belle_sip_timeout_source_new((belle_sip_source_func_t)ict_on_timer_B,obj,cfg->T1*64);
 	belle_sip_transaction_start_timer(base,obj->timer_B);
-	
+
 	belle_sip_channel_queue_message(base->channel,(belle_sip_message_t*)base->request);
 }
 
@@ -192,7 +200,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_ict_t)
 				BELLE_SIP_VPTR_INIT(belle_sip_ict_t,belle_sip_client_transaction_t,TRUE),
 				(belle_sip_object_destroy_t)ict_destroy,
 				NULL,
-				NULL
+				NULL,
+				BELLE_SIP_DEFAULT_BUFSIZE_HINT
 			},
 			(void (*)(belle_sip_transaction_t*))on_ict_terminate
 		},
@@ -207,4 +216,3 @@ belle_sip_ict_t *belle_sip_ict_new(belle_sip_provider_t *prov, belle_sip_request
 	belle_sip_client_transaction_init((belle_sip_client_transaction_t*)obj,prov,req);
 	return obj;
 }
-
