@@ -157,7 +157,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_nict_t)
 				BELLE_SIP_VPTR_INIT(belle_sip_nict_t,belle_sip_client_transaction_t,TRUE),
 				(belle_sip_object_destroy_t)nict_destroy,
 				NULL,
-				NULL
+				NULL,
+				BELLE_SIP_DEFAULT_BUFSIZE_HINT
 			},
 			(void (*)(belle_sip_transaction_t *))nict_on_terminate
 		},
