@@ -16,10 +16,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/** 
+/**
  * non-INVITE server transaction implementation.
 **/
 
+#include <bctoolbox/defs.h>
+
 #include "belle_sip_internal.h"
 
 static void nist_on_terminate(belle_sip_nist_t *obj){
@@ -63,7 +65,7 @@ static int nist_send_new_response(belle_sip_nist_t *obj, belle_sip_response_t *r
 				belle_sip_channel_queue_message(base->channel,(belle_sip_message_t*)resp);
 				break;
 			}
-			/* no break nist can directly pass from TRYING to PROCEEDING*/
+			BCTBX_NO_BREAK; /* no break nist can directly pass from TRYING to PROCEEDING*/
 		case BELLE_SIP_TRANSACTION_PROCEEDING:
 			if (code>=200){
 				nist_set_completed(obj);
@@ -103,7 +105,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_nist_t)
 				BELLE_SIP_VPTR_INIT(belle_sip_nist_t,belle_sip_server_transaction_t,TRUE),
 				(belle_sip_object_destroy_t)nist_destroy,
 				NULL,
-				NULL
+				NULL,
+				BELLE_SIP_DEFAULT_BUFSIZE_HINT
 			},
 			(void (*)(belle_sip_transaction_t *))nist_on_terminate
 		},
