@@ -20,8 +20,8 @@
 #include "listeningpoint_internal.h"
 #include "md5.h"
 #include "belle-sip/message.h"
-
-typedef struct authorization_context {
+static void  belle_sip_provider_update_or_create_auth_context(belle_sip_provider_t *p,belle_sip_header_call_id_t* call_id,belle_sip_header_www_authenticate_t* authenticate,belle_sip_uri_t *from_uri,const char* realm) ;
+struct authorization_context {
 	belle_sip_header_call_id_t* callid;
 	const char* scheme;
 	const char* realm;
@@ -32,7 +32,7 @@ typedef struct authorization_context {
 	const char* algorithm;
 	int nonce_count;
 	int is_proxy;
-}authorization_context_t;
+};
 
 GET_SET_STRING(authorization_context,realm)
 GET_SET_STRING(authorization_context,nonce)
@@ -49,7 +49,7 @@ static authorization_context_t* belle_sip_authorization_create(belle_sip_header_
 	belle_sip_object_ref(result->callid);
 	return result;
 }
-static void belle_sip_authorization_destroy(authorization_context_t* object) {
+void belle_sip_authorization_destroy(authorization_context_t* object) {
 	DESTROY_STRING(object,scheme);
 	DESTROY_STRING(object,realm);
 	DESTROY_STRING(object,nonce);
@@ -132,6 +132,13 @@ static void belle_sip_provider_dispatch_request(belle_sip_provider_t* prov, bell
 		/* Should we limit to ACK ?  */
 		/*Search for a dialog if exist */
 
+		if (strcmp("CANCEL",method) == 0) {
+			/* Call leg does not exist */
+			belle_sip_server_transaction_t *tr = belle_sip_provider_create_server_transaction(prov, req);
+			belle_sip_server_transaction_send_response(tr, belle_sip_response_create_from_request(req, 481));
+			return;
+		}
+
 		ev.dialog=belle_sip_provider_find_dialog_from_message(prov,(belle_sip_message_t*)req,1/*request=uas*/);
 		if (ev.dialog){
 			if (strcmp("ACK",method)==0){
@@ -139,6 +146,8 @@ static void belle_sip_provider_dispatch_request(belle_sip_provider_t* prov, bell
 					/*absorbed ACK retransmission, ignore */
 					return;
 				}
+			}else if ((strcmp("INVITE",method)==0)&&(ev.dialog->needs_ack)){
+				belle_sip_dialog_stop_200Ok_retrans(ev.dialog);
 			}else if (!belle_sip_dialog_is_authorized_transaction(ev.dialog,method)){
 				belle_sip_server_transaction_t *tr=belle_sip_provider_create_server_transaction(prov,req);
 				belle_sip_server_transaction_send_response(tr,
@@ -201,13 +210,58 @@ static void belle_sip_provider_dispatch_response(belle_sip_provider_t* p, belle_
 				}
 			}
 		}
+		break;
 	}
+		default:
+			if (t!=NULL){
+				belle_sip_message_t* req = BELLE_SIP_MESSAGE(belle_sip_transaction_get_request((belle_sip_transaction_t*)t));
+				belle_sip_header_authentication_info_t *authentication_info = belle_sip_message_get_header_by_type(msg,belle_sip_header_authentication_info_t);
+				belle_sip_list_t *authorization_lst = NULL;
+				belle_sip_header_call_id_t *call_id = belle_sip_message_get_header_by_type(msg,belle_sip_header_call_id_t);
+				belle_sip_header_from_t *from = belle_sip_message_get_header_by_type(req,belle_sip_header_from_t);
+				belle_sip_uri_t *from_uri=belle_sip_header_address_get_uri((belle_sip_header_address_t*)from);
+				/*searching for authentication headers*/
+				authorization_lst = belle_sip_list_copy(belle_sip_message_get_headers(BELLE_SIP_MESSAGE(req),BELLE_SIP_AUTHORIZATION));
+				/*search for proxy authenticate*/
+				authorization_lst=belle_sip_list_concat(authorization_lst,belle_sip_list_copy(belle_sip_message_get_headers(BELLE_SIP_MESSAGE(req),BELLE_SIP_PROXY_AUTHORIZATION)));
+				/*update auth contexts with authenticate headers from response*/
+				for (;authentication_info && authorization_lst!=NULL;authorization_lst=authorization_lst->next) {
+					belle_sip_header_authorization_t *authorization=BELLE_SIP_HEADER_AUTHORIZATION(authorization_lst->data);
+					belle_sip_header_www_authenticate_t *www_authenticate = belle_sip_auth_helper_create_www_authenticate(authorization);
+					belle_sip_header_www_authenticate_set_nonce(www_authenticate, belle_sip_header_authentication_info_get_next_nonce(authentication_info));
+					belle_sip_message( "Updating auth context for ream [%s] next nonce is going to be [%s]"
+									  , belle_sip_header_www_authenticate_get_realm(www_authenticate)
+									  , belle_sip_header_authentication_info_get_next_nonce(authentication_info));
+					belle_sip_provider_update_or_create_auth_context(p
+																	 , call_id
+																	 , www_authenticate
+																	 , from_uri
+																	 ,belle_sip_header_www_authenticate_get_realm(www_authenticate));
+					belle_sip_object_unref(www_authenticate);
+				}
+				if (authorization_lst)
+					belle_sip_list_free(authorization_lst);
+			}
+			
 	}
+	if (t){ /*In some re-connection case, specially over udp, transaction may be found, but without associated channel*/
+		if (t->base.channel == NULL) {
+			belle_sip_channel_t *chan;
+			belle_sip_message("Transaction [%p] does not have any channel associated, searching for a new one",t);
+			chan=belle_sip_provider_get_channel(p,t->next_hop); /*might be faster to get channel directly from upper level*/
+			if (chan){
+				belle_sip_object_ref(chan);
+				belle_sip_channel_add_listener(chan,BELLE_SIP_CHANNEL_LISTENER(t));
+				t->base.channel=chan;
+			}
+		}
+	}
+		
 	/*
-	 * If a transaction is found, pass it to the transaction and let it decide what to do.
+	 * If a transaction is found and have a channel, pass it to the transaction and let it decide what to do.
 	 * Else notifies directly.
 	 */
-	if (t){
+	if (t && t->base.channel){
 		/*since the add_response may indirectly terminate the transaction, we need to guarantee the transaction is not freed
 		 * until full completion*/
 		belle_sip_object_ref(t);
@@ -631,41 +685,34 @@ belle_sip_dialog_t * belle_sip_provider_create_dialog_internal(belle_sip_provide
 	return dialog;
 }
 
-/*find a dialog given the call id, from-tag and to-tag*/
-belle_sip_dialog_t* belle_sip_provider_find_dialog(const belle_sip_provider_t *prov, const char* call_id, const char* from_tag, const char* to_tag) {
+/*find a dialog given the call id, local-tag and to-tag*/
+belle_sip_dialog_t* belle_sip_provider_find_dialog(const belle_sip_provider_t *prov, const char* call_id, const char* local_tag, const char* remote_tag) {
 	belle_sip_list_t* iterator;
-
-	if (call_id == NULL || from_tag == NULL || to_tag == NULL) {
+	belle_sip_dialog_t*returned_dialog=NULL;
+	
+	if (call_id == NULL || local_tag == NULL || remote_tag == NULL) {
 		return NULL;
 	}
 	
 	for(iterator=prov->dialogs;iterator!=NULL;iterator=iterator->next) {
 		belle_sip_dialog_t* dialog=(belle_sip_dialog_t*)iterator->data;
-		if (belle_sip_dialog_get_state(dialog) != BELLE_SIP_DIALOG_NULL && strcmp(belle_sip_header_call_id_get_call_id(belle_sip_dialog_get_call_id(dialog)),call_id)==0) {
-			const char* target_from;
-			const char* target_to;
-			if (belle_sip_dialog_is_server(dialog)) {
-				target_to=belle_sip_dialog_get_local_tag(dialog);
-				target_from=belle_sip_dialog_get_remote_tag(dialog);
-			} else {
-				target_from=belle_sip_dialog_get_local_tag(dialog);
-				target_to=belle_sip_dialog_get_remote_tag(dialog);
-			}
-			if (strcmp(from_tag,target_from)==0 && strcmp(to_tag,target_to)==0) {
-				return dialog;
-			}
+		dialog=(belle_sip_dialog_t*)iterator->data;
+		/*ignore dialog in state BELLE_SIP_DIALOG_NULL, is it really the correct things to do*/
+		if (belle_sip_dialog_get_state(dialog) != BELLE_SIP_DIALOG_NULL && _belle_sip_dialog_match(dialog,call_id,local_tag,remote_tag)) {
+			if (!returned_dialog)
+				returned_dialog=dialog;
+			else
+				belle_sip_fatal("More than 1 dialog is matching, check your app");
 		}
 	}
-	return NULL;
+	return returned_dialog;
 }
 
 /*finds an existing dialog for an outgoing or incoming message */
 belle_sip_dialog_t *belle_sip_provider_find_dialog_from_message(belle_sip_provider_t *prov, belle_sip_message_t *msg, int as_uas){
-	belle_sip_dialog_t *returned_dialog=NULL,*dialog;
 	belle_sip_header_call_id_t *call_id;
 	belle_sip_header_from_t *from;
 	belle_sip_header_to_t *to;
-	belle_sip_list_t *elem;
 	const char *from_tag;
 	const char *to_tag;
 	const char *call_id_value;
@@ -692,18 +739,7 @@ belle_sip_dialog_t *belle_sip_provider_find_dialog_from_message(belle_sip_provid
 	call_id_value=belle_sip_header_call_id_get_call_id(call_id);
 	local_tag=as_uas ? to_tag : from_tag;
 	remote_tag=as_uas ? from_tag : to_tag;
-
-	for (elem=prov->dialogs;elem!=NULL;elem=elem->next){
-		dialog=(belle_sip_dialog_t*)elem->data;
-		/*ignore dialog in state BELLE_SIP_DIALOG_NULL, is it really the correct things to do*/
-		if (belle_sip_dialog_get_state(dialog) != BELLE_SIP_DIALOG_NULL && _belle_sip_dialog_match(dialog,call_id_value,local_tag,remote_tag)) {
-			if (!returned_dialog)
-				returned_dialog=dialog;
-			else
-				belle_sip_fatal("More than 1 dialog is matching, check your app");
-		}
-	}
-	return returned_dialog;
+	return belle_sip_provider_find_dialog(prov,call_id_value,local_tag,remote_tag);
 }
 
 void belle_sip_provider_add_dialog(belle_sip_provider_t *prov, belle_sip_dialog_t *dialog){
@@ -797,7 +833,10 @@ belle_sip_channel_t * belle_sip_provider_get_channel(belle_sip_provider_t *p, co
 			lp=(belle_sip_listening_point_t*)l->data;
 			if (strcasecmp(belle_sip_listening_point_get_transport(lp),hop->transport)==0){
 				chan=belle_sip_listening_point_get_channel(lp,hop);
-				if (chan) return chan;
+				if (chan) {
+					belle_sip_channel_check_dns_reusability(chan);
+					return chan;
+				}
 				candidate=lp;
 			}
 		}
@@ -1009,6 +1048,7 @@ static void authorization_context_fill_from_auth(authorization_context_t* auth_c
 	}
 }
 
+
 static belle_sip_list_t* belle_sip_provider_get_auth_context_by_realm_or_call_id(belle_sip_provider_t *p,belle_sip_header_call_id_t* call_id,belle_sip_uri_t *from_uri,const char* realm) {
 	belle_sip_list_t* auth_context_lst=NULL;
 	belle_sip_list_t* result=NULL;
@@ -1085,6 +1125,7 @@ static void  belle_sip_provider_update_or_create_auth_context(belle_sip_provider
 	return;
 }
 
+
 int belle_sip_provider_add_authorization(belle_sip_provider_t *p, belle_sip_request_t* request, belle_sip_response_t *resp,
 					 belle_sip_uri_t *from_uri, belle_sip_list_t** auth_infos, const char* realm) {
 	belle_sip_header_call_id_t* call_id;
