@@ -16,6 +16,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <bctoolbox/defs.h>
+
 #include "belle_sip_internal.h"
 #include "belle-sip/refresher.h"
 
@@ -69,7 +71,8 @@ static void cancel_retry(belle_sip_refresher_t* refresher) {
 	}
 }
 static void schedule_timer_at(belle_sip_refresher_t* refresher,int delay, timer_purpose_t purpose) {
-	belle_sip_message("Refresher: scheduling next timer in %i ms",delay);
+	belle_sip_message("Refresher[%p]: scheduling next timer in %i ms for purpose [%s]",refresher, delay,
+		purpose == NORMAL_REFRESH ? "normal refresh" : "retry");
 	refresher->timer_purpose=purpose;
 	/*cancel timer if any*/
 	cancel_retry(refresher);
@@ -104,11 +107,15 @@ static void process_dialog_terminated(belle_sip_listener_t *user_ctx, const bell
 	if (refresher && refresher->transaction && dialog != belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(refresher->transaction)))
 		return; /*not for me*/
 
-	if (refresher->state == started) {
-		belle_sip_warning("Refresher [%p] still started but receiving unexpected dialog deleted event on [%p], retrying",refresher,dialog);
-		retry_later_on_io_error(refresher);
-		if (refresher->listener) refresher->listener(refresher,refresher->user_data,481, "dialod terminated");
+	if (belle_sip_dialog_expired(dialog) && refresher->state == started) {
+		/* We notify the app that the dialog is expired due to failure to refresh the subscription on time.
+		 * However the transaction to renew the dialog is either pending or already failed, and has scheduled a retry already,
+		 * so there is no need to reschedule a retry here.*/
+		belle_sip_warning("Refresher [%p] still started but expired, retrying",refresher);
+		if (refresher->listener) refresher->listener(refresher,refresher->user_data,481, "dialod terminated", TRUE);
+
 	}
+
 }
 
 static void process_io_error(belle_sip_listener_t *user_ctx, const belle_sip_io_error_event_t *event){
@@ -131,7 +138,7 @@ static void process_io_error(belle_sip_listener_t *user_ctx, const belle_sip_io_
 			return; /*not for me or no longuer involved because expire=0*/
 		}
 		if (refresher->state==started) retry_later_on_io_error(refresher);
-		if (refresher->listener) refresher->listener(refresher,refresher->user_data,503, "io error");
+		if (refresher->listener) refresher->listener(refresher,refresher->user_data,503, "io error", refresher->state == started);
 
 		return;
 	} else if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(belle_sip_io_error_event_get_source(event),belle_sip_provider_t)) {
@@ -146,7 +153,7 @@ static void process_io_error(belle_sip_listener_t *user_ctx, const belle_sip_io_
 								,refresher->transaction->base.channel
 								,belle_sip_channel_state_to_string(belle_sip_channel_get_state(refresher->transaction->base.channel)));
 			if (refresher->state==started) retry_later_on_io_error(refresher);
-			if (refresher->listener) refresher->listener(refresher,refresher->user_data,503, "io error");
+			if (refresher->listener) refresher->listener(refresher,refresher->user_data,503, "io error", refresher->state == started);
 			refresher->on_io_error=1;
 		}
 		return;
@@ -206,6 +213,7 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 	int response_code = belle_sip_response_get_status_code(response);
 	belle_sip_refresher_t* refresher=(belle_sip_refresher_t*)user_ctx;
 	belle_sip_header_contact_t *contact;
+	int will_retry = TRUE; /*most error codes are retryable*/
 
 
 	if (refresher && (client_transaction !=refresher->transaction))
@@ -252,7 +260,7 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 		if (refresher->state==started) {
 			if (!refresher->first_acknoleged_request)
 				belle_sip_object_ref(refresher->first_acknoleged_request = request);
-			if (is_contact_address_acurate(refresher,request)) {
+			if (is_contact_address_acurate(refresher,request) || !belle_sip_provider_nat_helper_enabled(client_transaction->base.provider)) {
 				schedule_timer(refresher); /*re-arm timer*/
 			} else {
 				belle_sip_message("belle_sip_refresher_start(): refresher [%p] is resubmitting request because contact sent was not correct in original request.",refresher);
@@ -261,7 +269,7 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 			}
 		}
 		else belle_sip_message("Refresher [%p] not scheduling next refresh, because it was stopped",refresher);
-	}else{/*special error cases*/
+	}else if (response_code >= 300) {/*special error cases*/
 		switch (response_code) {
 		case 301:
 		case 302:
@@ -291,6 +299,7 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 		case 403:
 			/*In case of 403, we will retry later, just in case*/
 			if (refresher->target_expires>0) retry_later(refresher);
+			else will_retry = FALSE;
 			break;
 		case 412:
 			if (strcmp(belle_sip_request_get_method(request),"PUBLISH")==0) {
@@ -298,9 +307,12 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 				if (refresher->target_expires>0) {
 					retry_later_on_io_error(refresher);
 					return; /*do not notify this kind of error*/
+				}else{
+					will_retry = FALSE;
 				}
 			} else {
 				if (refresher->target_expires>0) retry_later(refresher);
+				else will_retry = FALSE;
 			}
 			break;
 		case 423:{
@@ -316,30 +328,37 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 					}
 				}
 			}else belle_sip_warning("Receiving 423 but no min-expires header.");
+			will_retry = FALSE;
 			break;
 		}
-		case 481: {
-			/* will trigger dialog terminated, so nothing to do here*/
-			return;
-		}
 		case 491: {
 			if (refresher->target_expires>0) {
-				retry_later_on_io_error(refresher);
+				int delay = belle_sip_random() % 10000; /*schedule a retry between 0 and 10 seconds*/
+				schedule_timer_at(refresher, delay, RETRY);
 				return; /*do not notify this kind of error*/
 			}
 		}
+		BCTBX_NO_BREAK; /*intentionally no break*/
 		case 505:
 		case 501:
 			/*irrecoverable errors, probably no need to retry later*/
+			will_retry = FALSE;
+			break;
+		case 481:
+		case 503:
+			if (refresher->target_expires>0) {
+				if (refresher->dialog) retry_later_on_io_error(refresher);
+				else retry_later(refresher);
+			}else will_retry = FALSE;
 			break;
-
 		default:
-			/*for all other errors, retry later*/
-			if (refresher->target_expires>0) retry_later(refresher);
+			/*for all other errors <600, retry later*/
+			if (response_code < 600 && refresher->target_expires>0) retry_later(refresher);
+			else will_retry = FALSE;
 			break;
 		}
 	}
-	if (refresher->listener) refresher->listener(refresher,refresher->user_data,response_code, belle_sip_response_get_reason_phrase(response));
+	if (refresher->listener) refresher->listener(refresher,refresher->user_data,response_code, belle_sip_response_get_reason_phrase(response), will_retry);
 
 }
 
@@ -354,7 +373,7 @@ static void process_timeout(belle_sip_listener_t *user_ctx, const belle_sip_time
 		/*retry in 2 seconds but not immediately to let the current transaction be cleaned*/
 		schedule_timer_at(refresher,2000,RETRY);
 	}
-	if (refresher->listener) refresher->listener(refresher,refresher->user_data,408, "timeout");
+	if (refresher->listener) refresher->listener(refresher,refresher->user_data,408, "timeout", refresher->state==started);
 }
 
 static void process_transaction_terminated(belle_sip_listener_t *user_ctx, const belle_sip_transaction_terminated_event_t *event) {
@@ -509,18 +528,16 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 			}
 			case BELLE_SIP_DIALOG_TERMINATED: {
 				if (refresher->first_acknoleged_request) {
-					char tmp[11];
 					belle_sip_message("Dialog [%p] is in state terminated, recreating a new one for refresher [%p]",dialog,refresher);
 					request = refresher->first_acknoleged_request;
 					belle_sip_header_cseq_set_seq_number(belle_sip_message_get_header_by_type(request,belle_sip_header_cseq_t)
 														 ,20);
 					belle_sip_parameters_remove_parameter(BELLE_SIP_PARAMETERS(belle_sip_message_get_header_by_type(request,belle_sip_header_to_t)),"tag");
 
-					belle_sip_header_call_id_set_call_id(	  belle_sip_message_get_header_by_type(request,belle_sip_header_call_id_t)
-															, belle_sip_random_token(tmp,sizeof(tmp)));
+					belle_sip_message_set_header((belle_sip_message_t*)request, (belle_sip_header_t*)belle_sip_provider_create_call_id(prov));
 					break;
-				} /*else nop, error case*/
-
+				}
+				BCTBX_NO_BREAK; /*else nop, error case*/
 			}
 			default: {
 				belle_sip_error("Unexpected dialog state [%s] for dialog [%p], cannot refresh [%s]"
@@ -559,7 +576,7 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 
 	client_transaction = belle_sip_provider_create_client_transaction(prov,request);
 	client_transaction->base.is_internal=1;
-	
+
 	if (request ==  refresher->first_acknoleged_request) { /*request is now ref by transaction so no need to keepo it*/
 		belle_sip_object_unref(refresher->first_acknoleged_request);
 		refresher->first_acknoleged_request = NULL;
@@ -597,7 +614,7 @@ static int timer_cb(void *user_data, unsigned int events) {
 	if (refresher->timer_purpose==NORMAL_REFRESH && refresher->manual) {
 		belle_sip_message("Refresher [%p] is in manual mode, skipping refresh.",refresher);
 		/*call listener with special code 0 to indicate request is about to expire*/
-		if (refresher->listener) refresher->listener(refresher,refresher->user_data,0, "about to expire");
+		if (refresher->listener) refresher->listener(refresher,refresher->user_data,0, "about to expire", FALSE);
 		return BELLE_SIP_STOP;
 	}
 
@@ -820,9 +837,18 @@ void belle_sip_refresher_enable_manual_mode(belle_sip_refresher_t *refresher, in
 	refresher->manual=enabled;
 }
 
-char *belle_sip_refresher_get_public_uri(belle_sip_refresher_t* refresher)  {
-	return belle_sip_channel_get_public_ip_port(refresher->transaction->base.channel);
+const char * belle_sip_refresher_get_local_address(belle_sip_refresher_t* refresher, int *port){
+	belle_sip_channel_t *chan = refresher->transaction->base.channel;
+	if (chan) return belle_sip_channel_get_local_address(chan, port);
+	return NULL;
 }
+
+const char * belle_sip_refresher_get_public_address(belle_sip_refresher_t* refresher, int *port){
+	belle_sip_channel_t *chan = refresher->transaction->base.channel;
+	if (chan) return belle_sip_channel_get_public_address(chan, port);
+	return NULL;
+}
+
 static void set_or_update_dialog(belle_sip_refresher_t* refresher, belle_sip_dialog_t* dialog) {
 	if (refresher->dialog!=dialog){
 		belle_sip_message("refresher [%p] : set_or_update_dialog() current=[%p] new=[%p]",refresher,refresher->dialog,dialog);
