@@ -55,12 +55,40 @@ BELLESIP_EXPORT const char *belle_sip_transaction_get_method(const belle_sip_tra
 	return belle_sip_request_get_method(t->request);
 }
 
+static void transaction_end_background_task(belle_sip_transaction_t *obj){
+	if (obj->bg_task_id){
+		belle_sip_message("transaction [%p]: ending transaction background task with id=[%lx].",obj,obj->bg_task_id);
+		belle_sip_end_background_task(obj->bg_task_id);
+		obj->bg_task_id=0;
+	}
+}
+
+static void transaction_background_task_ended(belle_sip_transaction_t *obj){
+	belle_sip_warning("transaction [%p]: transaction background task has to be ended now, but work isn't finished.",obj);
+	transaction_end_background_task(obj);
+}
+
+static void transaction_begin_background_task(belle_sip_transaction_t *obj){
+	if (obj->bg_task_id==0){
+		char *transaction = bctbx_strdup_printf("belle-sip transaction(%p)", obj);
+        obj->bg_task_id=belle_sip_begin_background_task(transaction,(void (*)(void*))transaction_background_task_ended, obj);
+        if (obj->bg_task_id) belle_sip_message("transaction [%p]: starting transaction background task with id=[%lx].",obj,obj->bg_task_id);
+        bctbx_free(transaction);
+	}
+}
+
 static void belle_sip_transaction_init(belle_sip_transaction_t *t, belle_sip_provider_t *prov, belle_sip_request_t *req){
+	transaction_begin_background_task(t);
 	t->request=(belle_sip_request_t*)belle_sip_object_ref(req);
 	t->provider=prov;
 }
 
 static void transaction_destroy(belle_sip_transaction_t *t){
+	if (t->call_repair_timer) {
+		belle_sip_transaction_stop_timer(t, t->call_repair_timer);
+		belle_sip_object_unref(t->call_repair_timer);
+		t->call_repair_timer = NULL;
+	}
 	if (t->request) belle_sip_object_unref(t->request);
 	if (t->last_response) belle_sip_object_unref(t->last_response);
 	if (t->channel) belle_sip_object_unref(t->channel);
@@ -78,9 +106,20 @@ static void notify_timeout(belle_sip_transaction_t *t){
 	BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_TRANSACTION(t,process_timeout,&ev);
 }
 
+static int server_transaction_on_call_repair_timer(belle_sip_transaction_t *t) {
+	belle_sip_server_transaction_send_response(BELLE_SIP_SERVER_TRANSACTION(t), belle_sip_response_create_from_request(t->request, 503));
+	return BELLE_SIP_STOP;
+}
+
+static int client_transaction_on_call_repair_timer(belle_sip_transaction_t *t) {
+	belle_sip_transaction_terminate(t);
+	return BELLE_SIP_STOP;
+}
+
 static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_channel_t *chan, belle_sip_channel_state_t state){
 	belle_sip_transaction_t *t=(belle_sip_transaction_t*)l;
 	belle_sip_io_error_event_t ev;
+	const belle_sip_timer_config_t *timercfg = NULL;
 	belle_sip_transaction_state_t tr_state=belle_sip_transaction_get_state((belle_sip_transaction_t*)t);
 
 	belle_sip_message("transaction [%p] channel state changed to [%s]"
@@ -106,11 +145,35 @@ static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_
 				&& tr_state!=BELLE_SIP_TRANSACTION_TERMINATED) {
 				BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_TRANSACTION(((belle_sip_transaction_t*)t),process_io_error,&ev);
 			}
-			if (t->timed_out)
+			if (t->timed_out) {
 				notify_timeout((belle_sip_transaction_t*)t);
+			} else {
+				if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ist_t) || BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ict_t)) {
+					timercfg = belle_sip_transaction_get_timer_config(t);
+					if (t->call_repair_timer) {
+						belle_sip_transaction_stop_timer(t, t->call_repair_timer);
+						belle_sip_object_unref(t->call_repair_timer);
+						t->call_repair_timer = NULL;
+					}
+				}
+			}
 
-			belle_sip_transaction_terminate(t);
+			if (!t->timed_out && belle_sip_transaction_state_is_transient(t->state) && BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ist_t)) {
+				t->call_repair_timer = belle_sip_timeout_source_new((belle_sip_source_func_t)server_transaction_on_call_repair_timer, t, 32 * timercfg->T1);
+				belle_sip_transaction_start_timer(t, t->call_repair_timer);
+			} else if (!t->timed_out && belle_sip_transaction_state_is_transient(t->state) && BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ict_t)) {
+				t->call_repair_timer = belle_sip_timeout_source_new((belle_sip_source_func_t)client_transaction_on_call_repair_timer, t, 32 * timercfg->T1);
+				belle_sip_transaction_start_timer(t, t->call_repair_timer);
+			} else {
+				belle_sip_transaction_terminate(t);
+			}
+			if (t->channel){
+				belle_sip_channel_remove_listener(t->channel, l);
+				belle_sip_object_unref(t->channel);
+				t->channel = NULL;
+			}
 			belle_sip_object_unref(t);
+
 		break;
 		default:
 			/*ignored*/
@@ -120,8 +183,10 @@ static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_
 
 BELLE_SIP_IMPLEMENT_INTERFACE_BEGIN(belle_sip_transaction_t,belle_sip_channel_listener_t)
 on_channel_state_changed,
-NULL,
-NULL
+NULL, /* on_message_headers */
+NULL, /* on_message */
+NULL, /* on_sending */
+NULL /* on_auth_requested */
 BELLE_SIP_IMPLEMENT_INTERFACE_END
 
 BELLE_SIP_DECLARE_IMPLEMENTED_INTERFACES_1(belle_sip_transaction_t, belle_sip_channel_listener_t);
@@ -132,6 +197,7 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_transaction_t)
 		(belle_sip_object_destroy_t) transaction_destroy,
 		NULL,/*no clone*/
 		NULL,/*no marshal*/
+		BELLE_SIP_DEFAULT_BUFSIZE_HINT
 	},
 	NULL /*on_terminate*/
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
@@ -174,6 +240,12 @@ int belle_sip_transaction_state_is_transient(const belle_sip_transaction_state_t
 }
 
 void belle_sip_transaction_terminate(belle_sip_transaction_t *t){
+	belle_sip_object_ref(t);
+	if (t->call_repair_timer) {
+		belle_sip_transaction_stop_timer(t, t->call_repair_timer);
+		belle_sip_object_unref(t->call_repair_timer);
+		t->call_repair_timer = NULL;
+	}
 	if (belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(t))!=BELLE_SIP_TRANSACTION_TERMINATED) {
 		int is_client=BELLE_SIP_OBJECT_IS_INSTANCE_OF(t,belle_sip_client_transaction_t);
 		belle_sip_transaction_set_state(t,BELLE_SIP_TRANSACTION_TERMINATED);
@@ -188,6 +260,8 @@ void belle_sip_transaction_terminate(belle_sip_transaction_t *t){
 		BELLE_SIP_OBJECT_VPTR(t,belle_sip_transaction_t)->on_terminate(t);
 		belle_sip_provider_set_transaction_terminated(t->provider,t);
 	}
+	transaction_end_background_task(t);
+	belle_sip_object_unref(t);
 }
 
 belle_sip_request_t *belle_sip_transaction_get_request(const belle_sip_transaction_t *t){
@@ -205,7 +279,7 @@ void belle_sip_transaction_notify_timeout(belle_sip_transaction_t *t){
 	belle_sip_object_ref(t);  /*take a ref in the case where the app calls belle_sip_transaction_terminate() within the timeout listener*/
 	if (strcmp(belle_sip_request_get_method(t->request),"REGISTER")==0){
 		if ( belle_sip_channel_notify_timeout(t->channel)==TRUE){
-			belle_sip_warning("Transaction [%p] reporting timeout, reporting to channel.",t);
+			belle_sip_warning("Transaction [%p] has timeout, reported to channel.",t);
 			t->timed_out=TRUE;
 		}
 	}else {
@@ -246,10 +320,13 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_server_transaction_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_server_transaction_t,belle_sip_transaction_t,FALSE),
 			(belle_sip_object_destroy_t) server_transaction_destroy,
 			NULL,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
 		NULL
-	}
+	},
+	NULL, /* send_new_response */
+	NULL /* on_request_retransmission */
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
 void belle_sip_server_transaction_init(belle_sip_server_transaction_t *t, belle_sip_provider_t *prov,belle_sip_request_t *req){
@@ -272,7 +349,7 @@ void belle_sip_server_transaction_send_response(belle_sip_server_transaction_t *
 	int status_code;
 
 	belle_sip_object_ref(resp);
-	if (!base->last_response){
+	if (!base->last_response || !base->channel){
 		belle_sip_hop_t* hop=belle_sip_response_get_return_hop(resp);
 		base->channel=belle_sip_provider_get_channel(base->provider,hop);
 		belle_sip_object_unref(hop);
@@ -353,8 +430,8 @@ belle_sip_request_t * belle_sip_client_transaction_create_cancel(belle_sip_clien
 		belle_sip_error("belle_sip_client_transaction_create_cancel() cannot be used for ACK or non-INVITE transactions.");
 		return NULL;
 	}
-	if (t->base.state!=BELLE_SIP_TRANSACTION_PROCEEDING){
-		belle_sip_error("belle_sip_client_transaction_create_cancel() can only be used in state BELLE_SIP_TRANSACTION_PROCEEDING"
+	if ((t->base.state != BELLE_SIP_TRANSACTION_PROCEEDING) && (t->base.state != BELLE_SIP_TRANSACTION_CALLING)) {
+		belle_sip_error("belle_sip_client_transaction_create_cancel() can only be used in state PROCEEDING or CALLING"
 		               " but current transaction state is %s",belle_sip_transaction_state_to_string(t->base.state));
 		return NULL;
 	}
@@ -414,13 +491,13 @@ int belle_sip_client_transaction_send_request_to(belle_sip_client_transaction_t
 	}
 
 	if (t->base.sent_by_dialog_queue){
-		
+
 		/*it can be sent immediately, so update the request with latest cseq and route_set */
 		/*update route and contact just in case they changed*/
 		belle_sip_dialog_update_request(dialog,req);
 	} else if (t->base.request->dialog_queued){
 		/*this request was created by belle_sip_dialog_create_queued_request().*/
-		
+
 		if (dialog == NULL){
 			belle_sip_error("belle_sip_client_transaction_send_request(): transaction [%p], cannot send"
 				" request because it was created in the context of a dialog that appears to be "
@@ -428,7 +505,7 @@ int belle_sip_client_transaction_send_request_to(belle_sip_client_transaction_t
 			belle_sip_transaction_terminate(BELLE_SIP_TRANSACTION(t));
 			return -1;
 		}
-		
+
 		if (belle_sip_dialog_request_pending(dialog) || dialog->queued_ct!=NULL){
 			/*it cannot be sent immediately, queue the transaction into dialog*/
 			belle_sip_message("belle_sip_client_transaction_send_request(): transaction [%p], cannot send request now because dialog [%p] is busy"
@@ -555,7 +632,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_client_transaction_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_client_transaction_t,belle_sip_transaction_t,FALSE),
 			(belle_sip_object_destroy_t)client_transaction_destroy,
 			NULL,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
 		NULL
 	},
@@ -618,8 +696,8 @@ belle_sip_request_t* belle_sip_client_transaction_create_authenticated_request(b
  contains a "Subscription-State" of "active" or "pending", it creates
  a new subscription and a new dialog (unless they have already been
  created by a matching response, as described above).
- 
- 
+
+
  */
 
 int belle_sip_client_transaction_is_notify_matching_pending_subscribe(belle_sip_client_transaction_t *trans
@@ -628,32 +706,32 @@ int belle_sip_client_transaction_is_notify_matching_pending_subscribe(belle_sip_
 	belle_sip_header_event_t *sub_event, *notif_event;
 	belle_sip_header_call_id_t *sub_call_id, *notif_call_id;
 	const char* sub_from_tag, *notif_to_tag;
-	
+
 	if (!belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(trans)))
 		|| strcmp("SUBSCRIBE", belle_sip_transaction_get_method(BELLE_SIP_TRANSACTION(trans)))!=0) return 0;
-	
-	
+
+
 	if (strcmp("NOTIFY",belle_sip_request_get_method(notify)) != 0) {
 		belle_sip_error("belle_sip_client_transaction_is_notify_matching_pending_subscribe for dialog [%p], requires a notify request",notify);
 	}
-	
+
 	subscription = belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(trans));
 	sub_event = belle_sip_message_get_header_by_type(subscription, belle_sip_header_event_t);
 	if (!sub_event || !belle_sip_header_event_get_package_name(sub_event))
 		return 0;
-	
+
 	notif_event = belle_sip_message_get_header_by_type(notify, belle_sip_header_event_t);
 	if (!notif_event || !belle_sip_header_event_get_package_name(notif_event))
 		return 0;
-	
+
 	sub_call_id = belle_sip_message_get_header_by_type(subscription, belle_sip_header_call_id_t);
 	notif_call_id = belle_sip_message_get_header_by_type(notify, belle_sip_header_call_id_t);
 	sub_from_tag = belle_sip_header_from_get_tag(belle_sip_message_get_header_by_type(subscription, belle_sip_header_from_t));
 	notif_to_tag = belle_sip_header_to_get_tag(belle_sip_message_get_header_by_type(notify, belle_sip_header_to_t));
-	
+
 	return strcmp(belle_sip_header_call_id_get_call_id(sub_call_id),belle_sip_header_call_id_get_call_id(notif_call_id))==0
 	&& sub_from_tag && notif_to_tag && strcmp(sub_from_tag,notif_to_tag)==0
 	&& strcasecmp(belle_sip_header_event_get_package_name(sub_event),belle_sip_header_event_get_package_name(notif_event))==0;
-	
+
 }
 
