@@ -17,13 +17,12 @@
 */
 #include "belle_sip_internal.h"
 
-#ifdef ENABLE_SERVER_SOCKETS
 static int on_new_connection(void *userdata, unsigned int events);
 
 
 void belle_sip_stream_listening_point_destroy_server_socket(belle_sip_stream_listening_point_t *lp){
 	if (lp->server_sock!=(belle_sip_socket_t)-1){
-		close_socket(lp->server_sock);
+		belle_sip_close_socket(lp->server_sock);
 		lp->server_sock=-1;
 	}
 	if (lp->source){
@@ -32,12 +31,9 @@ void belle_sip_stream_listening_point_destroy_server_socket(belle_sip_stream_lis
 		lp->source=NULL;
 	}
 }
-#endif /* ENABLE_SERVER_SOCKETS */
 
 static void belle_sip_stream_listening_point_uninit(belle_sip_stream_listening_point_t *lp){
-#ifdef ENABLE_SERVER_SOCKETS
 	belle_sip_stream_listening_point_destroy_server_socket(lp);
-#endif /* ENABLE_SERVER_SOCKETS */
 }
 
 static belle_sip_channel_t *stream_create_channel(belle_sip_listening_point_t *lp, const belle_sip_hop_t *hop){
@@ -55,14 +51,15 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_stream_listening_point_t)
 			BELLE_SIP_VPTR_INIT(belle_sip_stream_listening_point_t, belle_sip_listening_point_t,TRUE),
 			(belle_sip_object_destroy_t)belle_sip_stream_listening_point_uninit,
 			NULL,
-			NULL
+			NULL,
+			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
 		"TCP",
 		stream_create_channel
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
-#ifdef ENABLE_SERVER_SOCKETS
+
 static belle_sip_socket_t create_server_socket(const char *addr, int * port, int *family){
 	struct addrinfo hints={0};
 	struct addrinfo *res=NULL;
@@ -73,6 +70,8 @@ static belle_sip_socket_t create_server_socket(const char *addr, int * port, int
 	
 	if (*port==-1) *port=0; /*random port for bind()*/
 
+	belle_sip_set_socket_api(NULL);
+
 	snprintf(portnum,sizeof(portnum),"%i",*port);
 	hints.ai_family=AF_UNSPEC;
 	hints.ai_socktype=SOCK_STREAM;
@@ -84,13 +83,13 @@ static belle_sip_socket_t create_server_socket(const char *addr, int * port, int
 		return -1;
 	}
 	*family=res->ai_family;
-	sock=socket(res->ai_family,res->ai_socktype,res->ai_protocol);
+	sock=bctbx_socket(res->ai_family,res->ai_socktype,res->ai_protocol);
 	if (sock==(belle_sip_socket_t)-1){
 		belle_sip_error("Cannot create TCP socket: %s",belle_sip_get_socket_error_string());
 		freeaddrinfo(res);
 		return -1;
 	}
-	err = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
+	err = bctbx_setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
 			(char*)&optval, sizeof (optval));
 	if (err == -1){
 		belle_sip_warning ("Fail to set SIP/TCP address reusable: %s.", belle_sip_get_socket_error_string());
@@ -102,7 +101,7 @@ static belle_sip_socket_t create_server_socket(const char *addr, int * port, int
 	err=bctbx_bind(sock,res->ai_addr,(socklen_t)res->ai_addrlen);
 	if (err==-1){
 		belle_sip_error("TCP bind() failed for %s port %i: %s",addr,*port,belle_sip_get_socket_error_string());
-		close_socket(sock);
+		belle_sip_close_socket(sock);
 		freeaddrinfo(res);
 		return -1;
 	}
@@ -111,20 +110,20 @@ static belle_sip_socket_t create_server_socket(const char *addr, int * port, int
 	if (*port==0){
 		struct sockaddr_storage saddr;
 		socklen_t saddr_len=sizeof(saddr);
-		err=getsockname(sock,(struct sockaddr*)&saddr,&saddr_len);
+		err=bctbx_getsockname(sock,(struct sockaddr*)&saddr,&saddr_len);
 		if (err==0){
-			err=getnameinfo((struct sockaddr*)&saddr,saddr_len,NULL,0,portnum,sizeof(portnum),NI_NUMERICSERV|NI_NUMERICHOST);
+			err=bctbx_getnameinfo((struct sockaddr*)&saddr,saddr_len,NULL,0,portnum,sizeof(portnum),NI_NUMERICSERV|NI_NUMERICHOST);
 			if (err==0){
 				*port=atoi(portnum);
 				belle_sip_message("Random TCP port is %i",*port);
 			}else belle_sip_error("TCP bind failed, getnameinfo(): %s",gai_strerror(err));
-		}else belle_sip_error("TCP bind failed, getsockname(): %s",belle_sip_get_socket_error_string());
+		}else belle_sip_error("TCP bind failed, bctbx_getsockname(): %s",belle_sip_get_socket_error_string());
 	}
 	
 	err=listen(sock,64);
 	if (err==-1){
 		belle_sip_error("TCP listen() failed for %s port %i: %s",addr,*port,belle_sip_get_socket_error_string());
-		close_socket(sock);
+		belle_sip_close_socket(sock);
 		return -1;
 	}
 	return sock;
@@ -165,28 +164,19 @@ static int on_new_connection(void *userdata, unsigned int events){
 
 void belle_sip_stream_listening_point_init(belle_sip_stream_listening_point_t *obj, belle_sip_stack_t *s, const char *ipaddress, int port, belle_sip_source_func_t on_new_connection_cb ){
 	belle_sip_listening_point_init((belle_sip_listening_point_t*)obj,s,ipaddress,port);
-	belle_sip_stream_listening_point_setup_server_socket(obj, on_new_connection_cb);
+	if (port != BELLE_SIP_LISTENING_POINT_DONT_BIND) belle_sip_stream_listening_point_setup_server_socket(obj, on_new_connection_cb);
 }
 
-#else
 
-void belle_sip_stream_listening_point_init(belle_sip_stream_listening_point_t *obj, belle_sip_stack_t *s, const char *ipaddress, int port ){
-	belle_sip_listening_point_init((belle_sip_listening_point_t*)obj,s,ipaddress,port);
-}
-
-#endif /* ENABLE_SERVER_SOCKETS */
 
 belle_sip_listening_point_t * belle_sip_stream_listening_point_new(belle_sip_stack_t *s, const char *ipaddress, int port){
 	belle_sip_stream_listening_point_t *lp=belle_sip_object_new(belle_sip_stream_listening_point_t);
-#ifdef ENABLE_SERVER_SOCKETS
+
 	belle_sip_stream_listening_point_init(lp,s,ipaddress,port,on_new_connection);
-	if (lp->server_sock==(belle_sip_socket_t)-1){
+	if (port != BELLE_SIP_LISTENING_POINT_DONT_BIND &&  lp->server_sock==(belle_sip_socket_t)-1){
 		belle_sip_object_unref(lp);
 		return NULL;
 	}
-#else
-	belle_sip_stream_listening_point_init(lp,s,ipaddress,port);
-#endif /* ENABLE_SERVER_SOCKETS */
 	return BELLE_SIP_LISTENING_POINT(lp);
 }
 
