@@ -21,6 +21,9 @@
 
 #include "bctoolbox/crypto.h"
 
+static int belle_sip_tls_channel_init_bctbx_ssl(belle_sip_tls_channel_t *obj);
+static void belle_sip_tls_channel_deinit_bctbx_ssl(belle_sip_tls_channel_t *obj);
+
 /*****************************************************************************/
 /***               signing key structure and methods                       ***/
 /*****************************************************************************/
@@ -350,11 +353,11 @@ struct belle_sip_tls_channel{
 
 static void tls_channel_close(belle_sip_tls_channel_t *obj){
 	belle_sip_socket_t sock = belle_sip_source_get_socket((belle_sip_source_t*)obj);
-	if (sock!=-1 && belle_sip_channel_get_state((belle_sip_channel_t*)obj)!=BELLE_SIP_CHANNEL_ERROR) {
-		bctbx_ssl_close_notify(obj->sslctx);
+	if (sock!=-1 && belle_sip_channel_get_state((belle_sip_channel_t*)obj)!=BELLE_SIP_CHANNEL_ERROR && !obj->base.base.closed_by_remote) {
+		if (obj->sslctx) bctbx_ssl_close_notify(obj->sslctx);
 	}
 	stream_channel_close((belle_sip_stream_channel_t*)obj);
-	bctbx_ssl_session_reset(obj->sslctx);
+	belle_sip_tls_channel_deinit_bctbx_ssl(obj);
 	obj->socket_connected=0;
 }
 
@@ -362,15 +365,7 @@ static void tls_channel_uninit(belle_sip_tls_channel_t *obj){
 	belle_sip_socket_t sock = belle_sip_source_get_socket((belle_sip_source_t*)obj);
 	if (sock!=(belle_sip_socket_t)-1)
 		tls_channel_close(obj);
-	if (obj->sslctx) {
-		bctbx_ssl_context_free(obj->sslctx);
-	}
-	if (obj->sslcfg) {
-		bctbx_ssl_config_free(obj->sslcfg);
-	}
-	if (obj->root_ca) {
-		bctbx_x509_certificate_free(obj->root_ca);
-	}
+	belle_sip_tls_channel_deinit_bctbx_ssl(obj);
 
 	if (obj->cur_debug_msg)
 		belle_sip_free(obj->cur_debug_msg);
@@ -407,6 +402,9 @@ static int tls_channel_recv(belle_sip_channel_t *obj, void *buf, size_t buflen){
 
 static int tls_channel_connect_to(belle_sip_channel_t *obj, const struct addrinfo *ai){
 	int err;
+	
+	if (belle_sip_tls_channel_init_bctbx_ssl((belle_sip_tls_channel_t*)obj)==-1) return -1;
+	
 	err= stream_channel_connect((belle_sip_stream_channel_t*)obj,ai);
 	if (err==0){
 		belle_sip_source_set_notify((belle_sip_source_t *)obj, (belle_sip_source_func_t)tls_process_data);
@@ -415,7 +413,7 @@ static int tls_channel_connect_to(belle_sip_channel_t *obj, const struct addrinf
 	return -1;
 }
 
-static void http_proxy_res_done(void *data, const char *name, struct addrinfo *ai_list){
+static void http_proxy_res_done(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl){
 	belle_sip_tls_channel_t *obj=(belle_sip_tls_channel_t*)data;
 	if (obj->http_proxy_resolver_ctx){
 		belle_sip_object_unref(obj->http_proxy_resolver_ctx);
@@ -455,7 +453,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_tls_channel_t)
 				BELLE_SIP_VPTR_INIT(belle_sip_tls_channel_t,belle_sip_stream_channel_t,FALSE),
 				(belle_sip_object_destroy_t)tls_channel_uninit,
 				NULL,
-				NULL
+				NULL,
+				BELLE_SIP_DEFAULT_BUFSIZE_HINT
 			},
 			"TLS",
 			1, /*is_reliable*/
@@ -471,7 +470,7 @@ static int belle_sip_client_certificate_request_callback(void *data, bctbx_ssl_c
 	belle_sip_tls_channel_t *channel = (belle_sip_tls_channel_t *)data;
 
 	/* ask certificate */
-	BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(	channel->base.base.listeners
+	BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(	channel->base.base.full_listeners
 			,belle_sip_channel_listener_t
 			,on_auth_requested
 			,&channel->base.base
@@ -502,7 +501,8 @@ static int tls_process_handshake(belle_sip_channel_t *obj){
 	belle_sip_tls_channel_t* channel=(belle_sip_tls_channel_t*)obj;
 	int err=bctbx_ssl_handshake(channel->sslctx);
 	if (err==0){
-		belle_sip_message("Channel [%p]: SSL handshake finished.",obj);
+		belle_sip_message("Channel [%p]: SSL handshake finished, SSL version is [%s], selected ciphersuite is [%s]",obj,
+				  bctbx_ssl_get_version(channel->sslctx), bctbx_ssl_get_ciphersuite(channel->sslctx));
 		belle_sip_source_set_timeout((belle_sip_source_t*)obj,-1);
 		belle_sip_channel_set_ready(obj,(struct sockaddr*)&channel->ss,channel->socklen);
 	}else if (err==BCTBX_ERROR_NET_WANT_READ || err==BCTBX_ERROR_NET_WANT_WRITE){
@@ -581,7 +581,8 @@ static int tls_process_data(belle_sip_channel_t *obj,unsigned int revents){
 				belle_sip_message("Channel [%p]: Connected at TCP level, now doing http proxy connect",obj);
 				if (tls_process_http_connect(channel)) goto process_error;
 			} else {
-				belle_sip_message("Channel [%p]: Connected at TCP level, now doing TLS handshake",obj);
+				belle_sip_message("Channel [%p]: Connected at TCP level, now doing TLS handshake with cname=%s",obj, 
+						  obj->peer_cname ? obj->peer_cname : obj->peer_name);
 				if (tls_process_handshake(obj)==-1) goto process_error;
 			}
 		} else if (obj->stack->http_proxy_host && !channel->http_proxy_connected) {
@@ -678,10 +679,10 @@ static int random_generator(void *ctx, unsigned char *ptr, size_t size){
 int belle_sip_tls_set_verify_error_cb(void * callback) {
 
 	if (callback) {
-        tls_verify_cb_error_cb = (verify_cb_error_cb_t)callback;
+		tls_verify_cb_error_cb = (verify_cb_error_cb_t)callback;
 		belle_sip_message("belle_sip_tls_set_verify_error_cb: callback set");
 	} else {
-        tls_verify_cb_error_cb = NULL;
+		tls_verify_cb_error_cb = NULL;
 		belle_sip_message("belle_sip_tls_set_verify_error_cb: callback cleared");
 	}
 	return 0;
@@ -745,10 +746,12 @@ static int belle_sip_ssl_verify(void *data , bctbx_x509_certificate_t *cert , in
 	belle_sip_message("Found certificate depth=[%i], flags=[%s]:\n%s", depth, flags_str, tmp);
 
 	if (crypto_config->exception_flags==BELLE_TLS_VERIFY_ANY_REASON){
+		belle_sip_warning("Certificate verification bypassed (requested by application).");
 		/* verify context ask to ignore any exception: reset all flags */
 		bctbx_x509_certificate_unset_flag(flags, BCTBX_CERTIFICATE_VERIFY_ALL_FLAGS);
 	}else if (crypto_config->exception_flags & BELLE_TLS_VERIFY_CN_MISMATCH){
 		/* verify context ask to ignore CN mismatch exception : reset this flag */
+		belle_sip_warning("Allowing CN-mistmatch exception.");
 		bctbx_x509_certificate_unset_flag(flags, BCTBX_CERTIFICATE_VERIFY_BADCERT_CN_MISMATCH);
 	}
 
@@ -787,14 +790,46 @@ static int belle_sip_tls_channel_load_root_ca(belle_sip_tls_channel_t *obj, cons
 	return -1;
 }
 
-belle_sip_channel_t * belle_sip_channel_new_tls(belle_sip_stack_t *stack, belle_tls_crypto_config_t *crypto_config, const char *bindip, int localport, const char *peer_cname, const char *dest, int port) {
-	belle_sip_tls_channel_t *obj=belle_sip_object_new(belle_sip_tls_channel_t);
+static int belle_sip_tls_channel_load_root_ca_from_buffer(belle_sip_tls_channel_t *obj, const char *data) {
+	int err = 0;
+	if (data != NULL) {
+		if (obj->root_ca) {
+			bctbx_x509_certificate_free(obj->root_ca);
+		}
+		obj->root_ca = bctbx_x509_certificate_new();
+		//certificate data must to contain in size the NULL character
+		err = bctbx_x509_certificate_parse(obj->root_ca, data, strlen(data) + 1);
+		if (err) {
+			belle_sip_error("Failed to load root ca from string data: 0x%x", err);
+			return -1;
+		}
+		belle_sip_message("Root ca loaded from string data");
+		return 0;
+	}
+	belle_sip_error("Could not load root ca from null string");
+	return -1;
+}
+
+static void belle_sip_tls_channel_deinit_bctbx_ssl(belle_sip_tls_channel_t *obj){
+	if (obj->sslctx) {
+		bctbx_ssl_context_free(obj->sslctx);
+		obj->sslctx = NULL;
+	}
+	if (obj->sslcfg) {
+		bctbx_ssl_config_free(obj->sslcfg);
+		obj->sslcfg = NULL;
+	}
+	if (obj->root_ca) {
+		bctbx_x509_certificate_free(obj->root_ca);
+		obj->root_ca = NULL;
+	}
+	
+}
+
+static int belle_sip_tls_channel_init_bctbx_ssl(belle_sip_tls_channel_t *obj){
 	belle_sip_stream_channel_t* super=(belle_sip_stream_channel_t*)obj;
-
-	belle_sip_stream_channel_init_client(super
-					,stack
-					,bindip,localport,peer_cname,dest,port);
-
+	belle_tls_crypto_config_t *crypto_config = obj->crypto_config;
+	
 	/* create and initialise ssl context and configuration */
 	obj->sslctx = bctbx_ssl_context_new();
 	obj->sslcfg = bctbx_ssl_config_new();
@@ -806,22 +841,34 @@ belle_sip_channel_t * belle_sip_channel_new_tls(belle_sip_stack_t *stack, belle_
 		if (ret<0) {
 			belle_sip_error("Unable to set external config for SSL context at TLS channel creation ret [-0x%x]", -ret);
 			belle_sip_object_unref(obj);
-			return NULL;
+			return -1;
 		}
 		belle_sip_message("Use externally provided SSL configuration when creating TLS channel [%p]", obj);
 	}
 
 	bctbx_ssl_config_set_rng(obj->sslcfg, random_generator, NULL);
 	bctbx_ssl_set_io_callbacks(obj->sslctx, obj, tls_callback_write, tls_callback_read);
-	if (crypto_config->root_ca && belle_sip_tls_channel_load_root_ca(obj,crypto_config->root_ca)==0){
-		bctbx_ssl_config_set_ca_chain(obj->sslcfg, obj->root_ca, super->base.peer_cname ? super->base.peer_cname : super->base.peer_name );
+	if ((crypto_config->root_ca_data && belle_sip_tls_channel_load_root_ca_from_buffer(obj, crypto_config->root_ca_data) == 0)
+		||  (crypto_config->root_ca && belle_sip_tls_channel_load_root_ca(obj, crypto_config->root_ca) == 0)){
+		bctbx_ssl_config_set_ca_chain(obj->sslcfg, obj->root_ca);
 	}
 	bctbx_ssl_config_set_callback_verify(obj->sslcfg, belle_sip_ssl_verify, crypto_config);
 	bctbx_ssl_config_set_callback_cli_cert(obj->sslcfg, belle_sip_client_certificate_request_callback, obj);
 
-	obj->crypto_config=(belle_tls_crypto_config_t*)belle_sip_object_ref(crypto_config);
-
 	bctbx_ssl_context_setup(obj->sslctx, obj->sslcfg);
+	bctbx_ssl_set_hostname(obj->sslctx, super->base.peer_cname ? super->base.peer_cname : super->base.peer_name);
+	return 0;
+}
+
+belle_sip_channel_t * belle_sip_channel_new_tls(belle_sip_stack_t *stack, belle_tls_crypto_config_t *crypto_config, const char *bindip, int localport, const char *peer_cname, const char *dest, int port) {
+	belle_sip_tls_channel_t *obj=belle_sip_object_new(belle_sip_tls_channel_t);
+	belle_sip_stream_channel_t* super=(belle_sip_stream_channel_t*)obj;
+
+	belle_sip_stream_channel_init_client(super
+					,stack
+					,bindip,localport,peer_cname,dest,port);
+
+	obj->crypto_config=(belle_tls_crypto_config_t*)belle_sip_object_ref(crypto_config);
 	return (belle_sip_channel_t*)obj;
 }
 
