@@ -244,7 +244,7 @@ static void https_post_long_body(void){
 	if (url_supported(url)==-1) {
 		return;
 	}
-	bh=belle_sip_user_body_handler_new(image_size+sizeof(MULTIPART_BEGIN)+sizeof(MULTIPART_END), on_progress, NULL, on_send_body, NULL);
+	bh=belle_sip_user_body_handler_new(image_size+sizeof(MULTIPART_BEGIN)+sizeof(MULTIPART_END), on_progress, NULL, NULL, on_send_body, NULL, NULL);
 	content_type=belle_sip_strdup_printf("multipart/form-data; boundary=%s",multipart_boudary);
 
 	uri=belle_generic_uri_parse(url);
@@ -266,7 +266,7 @@ static void https_post_long_body(void){
 	belle_sip_object_unref(l);
 }
 
-static void on_recv_body(belle_sip_user_body_handler_t *bh, belle_sip_message_t *msg, void *data, size_t offset, const uint8_t *buffer, size_t size){
+static void on_recv_body(belle_sip_user_body_handler_t *bh, belle_sip_message_t *msg, void *data, size_t offset, uint8_t *buffer, size_t size){
 	FILE *file=(FILE*)data;
 	if (file)
 		fwrite(buffer,1,size,file);
@@ -282,7 +282,7 @@ static void process_response_headers(void *data, const belle_http_response_event
 		FILE *file=belle_sip_object_data_get(BELLE_SIP_OBJECT(event->request),"file");
 		belle_sip_message_set_body_handler(
 			(belle_sip_message_t*)event->response,
-			(belle_sip_body_handler_t*)belle_sip_user_body_handler_new(0,on_progress,on_recv_body,NULL,file)
+			(belle_sip_body_handler_t*)belle_sip_user_body_handler_new(0,on_progress, NULL,on_recv_body,NULL, NULL,file)
 		);
 	}
 }
@@ -345,16 +345,16 @@ static void one_https_get_with_proxy(void){
 
 
 test_t http_tests[] = {
-	{ "One http GET", one_http_get },
-	{ "http GET of empty body", http_get_empty_body },
-	{ "One https GET", one_https_get },
-	{ "One https GET with http proxy", one_https_get_with_proxy },
-	{ "http request with io error", http_get_io_error },
-	{ "https GET with long body", https_get_long_body },
-	{ "https digest GET", https_digest_get },/*, FIXME, need a server for testing
-	{ "https with client certificate", https_client_cert_connection }*/
-	{ "https POST with long body", https_post_long_body },
-	{ "http GET with long user body", http_get_long_user_body}
+	TEST_NO_TAG("One http GET", one_http_get),
+	TEST_NO_TAG("http GET of empty body", http_get_empty_body),
+	TEST_NO_TAG("One https GET", one_https_get),
+	TEST_NO_TAG("One https GET with http proxy", one_https_get_with_proxy),
+	TEST_NO_TAG("http request with io error", http_get_io_error),
+	TEST_NO_TAG("https GET with long body", https_get_long_body),
+	TEST_NO_TAG("https digest GET", https_digest_get),/*, FIXME, need a server for testing
+	TEST_NO_TAG("https with client certificate", https_client_cert_connection),*/
+	TEST_NO_TAG("https POST with long body", https_post_long_body),
+	TEST_NO_TAG("http GET with long user body", http_get_long_user_body)
 };
 
 test_suite_t http_test_suite = {"HTTP stack", http_before_all, http_after_all, NULL,
