@@ -741,7 +741,7 @@ static void test_address_with_error_header(void) {
 static void test_address_header(void) {
 	belle_sip_uri_t* L_uri;
 	char* L_raw;
-	belle_sip_header_address_t* laddress = belle_sip_header_address_parse("\"toto\" <sip:liblinphone_tester@81.56.11.2:5060>");
+	belle_sip_header_address_t* laddress = belle_sip_header_address_fast_parse("\"toto\" <sip:liblinphone_tester@81.56.11.2:5060>");
 	BC_ASSERT_PTR_NOT_NULL(laddress);
 	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(laddress));
 	BC_ASSERT_PTR_NOT_NULL(L_raw);
@@ -768,6 +768,15 @@ static void test_address_header(void) {
 	BC_ASSERT_STRING_EQUAL("\"to\\to\"",belle_sip_header_address_get_displayname(laddress));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
 
+	laddress = belle_sip_header_address_parse("sips:example.com;transport=tls;maddr=sip1.example.com");
+	BC_ASSERT_PTR_NOT_NULL(laddress);
+	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(laddress));
+	BC_ASSERT_PTR_NOT_NULL(L_raw);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+	laddress = belle_sip_header_address_parse(L_raw);
+	belle_sip_free(L_raw);
+	BC_ASSERT_STRING_EQUAL("sip1.example.com",belle_sip_uri_get_maddr_param(belle_sip_header_address_get_uri(laddress)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
 
 
 }
@@ -775,7 +784,7 @@ static void test_address_header(void) {
 static void test_address_header_with_tel_uri(void) {
 	belle_generic_uri_t* L_uri;
 	char* L_raw;
-	belle_sip_header_address_t* laddress = belle_sip_header_address_parse("\"toto\" <tel:123456>");
+	belle_sip_header_address_t* laddress = belle_sip_header_address_fast_parse("\"toto\" <tel:123456>");
 	BC_ASSERT_PTR_NOT_NULL(laddress);
 	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(laddress));
 	BC_ASSERT_PTR_NOT_NULL(L_raw);
@@ -791,6 +800,27 @@ static void test_address_header_with_tel_uri(void) {
 	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
 }
 
+static void test_address_header_with_urn(void) {
+	belle_generic_uri_t* L_uri;
+	char* L_raw;
+	belle_sip_header_address_t* laddress = belle_sip_header_address_fast_parse("urn:service:sos");
+	BC_ASSERT_PTR_NOT_NULL(laddress);
+	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(laddress));
+	BC_ASSERT_PTR_NOT_NULL(L_raw);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+	laddress = belle_sip_header_address_parse(L_raw);
+	belle_sip_free(L_raw);
+
+	BC_ASSERT_PTR_NULL(belle_sip_header_address_get_displayname(laddress));
+	L_uri = belle_sip_header_address_get_absolute_uri(laddress);
+
+	BC_ASSERT_PTR_NOT_NULL(belle_generic_uri_get_scheme(L_uri));
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(L_uri), "urn");
+	BC_ASSERT_PTR_NOT_NULL(belle_generic_uri_get_opaque_part(L_uri));
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(L_uri), "service:sos");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+}
+
 static void test_very_long_address_header(void) {
 	belle_sip_uri_t* L_uri;
 	const char* raw = "<sip:jehan@sip.linphone.org"
@@ -1133,49 +1163,110 @@ static void test_accept_header(void) {
 
 	BC_ASSERT_PTR_NULL(belle_sip_header_accept_parse("nimportequoi"));
 }
+static void test_reason_header(void) {
+	belle_sip_header_reason_t *L_tmp, *l_next;
+	belle_sip_header_reason_t* L_reason = BELLE_SIP_HEADER_REASON(belle_sip_header_create("Reason", "Q.850 ;cause=16 ;text=\"Busy Everywhere\""));
+	char* l_raw_header;
+	
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_reason_get_text(L_reason),"Busy Everywhere");
+	belle_sip_header_reason_set_text(L_reason, "Terminated");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_reason));
+	
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_reason));
+	L_tmp = belle_sip_header_reason_parse(l_raw_header);
+	L_reason = BELLE_SIP_HEADER_REASON(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_reason_get_text(L_reason),"Terminated");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_reason_get_protocol(L_reason),"Q.850");
+	BC_ASSERT_EQUAL(belle_sip_header_reason_get_cause(L_reason),16 , int,"%d");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_reason));
+	belle_sip_free(l_raw_header);
+
+	L_reason = belle_sip_header_reason_parse("Reason: SIP ;cause=600 ;text=\"Busy Everywhere\", SIP ;cause=580 ;text=\"Precondition Failure\"");
+	l_next =BELLE_SIP_HEADER_REASON(belle_sip_header_get_next(BELLE_SIP_HEADER(L_reason)));
+	BC_ASSERT_PTR_NOT_NULL(l_next);
+	BC_ASSERT_EQUAL(belle_sip_header_reason_get_cause(l_next),580 , int,"%d");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_reason));
+
+	
+	BC_ASSERT_PTR_NULL(belle_sip_header_reason_parse("nimportequoi"));
+}
+static void test_authentication_info_header(void) {
+	belle_sip_header_authentication_info_t *L_tmp;
+	belle_sip_header_authentication_info_t* L_authentication_info = BELLE_SIP_HEADER_AUTHENTICATION_INFO(belle_sip_header_create("Authentication-Info"
+																													, "nextnonce=\"47364c23432d2e131a5fb210812c\""
+																														", qop=auth"));
+	char* l_raw_header;
+	
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authentication_info_get_next_nonce(L_authentication_info),"47364c23432d2e131a5fb210812c");
+	belle_sip_header_authentication_info_set_next_nonce(L_authentication_info, "31a5fb210812c47364c23432d2e1");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_authentication_info));
+	
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_authentication_info));
+	L_tmp = belle_sip_header_authentication_info_parse(l_raw_header);
+	L_authentication_info = BELLE_SIP_HEADER_AUTHENTICATION_INFO(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authentication_info_get_next_nonce(L_authentication_info),"31a5fb210812c47364c23432d2e1");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authentication_info_get_qop(L_authentication_info),"auth");
+	
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_authentication_info));
+	belle_sip_free(l_raw_header);
+	
+	
+	L_authentication_info = belle_sip_header_authentication_info_parse("Authentication-Info:nextnonce=\"WRSE5VkUg7kOjnP5sBZZzsxrsOQ8Eyp6ZNttXIA=\", qop=auth , rspauth=\"5e8518291c1320989df6966405e0b3d1\", cnonce=\"05bc00e63a7794182f9425da9266d00d\", nc=00000001");
+	BC_ASSERT_EQUAL(belle_sip_header_authentication_info_get_nonce_count(L_authentication_info),1, int, "%i");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_authentication_info));
+	
+	BC_ASSERT_PTR_NULL(belle_sip_header_authentication_info_parse("nimportequoi"));
+}
 
 test_t headers_tests[] = {
-	{ "Address", test_address_header },
-	{ "Address tel uri", test_address_header_with_tel_uri },
-	{ "Header address (very long)", test_very_long_address_header },
-	{ "Address with error", test_address_with_error_header },
-	{ "Allow", test_allow_header },
-	{ "Authorization", test_authorization_header },
-	{ "Call-ID", test_call_id_header },
-	{ "Contact (Simple)", test_simple_contact_header },
-	{ "Contact (Complex)", test_complex_contact_header },
-	{ "Contact (Param-less address spec)", test_contact_header_with_paramless_address_spec },
-	{ "Content-Length", test_content_length_header },
-	{ "Content-Type", test_content_type_header },
-	{ "CSeq", test_cseq_header },
-	{ "Date", test_date_header },
-	{ "Expires", test_expires_header },
-	{ "From", test_from_header },
-	{ "From (Param-less address spec)", test_from_header_with_paramless_address_spec },
-	{ "Max-Forwards", test_max_forwards_header },
-	{ "Privacy", test_privacy_header },
-	{ "P-Preferred-Identity", test_p_preferred_identity_header },
-	{ "Proxy-Authenticate", test_proxy_authenticate_header },
-	{ "Proxy-Authorization", test_proxy_authorization_header },
-	{ "Record-Route", test_record_route_header },
-	{ "Refer-To", test_refer_to_header },
-	{ "Referred-By", test_referred_by_header },
-	{ "Replaces", test_replaces_header },
-	{ "Replaces (Escaped)", test_replaces_escaped_header },
-	{ "Route", test_route_header },
-	{ "Service-Route", test_service_route_header },
-	{ "Subscription-State", test_subscription_state_header },
-	{ "To", test_to_header },
-	{ "To (Param-less address spec)", test_to_header_with_paramless_address_spec },
-	{ "User-Agent", test_user_agent_header },
-	{ "Via", test_via_header },
-	{ "WWW-Authenticate", test_www_authenticate_header },
-	{ "Header extension", test_header_extension_1 },
-	{ "Header extension 2", test_header_extension_2 },
-	{ "Header event", test_event_header },
-	{ "Header Supported", test_supported_header },
-	{ "Header Content-Disposition", test_content_disposition_header },
-	{ "Header Accept", test_accept_header }
+	TEST_NO_TAG("Address", test_address_header),
+	TEST_NO_TAG("Address tel uri", test_address_header_with_tel_uri),
+	TEST_NO_TAG("Address urn", test_address_header_with_urn),
+	TEST_NO_TAG("Header address (very long)", test_very_long_address_header),
+	TEST_NO_TAG("Address with error", test_address_with_error_header),
+	TEST_NO_TAG("Allow", test_allow_header),
+	TEST_NO_TAG("Authorization", test_authorization_header),
+	TEST_NO_TAG("Call-ID", test_call_id_header),
+	TEST_NO_TAG("Contact (Simple)", test_simple_contact_header),
+	TEST_NO_TAG("Contact (Complex)", test_complex_contact_header),
+	TEST_NO_TAG("Contact (Param-less address spec)", test_contact_header_with_paramless_address_spec),
+	TEST_NO_TAG("Content-Length", test_content_length_header),
+	TEST_NO_TAG("Content-Type", test_content_type_header),
+	TEST_NO_TAG("CSeq", test_cseq_header),
+	TEST_NO_TAG("Date", test_date_header),
+	TEST_NO_TAG("Expires", test_expires_header),
+	TEST_NO_TAG("From", test_from_header),
+	TEST_NO_TAG("From (Param-less address spec)", test_from_header_with_paramless_address_spec),
+	TEST_NO_TAG("Max-Forwards", test_max_forwards_header),
+	TEST_NO_TAG("Privacy", test_privacy_header),
+	TEST_NO_TAG("P-Preferred-Identity", test_p_preferred_identity_header),
+	TEST_NO_TAG("Proxy-Authenticate", test_proxy_authenticate_header),
+	TEST_NO_TAG("Proxy-Authorization", test_proxy_authorization_header),
+	TEST_NO_TAG("Record-Route", test_record_route_header),
+	TEST_NO_TAG("Refer-To", test_refer_to_header),
+	TEST_NO_TAG("Referred-By", test_referred_by_header),
+	TEST_NO_TAG("Replaces", test_replaces_header),
+	TEST_NO_TAG("Replaces (Escaped)", test_replaces_escaped_header),
+	TEST_NO_TAG("Route", test_route_header),
+	TEST_NO_TAG("Service-Route", test_service_route_header),
+	TEST_NO_TAG("Subscription-State", test_subscription_state_header),
+	TEST_NO_TAG("To", test_to_header),
+	TEST_NO_TAG("To (Param-less address spec)", test_to_header_with_paramless_address_spec),
+	TEST_NO_TAG("User-Agent", test_user_agent_header),
+	TEST_NO_TAG("Via", test_via_header),
+	TEST_NO_TAG("WWW-Authenticate", test_www_authenticate_header),
+	TEST_NO_TAG("Header extension", test_header_extension_1),
+	TEST_NO_TAG("Header extension 2", test_header_extension_2),
+	TEST_NO_TAG("Header event", test_event_header),
+	TEST_NO_TAG("Header Supported", test_supported_header),
+	TEST_NO_TAG("Header Content-Disposition", test_content_disposition_header),
+	TEST_NO_TAG("Header Accept", test_accept_header),
+	TEST_NO_TAG("Header Reason", test_reason_header),
+	TEST_NO_TAG("Header Authentication-Info", test_authentication_info_header)
 };
 
 test_suite_t headers_test_suite = {"Headers", NULL, NULL, belle_sip_tester_before_each, belle_sip_tester_after_each,
