@@ -1009,35 +1009,45 @@ void testGetBody(void) {
 	belle_sip_object_unref(stack);
 }
 
+static void testHop(void){
+	belle_sip_uri_t * uri = belle_sip_uri_parse("sip:sip.linphone.org;maddr=[2001:41d0:8:6e48::]");
+	belle_sip_hop_t *hop = belle_sip_hop_new_from_uri(uri);
+	BC_ASSERT_STRING_EQUAL(hop->host, "2001:41d0:8:6e48::");
+	belle_sip_object_unref(uri);
+	belle_sip_object_unref(hop);
+}
+
+
 /* NOTE - ORDER IS IMPORTANT - MUST TEST fread() AFTER fprintf() */
 test_t message_tests[] = {
-	{ "REGISTER", testRegisterMessage },
-	{ "INVITE", testInviteMessage },
-	{ "INVITE with tel uri", testInviteMessageWithTelUri},
-	{ "Option message", testOptionMessage },
-	{ "REGISTER (Raw)", testRegisterRaw },
-	{ "401 Response", test401Response },
-	{ "Response without response phrase",test401ResponseWithoutResponsePhrase},
-	{ "Origin extraction", test_extract_source },
-	{ "SIP frag", test_sipfrag },
-	{ "Malformed invite", testMalformedMessage },
-	{ "Malformed from", testMalformedFrom },
-	{ "Malformed mandatory field", testMalformedMandatoryField },
-	{ "Malformed invite with bad begin", testMalformedMessageWithWrongStart },
-	{ "Malformed register", testMalformedOptionnalHeaderInMessage },
-	{ "Channel parser error recovery", channel_parser_tester_recovery_from_error},
-	{ "Channel parser malformed start", channel_parser_malformed_start},
-	{ "Channel parser truncated start", channel_parser_truncated_start},
-	{ "Channel parser truncated start with garbage",channel_parser_truncated_start_with_garbage},
+	TEST_NO_TAG("REGISTER", testRegisterMessage),
+	TEST_NO_TAG("INVITE", testInviteMessage),
+	TEST_NO_TAG("INVITE with tel uri", testInviteMessageWithTelUri),
+	TEST_NO_TAG("Option message", testOptionMessage),
+	TEST_NO_TAG("REGISTER (Raw)", testRegisterRaw),
+	TEST_NO_TAG("401 Response", test401Response),
+	TEST_NO_TAG("Response without response phrase",test401ResponseWithoutResponsePhrase),
+	TEST_NO_TAG("Origin extraction", test_extract_source),
+	TEST_NO_TAG("SIP frag", test_sipfrag),
+	TEST_NO_TAG("Malformed invite", testMalformedMessage),
+	TEST_NO_TAG("Malformed from", testMalformedFrom),
+	TEST_NO_TAG("Malformed mandatory field", testMalformedMandatoryField),
+	TEST_NO_TAG("Malformed invite with bad begin", testMalformedMessageWithWrongStart),
+	TEST_NO_TAG("Malformed register", testMalformedOptionnalHeaderInMessage),
+	TEST_NO_TAG("Channel parser error recovery", channel_parser_tester_recovery_from_error),
+	TEST_NO_TAG("Channel parser malformed start", channel_parser_malformed_start),
+	TEST_NO_TAG("Channel parser truncated start", channel_parser_truncated_start),
+	TEST_NO_TAG("Channel parser truncated start with garbage",channel_parser_truncated_start_with_garbage),
 	TEST_ONE_TAG("RFC2543 compatibility", testRFC2543Compat, "LeaksMemory"),
 	TEST_ONE_TAG("RFC2543 compatibility with branch id",testRFC2543CompatWithBranch, "LeaksMemory"),
-	{ "Uri headers in sip INVITE",testUriHeadersInInvite},
-	{ "Uris components in request",testUrisComponentsForRequest},
-	{ "Generic message test",testGenericMessage},
-	{ "HTTP get",testHttpGet},
-	{ "HTTP 200 Ok",testHttp200Ok},
-	{ "Channel parser for HTTP reponse",channel_parser_http_response},
-	{ "Get body size",testGetBody}
+	TEST_NO_TAG("Uri headers in sip INVITE",testUriHeadersInInvite),
+	TEST_NO_TAG("Uris components in request",testUrisComponentsForRequest),
+	TEST_NO_TAG("Generic message test",testGenericMessage),
+	TEST_NO_TAG("HTTP get",testHttpGet),
+	TEST_NO_TAG("HTTP 200 Ok",testHttp200Ok),
+	TEST_NO_TAG("Channel parser for HTTP reponse",channel_parser_http_response),
+	TEST_NO_TAG("Get body size",testGetBody),
+	TEST_NO_TAG("Create hop from uri", testHop)
 };
 
 test_suite_t message_test_suite = {"Message", NULL, NULL, belle_sip_tester_before_each, belle_sip_tester_after_each,
