@@ -317,7 +317,7 @@ static void client_process_auth_requested(void *obj, belle_sip_auth_event_t *eve
 static void belle_sip_refresher_listener (belle_sip_refresher_t* refresher
 		,void* user_pointer
 		,unsigned int status_code
-		,const char* reason_phrase) {
+		,const char* reason_phrase, int will_retry) {
 	endpoint_t* endpoint = (endpoint_t*)user_pointer;
 	BELLESIP_UNUSED(refresher);
 	belle_sip_message("belle_sip_refresher_listener [%i] reason [%s]",status_code,reason_phrase);
@@ -333,7 +333,9 @@ static void belle_sip_refresher_listener (belle_sip_refresher_t* refresher
 	}
 	if (endpoint->stat.refreshKo==1 && endpoint->transiant_network_failure) {
 		belle_sip_stack_set_send_error(endpoint->stack,0);
-	} else 	if (endpoint->stat.refreshOk==1 && endpoint->transiant_network_failure) {
+	} else 	if (endpoint->stat.refreshOk==1
+				&& endpoint->stat.refreshKo==0
+				&& endpoint->transiant_network_failure) {
 		/*generate a network failure*/
 		belle_sip_refresher_set_retry_after(endpoint->refresher,100);
 		belle_sip_stack_set_send_error(endpoint->stack,-1);
@@ -515,6 +517,7 @@ static void subscribe_base(int with_resource_lists) {
 	belle_sip_refresher_t* refresher;
 	belle_sip_header_contact_t* contact=belle_sip_header_contact_new();
 	belle_sip_dialog_t * client_dialog;
+	char *call_id = NULL;
 	int dummy = 0;
 	uint64_t begin;
 	uint64_t end;
@@ -570,6 +573,8 @@ static void subscribe_base(int with_resource_lists) {
 	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.twoHundredOk,1,1000));
 	 /*maybe dialog should be automatically created*/
 	BC_ASSERT_PTR_NOT_NULL(client_dialog = belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(trans)));
+	call_id = belle_sip_strdup(belle_sip_header_call_id_get_call_id(belle_sip_dialog_get_call_id(client_dialog)));
+	belle_sip_object_ref(client_dialog);
 
 	refresher = belle_sip_client_transaction_create_refresher(trans);
 	belle_sip_object_unref(trans);
@@ -582,22 +587,27 @@ static void subscribe_base(int with_resource_lists) {
 	BC_ASSERT_LOWER_STRICT(end-begin,5000,unsigned long long,"%llu");
 
 	belle_sip_message("simulating dialog error and recovery");
-	belle_sip_stack_set_send_error(client->stack, 1500);
+	belle_sip_stack_set_send_error(client->stack, 1);
 	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.fourHundredEightyOne,1,4000));
-
+	/*let the transaction timeout*/
+	wait_for(server->stack,client->stack, &dummy, 1, 32000);
 	belle_sip_stack_set_send_error(client->stack, 0);
-	wait_for(server->stack,client->stack, &dummy, 1, 1000);
+	
 
 	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,4,4000));
 	BC_ASSERT_EQUAL(client->stat.dialogTerminated, 0, int, "%i");
 
 	/*make sure dialog has changed*/
 	BC_ASSERT_PTR_NOT_EQUAL(client_dialog, belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(belle_sip_refresher_get_transaction(refresher))));
-
+	BC_ASSERT_STRING_NOT_EQUAL(call_id, belle_sip_header_call_id_get_call_id(belle_sip_dialog_get_call_id(belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(belle_sip_refresher_get_transaction(refresher))))));
+	belle_sip_free(call_id);
+	belle_sip_object_unref(client_dialog);
 	
 	belle_sip_message("simulating dialog terminated server side and recovery");
 	
 	client_dialog = belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(belle_sip_refresher_get_transaction(refresher)));
+	call_id = belle_sip_strdup(belle_sip_header_call_id_get_call_id(belle_sip_dialog_get_call_id(client_dialog)));
+	belle_sip_object_ref(client_dialog);
 	
 	belle_sip_provider_enable_unconditional_answer(server->provider,TRUE);
 	belle_sip_provider_set_unconditional_answer(server->provider,481);
@@ -611,12 +621,15 @@ static void subscribe_base(int with_resource_lists) {
 	
 	/*make sure dialog has changed*/
 	BC_ASSERT_PTR_NOT_EQUAL(client_dialog, belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(belle_sip_refresher_get_transaction(refresher))));
-
+	BC_ASSERT_STRING_NOT_EQUAL(call_id, belle_sip_header_call_id_get_call_id(belle_sip_dialog_get_call_id(belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(belle_sip_refresher_get_transaction(refresher))))));
+	belle_sip_free(call_id);
+	belle_sip_object_unref(client_dialog);
 	belle_sip_refresher_refresh(refresher, 0);
 	belle_sip_refresher_refresh(refresher, 0);
 
 	belle_sip_refresher_stop(refresher);
-	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&server->stat.dialogTerminated,1,4000));
+	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&server->stat.dialogTerminated,3,4000));
+	
 	belle_sip_object_unref(refresher);
 
 	if (with_resource_lists) {
@@ -861,29 +874,29 @@ static void simple_publish_with_early_refresher(void) {
 }
 
 test_t refresher_tests[] = {
-	{ "REGISTER Expires header", register_expires_header },
-	{ "REGISTER Expires in Contact", register_expires_in_contact },
-	{ "REGISTER Expires header digest", register_expires_header_digest },
-	{ "REGISTER Expires in Contact digest auth", register_expires_in_contact_header_digest_auth },
-	{ "REGISTER with failure", register_with_failure },
-	{ "REGISTER with early refresher",register_early_refresher},
-	TEST_ONE_TAG("SUBSCRIBE", subscribe_test, "LeaksMemory"),
-	TEST_ONE_TAG("SUBSCRIBE of list" , subscribe_list_test, "LeaksMemory"),
-	{ "PUBLISH", simple_publish },
-	{ "PUBLISH with early refresher", simple_publish_with_early_refresher },
-	{ "REGISTER with unrecognizable Contact", register_with_unrecognizable_contact },
-	{ "REGISTER UDP from ipv6 to ipv4", register_test_ipv6_to_ipv4 },
-	{ "REGISTER UDP from ipv4 to ipv6", register_test_ipv4_to_ipv6 },
-	{ "REGISTER UDP from ipv6 to ipv6 with ipv4", register_test_ipv6_to_ipv6_with_ipv4 },
-	{ "REGISTER UDP from ipv6 to ipv6 with ipv6", register_test_ipv6_to_ipv6_with_ipv6 },
-	{ "REGISTER TCP from ipv6 to ipv4", register_tcp_test_ipv6_to_ipv4 },
-	{ "REGISTER TCP from ipv4 to ipv6", register_tcp_test_ipv4_to_ipv6 },
-	{ "REGISTER TCP from ipv6 to ipv6 with ipv4", register_tcp_test_ipv6_to_ipv6_with_ipv4 },
-	{ "REGISTER TCP from ipv6 to ipv6 with ipv6", register_tcp_test_ipv6_to_ipv6_with_ipv6 },
-	{ "REGISTER UDP from random port using AF_INET", register_udp_test_ipv4_random_port },
-	{ "REGISTER UDP from random port using AF_INET6", register_udp_test_ipv6_random_port },
-	{ "REGISTER TCP from random port using AF_INET", register_tcp_test_ipv4_random_port },
-	{ "REGISTER TCP from random port using AF_INET6", register_tcp_test_ipv6_random_port },
+	TEST_NO_TAG("REGISTER Expires header", register_expires_header),
+	TEST_NO_TAG("REGISTER Expires in Contact", register_expires_in_contact),
+	TEST_NO_TAG("REGISTER Expires header digest", register_expires_header_digest),
+	TEST_NO_TAG("REGISTER Expires in Contact digest auth", register_expires_in_contact_header_digest_auth),
+	TEST_NO_TAG("REGISTER with failure", register_with_failure),
+	TEST_NO_TAG("REGISTER with early refresher",register_early_refresher),
+	TEST_NO_TAG("SUBSCRIBE", subscribe_test),
+	TEST_NO_TAG("SUBSCRIBE of list" , subscribe_list_test),
+	TEST_NO_TAG("PUBLISH", simple_publish),
+	TEST_NO_TAG("PUBLISH with early refresher", simple_publish_with_early_refresher),
+	TEST_NO_TAG("REGISTER with unrecognizable Contact", register_with_unrecognizable_contact),
+	TEST_NO_TAG("REGISTER UDP from ipv6 to ipv4", register_test_ipv6_to_ipv4),
+	TEST_NO_TAG("REGISTER UDP from ipv4 to ipv6", register_test_ipv4_to_ipv6),
+	TEST_NO_TAG("REGISTER UDP from ipv6 to ipv6 with ipv4", register_test_ipv6_to_ipv6_with_ipv4),
+	TEST_NO_TAG("REGISTER UDP from ipv6 to ipv6 with ipv6", register_test_ipv6_to_ipv6_with_ipv6),
+	TEST_NO_TAG("REGISTER TCP from ipv6 to ipv4", register_tcp_test_ipv6_to_ipv4),
+	TEST_NO_TAG("REGISTER TCP from ipv4 to ipv6", register_tcp_test_ipv4_to_ipv6),
+	TEST_NO_TAG("REGISTER TCP from ipv6 to ipv6 with ipv4", register_tcp_test_ipv6_to_ipv6_with_ipv4),
+	TEST_NO_TAG("REGISTER TCP from ipv6 to ipv6 with ipv6", register_tcp_test_ipv6_to_ipv6_with_ipv6),
+	TEST_NO_TAG("REGISTER UDP from random port using AF_INET", register_udp_test_ipv4_random_port),
+	TEST_NO_TAG("REGISTER UDP from random port using AF_INET6", register_udp_test_ipv6_random_port),
+	TEST_NO_TAG("REGISTER TCP from random port using AF_INET", register_tcp_test_ipv4_random_port),
+	TEST_NO_TAG("REGISTER TCP from random port using AF_INET6", register_tcp_test_ipv6_random_port),
 };
 
 test_suite_t refresher_test_suite = {"Refresher", NULL, NULL, belle_sip_tester_before_each, belle_sip_tester_after_each,
