@@ -31,6 +31,7 @@ const char *no_server_running_here="sip:test.linphone.org:3;transport=tcp";
 const char *no_response_here="sip:78.220.48.77:3;transport=%s";
 const char *test_domain_tls_to_tcp="sip:sip2.linphone.org:5060;transport=tls";
 const char *test_http_proxy_addr="sip.linphone.org";
+const char *test_with_wrong_cname="sips:rototo.com;maddr=91.121.209.194";
 int test_http_proxy_port = 3128 ;
 
 static int is_register_ok;
@@ -167,6 +168,52 @@ const char* belle_sip_tester_private_key =
 const char* belle_sip_tester_private_key_passwd="secret";
 
 
+const char *belle_sip_tester_root_ca = 
+"-----BEGIN CERTIFICATE-----\n"
+"MIIDRjCCAq+gAwIBAgIJAJ3nFcA7qFrOMA0GCSqGSIb3DQEBBQUAMIG7MQswCQYD\n"
+"VQQGEwJGUjETMBEGA1UECAwKU29tZS1TdGF0ZTERMA8GA1UEBwwIR3Jlbm9ibGUx\n"
+"IjAgBgNVBAoMGUJlbGxlZG9ubmUgQ29tbXVuaWNhdGlvbnMxDDAKBgNVBAsMA0xB\n"
+"QjEWMBQGA1UEAwwNSmVoYW4gTW9ubmllcjE6MDgGCSqGSIb3DQEJARYramVoYW4u\n"
+"bW9ubmllckBiZWxsZWRvbm5lLWNvbW11bmljYXRpb25zLmNvbTAeFw0xMzA0MzAx\n"
+"MzMwMThaFw0yMzA0MjgxMzMwMThaMIG7MQswCQYDVQQGEwJGUjETMBEGA1UECAwK\n"
+"U29tZS1TdGF0ZTERMA8GA1UEBwwIR3Jlbm9ibGUxIjAgBgNVBAoMGUJlbGxlZG9u\n"
+"bmUgQ29tbXVuaWNhdGlvbnMxDDAKBgNVBAsMA0xBQjEWMBQGA1UEAwwNSmVoYW4g\n"
+"TW9ubmllcjE6MDgGCSqGSIb3DQEJARYramVoYW4ubW9ubmllckBiZWxsZWRvbm5l\n"
+"LWNvbW11bmljYXRpb25zLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA\n"
+"z5F8mMh3SUr6NUd7tq2uW2Kdn22Zn3kNpLYb78AQK4IoQMOLGXbBdyoXvz1fublg\n"
+"bxtLYsiGhICd7Ul9zLGc3edn85LbD3Skb7ERx6MakRnYep3FzagZJhn14QEaZCx6\n"
+"3Qs0Ir4rSP7hmlpYt8VO/zqqNR3tsA59O0D9c7bpQ7UCAwEAAaNQME4wHQYDVR0O\n"
+"BBYEFAZfXccWr2L4LW5xA4ig1h0rBH+6MB8GA1UdIwQYMBaAFAZfXccWr2L4LW5x\n"
+"A4ig1h0rBH+6MAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQEFBQADgYEAKvmt2m1o\n"
+"axGKc0DjiJPypU/NsAf4Yu0nOnY8pHqJJCB0AWVoAPM7vGYPWpeH7LSdGZLuT9eK\n"
+"FUWGJhPnkrnklmBdVB0l7qXYjR5uf766HDkoDxuLhNifow3IYvsS+L2Y6puRQb9w\n"
+"HLMDE29mBDl0WyoX3h0yR0EiAO15V9A7I10=\n"
+"-----END CERTIFICATE-----\n"
+"\n"
+"AddTrust External Root used for *.linphone.org\n"
+"======================\n"
+"-----BEGIN CERTIFICATE-----\n"
+"MIIENjCCAx6gAwIBAgIBATANBgkqhkiG9w0BAQUFADBvMQswCQYDVQQGEwJTRTEUMBIGA1UEChML\n"
+"QWRkVHJ1c3QgQUIxJjAkBgNVBAsTHUFkZFRydXN0IEV4dGVybmFsIFRUUCBOZXR3b3JrMSIwIAYD\n"
+"VQQDExlBZGRUcnVzdCBFeHRlcm5hbCBDQSBSb290MB4XDTAwMDUzMDEwNDgzOFoXDTIwMDUzMDEw\n"
+"NDgzOFowbzELMAkGA1UEBhMCU0UxFDASBgNVBAoTC0FkZFRydXN0IEFCMSYwJAYDVQQLEx1BZGRU\n"
+"cnVzdCBFeHRlcm5hbCBUVFAgTmV0d29yazEiMCAGA1UEAxMZQWRkVHJ1c3QgRXh0ZXJuYWwgQ0Eg\n"
+"Um9vdDCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALf3GjPm8gAELTngTlvtH7xsD821\n"
+"+iO2zt6bETOXpClMfZOfvUq8k+0DGuOPz+VtUFrWlymUWoCwSXrbLpX9uMq/NzgtHj6RQa1wVsfw\n"
+"Tz/oMp50ysiQVOnGXw94nZpAPA6sYapeFI+eh6FqUNzXmk6vBbOmcZSccbNQYArHE504B4YCqOmo\n"
+"aSYYkKtMsE8jqzpPhNjfzp/haW+710LXa0Tkx63ubUFfclpxCDezeWWkWaCUN/cALw3CknLa0Dhy\n"
+"2xSoRcRdKn23tNbE7qzNE0S3ySvdQwAl+mG5aWpYIxG3pzOPVnVZ9c0p10a3CitlttNCbxWyuHv7\n"
+"7+ldU9U0WicCAwEAAaOB3DCB2TAdBgNVHQ4EFgQUrb2YejS0Jvf6xCZU7wO94CTLVBowCwYDVR0P\n"
+"BAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wgZkGA1UdIwSBkTCBjoAUrb2YejS0Jvf6xCZU7wO94CTL\n"
+"VBqhc6RxMG8xCzAJBgNVBAYTAlNFMRQwEgYDVQQKEwtBZGRUcnVzdCBBQjEmMCQGA1UECxMdQWRk\n"
+"VHJ1c3QgRXh0ZXJuYWwgVFRQIE5ldHdvcmsxIjAgBgNVBAMTGUFkZFRydXN0IEV4dGVybmFsIENB\n"
+"IFJvb3SCAQEwDQYJKoZIhvcNAQEFBQADggEBALCb4IUlwtYj4g+WBpKdQZic2YR5gdkeWxQHIzZl\n"
+"j7DYd7usQWxHYINRsPkyPef89iYTx4AWpb9a/IfPeHmJIZriTAcKhjW88t5RxNKWt9x+Tu5w/Rw5\n"
+"6wwCURQtjr0W4MHfRnXnJK3s9EK0hZNwEGe6nQY1ShjTK3rMUUKhemPR5ruhxSvCNr4TDea9Y355\n"
+"e6cJDUCrat2PisP29owaQgVR1EX1n6diIWgVIEM8med8vSTYqZEXc4g/VhsxOBi0cQ+azcgOno4u\n"
+"G+GMmIPLHzHxREzGBHNJdmAPx/i9F4BrLunMTA5amnkPIAou1Z5jJh5VkpTYghdae9C8x49OhgQ=\n"
+"-----END CERTIFICATE-----\n";
+
 static void process_auth_requested(void *user_ctx, belle_sip_auth_event_t *event){
 	BELLESIP_UNUSED(user_ctx);
 	if (belle_sip_auth_event_get_mode(event) == BELLE_SIP_AUTH_MODE_HTTP_DIGEST) {
@@ -201,11 +248,8 @@ int register_before_all(void) {
 	lp=belle_sip_stack_create_listening_point(stack,"0.0.0.0",7061,"TLS");
 	if (lp) {
 		belle_tls_crypto_config_t *crypto_config=belle_tls_crypto_config_new();
-		/* since test.linphone.org does not have proper certificates, don't verify anything*/
-		belle_tls_crypto_config_set_verify_exceptions(crypto_config, BELLE_TLS_VERIFY_ANY_REASON);
-		if (belle_sip_tester_get_root_ca_path() != NULL) {
-			belle_tls_crypto_config_set_root_ca(crypto_config,belle_sip_tester_get_root_ca_path());
-		}
+	
+		belle_tls_crypto_config_set_root_ca_data(crypto_config, belle_sip_tester_root_ca);
 		belle_sip_tls_listening_point_set_crypto_config(BELLE_SIP_TLS_LISTENING_POINT(lp), crypto_config);
 		belle_sip_provider_add_listening_point(prov,lp);
 		belle_sip_object_unref(crypto_config);
@@ -422,6 +466,13 @@ static void stateful_register_tls(void){
 	register_test("tls",1);
 }
 
+static void stateful_register_tls_with_wrong_cname(void){
+	belle_sip_request_t *req;
+	
+	req = try_register_user_at_domain(stack, prov, "tls", 1, "tester",test_domain, test_with_wrong_cname, 0);
+	if (req) belle_sip_object_unref(req);
+}
+
 static void stateful_register_tls_with_http_proxy(void) {
 	belle_sip_tls_listening_point_t * lp = (belle_sip_tls_listening_point_t*)belle_sip_provider_get_listening_point(prov, "tls");
 	if (!lp) {
@@ -516,6 +567,7 @@ static void test_bad_request(void) {
 	belle_sip_listening_point_clean_channels(lp);
 }
 
+
 static void test_register_authenticate(void) {
 	belle_sip_request_t *reg;
 	number_of_challenge=0;
@@ -537,12 +589,55 @@ static void test_register_channel_inactive(void){
 		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),0,int,"%d");
 		register_test("tcp",1);
 		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),1,int,"%d");
-		belle_sip_stack_sleep(stack,5000);
+		belle_sip_stack_sleep(stack, 3000);
+		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),1,int,"%d");
+		register_test("tcp",1);
+		belle_sip_stack_sleep(stack, 3000);
+		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),1,int,"%d");
+		belle_sip_stack_sleep(stack,3000);
 		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),0,int,"%d");
 		belle_sip_stack_set_inactive_transport_timeout(stack,3600);
 	}
 }
 
+static void test_channel_moving_to_error_and_cleaned(void){
+	belle_sip_listening_point_t *lp=belle_sip_provider_get_listening_point(prov,"UDP");
+	BC_ASSERT_PTR_NOT_NULL(lp);
+	if (lp) {
+		belle_sip_request_t *req;
+		belle_sip_client_transaction_t *tr;
+		char identity[128];
+		char uri[128];
+		
+		belle_sip_listening_point_clean_channels(lp);
+		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),0,int,"%d");
+		
+		snprintf(identity,sizeof(identity),"Tester <sip:%s@%s>","bellesip",test_domain);
+		snprintf(uri,sizeof(uri),"sip:%s",test_domain);
+		req = belle_sip_request_create(
+						belle_sip_uri_parse(uri),
+						"REGISTER",
+						belle_sip_provider_create_call_id(prov),
+						belle_sip_header_cseq_create(20,"REGISTER"),
+						belle_sip_header_from_create2(identity,BELLE_SIP_RANDOM_TAG),
+						belle_sip_header_to_create2(identity,NULL),
+						belle_sip_header_via_new(),
+						70);
+		tr = belle_sip_provider_create_client_transaction(prov, req);
+		belle_sip_client_transaction_send_request(tr);
+		belle_sip_object_ref(tr);
+		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),1,int,"%d");
+		/*calling notify_server_error() will make the channel enter the error state, which is what we want to test*/
+		belle_sip_channel_notify_server_error((belle_sip_channel_t*)lp->channels->data);
+		/*immediately after, we clean the channel from the listening point*/
+		belle_sip_listening_point_clean_channels(lp);
+		/*we just want to verify that it doesn't crash*/
+		belle_sip_stack_sleep(stack, 1000);
+		belle_sip_object_unref(tr);
+		
+	}
+}
+
 
 
 static void test_register_client_authenticated(void) {
@@ -611,12 +706,23 @@ static void register_dns_srv_tcp(void){
 	if (req) belle_sip_object_unref(req);
 }
 
+static void enable_cn_mismatch(int enable){
+	belle_sip_listening_point_t *lp =  belle_sip_provider_get_listening_point(prov, "TLS");
+	belle_sip_provider_clean_channels(prov);
+	if (lp) {
+		belle_tls_crypto_config_t *cfg = belle_sip_tls_listening_point_get_crypto_config(BELLE_SIP_TLS_LISTENING_POINT(lp));
+		belle_tls_crypto_config_set_verify_exceptions(cfg, enable ? BELLE_TLS_VERIFY_CN_MISMATCH : 0);
+	}
+}
+
 static void register_dns_srv_tls(void){
 	belle_sip_request_t *req;
 	io_error_count=0;
+	enable_cn_mismatch(TRUE);
 	req=try_register_user_at_domain(stack, prov, "TLS",1,"tester",client_auth_domain,"sip:linphone.net;transport=tls",1);
 	BC_ASSERT_EQUAL(io_error_count, 0, int, "%d");
 	if (req) belle_sip_object_unref(req);
+	enable_cn_mismatch(FALSE);
 }
 
 static void register_dns_srv_tls_with_http_proxy(void){
@@ -627,7 +733,7 @@ static void register_dns_srv_tls_with_http_proxy(void){
 		return;
 	}
 	io_error_count=0;
-	belle_sip_provider_clean_channels(prov);
+	enable_cn_mismatch(TRUE);
 	belle_sip_stack_set_http_proxy_host(stack, test_http_proxy_addr);
 	belle_sip_stack_set_http_proxy_port(stack, test_http_proxy_port);
 	req=try_register_user_at_domain(stack, prov, "TLS",1,"tester",client_auth_domain,"sip:linphone.net;transport=tls",1);
@@ -635,6 +741,7 @@ static void register_dns_srv_tls_with_http_proxy(void){
 	belle_sip_stack_set_http_proxy_port(stack, 0);
 	BC_ASSERT_EQUAL(io_error_count, 0, int, "%d");
 	if (req) belle_sip_object_unref(req);
+	enable_cn_mismatch(FALSE);
 }
 
 static void register_dns_load_balancing(void) {
@@ -659,7 +766,7 @@ static void process_message_response_event(void *user_ctx, const belle_sip_respo
 		}
 	}
 }
-static belle_sip_request_t* send_message(belle_sip_request_t *initial_request, const char* realm){
+static belle_sip_request_t* send_message_to(belle_sip_request_t *initial_request, const char* realm, belle_sip_uri_t *outbound){
 	int i;
 	int io_error_count=0;
 	belle_sip_request_t *message_request=NULL;
@@ -686,41 +793,54 @@ static belle_sip_request_t* send_message(belle_sip_request_t *initial_request, c
 	// 	belle_sip_header_set_name(BELLE_SIP_HEADER(h), BELLE_SIP_PROXY_AUTHORIZATION);
 	// }
 	clone_request = (belle_sip_request_t*)belle_sip_object_ref(belle_sip_object_clone((belle_sip_object_t*)message_request));
-	belle_sip_client_transaction_send_request_to(belle_sip_provider_create_client_transaction(prov,message_request),NULL);
+	belle_sip_client_transaction_send_request_to(belle_sip_provider_create_client_transaction(prov,message_request),outbound);
 	for(i=0; i<2 && io_error_count==0 &&is_register_ok==0;i++)
 		belle_sip_stack_sleep(stack,5000);
 
 	return clone_request;
 }
-static void reuse_nonce(void) {
+
+static void reuse_nonce_base(const char* outbound) {
 	belle_sip_request_t *register_request;
-	size_t initial_auth_context_count=belle_sip_list_size(prov->auth_contexts);
-	register_request=register_user_at_domain(stack, prov, "tcp",1,"marie","sip.linphone.org",NULL);
+	size_t initial_auth_context_count=0;/*belle_sip_list_size(prov->auth_contexts);*/
+	char outbound_uri[256];
+	/*reset auth context*/
+	prov->auth_contexts = belle_sip_list_free_with_data(prov->auth_contexts,(void(*)(void*))belle_sip_authorization_destroy);
+	
+	if (outbound)
+		snprintf(outbound_uri, sizeof(outbound_uri),"sip:%s",outbound);
+	
+	register_request=register_user_at_domain(stack, prov, "tcp",1,"marie",outbound,NULL);
+	
 	if (register_request) {
 		belle_sip_header_authorization_t * h = NULL;
 		belle_sip_request_t *message_request;
-		listener_callbacks.process_dialog_terminated=process_dialog_terminated;
-		listener_callbacks.process_io_error=process_io_error;
-		listener_callbacks.process_request_event=process_request_event;
-		listener_callbacks.process_response_event=process_message_response_event;
-		listener_callbacks.process_timeout=process_timeout;
-		listener_callbacks.process_transaction_terminated=process_transaction_terminated;
-		listener_callbacks.process_auth_requested=process_auth_requested;
-		listener_callbacks.listener_destroyed=NULL;
-		listener=belle_sip_listener_create_from_callbacks(&listener_callbacks,NULL);
+		belle_sip_listener_callbacks_t cbs;
+		belle_sip_listener_t *reuse_nonce_listener;
+		cbs.process_dialog_terminated=process_dialog_terminated;
+		cbs.process_io_error=process_io_error;
+		cbs.process_request_event=process_request_event;
+		cbs.process_response_event=process_message_response_event;
+		cbs.process_timeout=process_timeout;
+		cbs.process_transaction_terminated=process_transaction_terminated;
+		cbs.process_auth_requested=process_auth_requested;
+		cbs.listener_destroyed=NULL;
+		reuse_nonce_listener=belle_sip_listener_create_from_callbacks(&cbs,NULL);
 
-		belle_sip_provider_add_sip_listener(prov,BELLE_SIP_LISTENER(listener));
+		belle_sip_provider_add_sip_listener(prov,BELLE_SIP_LISTENER(reuse_nonce_listener));
 
 		/*currently only one nonce should have been used (the one for the REGISTER)*/
 		BC_ASSERT_EQUAL((unsigned int)belle_sip_list_size(prov->auth_contexts), (unsigned int)initial_auth_context_count+1,unsigned int,"%u");
 
 		/*this should reuse previous nonce*/
-		message_request=send_message(register_request, auth_domain);
+		message_request=send_message_to(register_request, auth_domain,outbound?belle_sip_uri_parse(outbound_uri):NULL);
 		BC_ASSERT_EQUAL(is_register_ok, 404,int,"%d");
 		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 					BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
 				));
-		if (BC_ASSERT_PTR_NOT_NULL(h)) {
+		if (BC_ASSERT_PTR_NOT_NULL(h)
+			&& belle_sip_header_authorization_get_qop(h)
+			&& strcmp(belle_sip_header_authorization_get_qop(h),"auth")==0) {
 			char * first_nonce_used;
 			BC_ASSERT_EQUAL(2, belle_sip_header_authorization_get_nonce_count(h),int,"%d");
 			first_nonce_used = belle_sip_strdup(belle_sip_header_authorization_get_nonce(h));
@@ -730,7 +850,7 @@ static void reuse_nonce(void) {
 
 
 		/*new nonce should be created when not using outbound proxy realm*/
-		message_request=send_message(register_request, NULL);
+		message_request=send_message_to(register_request, NULL,outbound?belle_sip_uri_parse(outbound_uri):NULL);
 		BC_ASSERT_EQUAL(is_register_ok, 407,int,"%d");
 		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 				BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
@@ -740,7 +860,7 @@ static void reuse_nonce(void) {
 
 
 		/*new nonce should be created here too*/
-		message_request=send_message(register_request, "wrongrealm");
+		message_request=send_message_to(register_request, "wrongrealm",outbound?belle_sip_uri_parse(outbound_uri):NULL);
 		BC_ASSERT_EQUAL(is_register_ok, 407,int,"%d");
 		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 				BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
@@ -749,50 +869,136 @@ static void reuse_nonce(void) {
 		belle_sip_object_unref(message_request);
 
 
-		/*first nonce created should be reused*/
-		message_request=send_message(register_request, auth_domain);
-		BC_ASSERT_EQUAL(is_register_ok, 404,int,"%d");
+		/*first nonce created should be reused. this test is only for qop = auth*/
+		message_request=send_message_to(register_request, auth_domain,outbound?belle_sip_uri_parse(outbound_uri):NULL);
+		
 		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 				BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
 			));
-		if (BC_ASSERT_PTR_NOT_NULL(h)) {
+		if (BC_ASSERT_PTR_NOT_NULL(h)
+			&& belle_sip_header_authorization_get_qop(h)
+			&& strcmp(belle_sip_header_authorization_get_qop(h),"auth")==0) {
+			BC_ASSERT_EQUAL(is_register_ok, 404,int,"%d");
 			BC_ASSERT_EQUAL(3, belle_sip_header_authorization_get_nonce_count(h),int,"%d");
 		}
 		belle_sip_object_unref(message_request);
 
-		belle_sip_provider_remove_sip_listener(prov,BELLE_SIP_LISTENER(listener));
+		belle_sip_provider_remove_sip_listener(prov,BELLE_SIP_LISTENER(reuse_nonce_listener));
 		unregister_user(stack,prov,register_request,1);
 		belle_sip_object_unref(register_request);
 	}
 }
+static void reuse_nonce(void) {
+	reuse_nonce_base("sip.linphone.org");
+}
+#define NONCE_SIZE  32
+void register_process_request_event(char *nonce, const belle_sip_request_event_t *event) {
+	belle_sip_request_t *req = belle_sip_request_event_get_request(event);
+	belle_sip_header_authorization_t *authorization;
+	int response_code = 407;
+	char *uri_as_string= belle_sip_uri_to_string(belle_sip_request_get_uri(req));
+	belle_sip_response_t * response_msg;
+	belle_sip_server_transaction_t *trans=belle_sip_provider_create_server_transaction(prov, req);
+	
+	if (strcasecmp(belle_sip_request_get_method(req), "REGISTER") == 0) {
+		response_code=401;
+	}
+	
+	if (	(authorization = belle_sip_message_get_header_by_type(req,belle_sip_header_authorization_t))
+		 || (authorization = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(req,belle_sip_header_proxy_authorization_t)))) {
+		 char ha1[33], ha2[33], response[33];
+		belle_sip_auth_helper_compute_ha1(belle_sip_header_authorization_get_username(authorization)
+										, belle_sip_header_authorization_get_realm(authorization)
+										  ,"secret", ha1);
+		belle_sip_auth_helper_compute_ha2(belle_sip_request_get_method(req)
+										  , uri_as_string, ha2);
+		belle_sip_auth_helper_compute_response(ha1,nonce, ha2, response);
+		if (strcmp(response, belle_sip_header_authorization_get_response(authorization)) == 0) {
+			belle_sip_message("Auth sucessfull");
+			if (strcasecmp(belle_sip_request_get_method(req), "MESSAGE") == 0) {
+				response_code = 404;
+			} else {
+				response_code = 200;
+			}
+		}
+	}
+	
+	belle_sip_random_token((nonce), NONCE_SIZE);
+	response_msg = belle_sip_response_create_from_request(req, response_code);
+	
+	if (response_code == 407 || response_code == 401 ) {
+		belle_sip_header_www_authenticate_t *www_authenticate = 401?belle_sip_header_www_authenticate_new():BELLE_SIP_HEADER_WWW_AUTHENTICATE(belle_sip_header_proxy_authenticate_new());
+	
+		
+		belle_sip_header_www_authenticate_set_realm(www_authenticate, "sip.linphone.org");
+		belle_sip_header_www_authenticate_set_domain(www_authenticate, "sip:sip.linphone.org");
+		belle_sip_header_www_authenticate_set_scheme(www_authenticate, "Digest");
+		belle_sip_header_www_authenticate_set_nonce(www_authenticate,nonce);
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response_msg), BELLE_SIP_HEADER(www_authenticate));
+	} else {
+		belle_sip_header_authentication_info_t* authentication_info = belle_sip_header_authentication_info_new();
+		belle_sip_header_authentication_info_set_next_nonce(authentication_info, nonce);
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response_msg), BELLE_SIP_HEADER(authentication_info));
+	}
+	
+	belle_sip_server_transaction_send_response(trans,response_msg);
+}
 
+static void test_register_with_next_nonce(void) {
+	belle_sip_listening_point_t *server_lp = belle_sip_stack_create_listening_point(stack,"0.0.0.0",bctbx_random()%20000 +1024,"TCP");
+	char  nonce [NONCE_SIZE];
+	belle_sip_listener_t *server_listener;
+	char listening_uri[256];
+	belle_sip_listener_callbacks_t cbs;
+	belle_sip_random_token((nonce), sizeof(nonce));
+	
+	cbs.process_dialog_terminated=NULL;
+	cbs.process_io_error=NULL;
+	cbs.process_request_event=(void (*)(void *user_ctx, const belle_sip_request_event_t *event))register_process_request_event;
+	cbs.process_response_event=NULL;
+	cbs.process_timeout=NULL;
+	cbs.process_transaction_terminated=NULL;
+	cbs.process_auth_requested=NULL;
+	cbs.listener_destroyed=NULL;
+	server_listener=belle_sip_listener_create_from_callbacks(&cbs,nonce);
+	belle_sip_provider_add_sip_listener(prov,server_listener);
+	belle_sip_provider_add_listening_point(prov,server_lp);
+	snprintf(listening_uri,sizeof(listening_uri), "127.0.0.1:%i;transport=tcp",belle_sip_listening_point_get_port(server_lp));
+
+	reuse_nonce_base(listening_uri);
+	belle_sip_provider_remove_sip_listener(prov, server_listener);
+	belle_sip_provider_remove_listening_point(prov, server_lp);
+}
 
 test_t register_tests[] = {
-	{ "Stateful UDP", stateful_register_udp },
-	{ "Stateful UDP with keep-alive", stateful_register_udp_with_keep_alive },
-	{ "Stateful UDP with network delay", stateful_register_udp_delayed },
-	{ "Stateful UDP with send error", stateful_register_udp_with_send_error },
-	{ "Stateful UDP with outbound proxy", stateful_register_udp_with_outbound_proxy },
-	{ "Stateful TCP", stateful_register_tcp },
-	{ "Stateful TLS", stateful_register_tls },
-	{ "Stateful TLS with http proxy", stateful_register_tls_with_http_proxy },
-	{ "Stateful TLS with wrong http proxy", stateful_register_tls_with_wrong_http_proxy },
-	{ "Stateless UDP", stateless_register_udp },
-	{ "Stateless TCP", stateless_register_tcp },
-	{ "Stateless TLS", stateless_register_tls },
-	{ "Bad TCP request", test_bad_request },
-	{ "Authenticate", test_register_authenticate },
-	{ "TLS client cert authentication", test_register_client_authenticated },
-	{ "Channel inactive", test_register_channel_inactive },
-	{ "TCP connection failure", test_connection_failure },
-	{ "TCP connection too long", test_connection_too_long_tcp },
-	{ "TLS connection too long", test_connection_too_long_tls },
-	{ "TLS connection to TCP server", test_tls_to_tcp },
-	{ "Register with DNS SRV failover TCP", register_dns_srv_tcp },
-	{ "Register with DNS SRV failover TLS", register_dns_srv_tls },
-	{ "Register with DNS SRV failover TLS with http proxy", register_dns_srv_tls_with_http_proxy },
-	{ "Register with DNS load-balancing", register_dns_load_balancing },
-	{ "Nonce reutilization", reuse_nonce }
+	TEST_NO_TAG("Stateful UDP", stateful_register_udp),
+	TEST_NO_TAG("Stateful UDP with keep-alive", stateful_register_udp_with_keep_alive),
+	TEST_NO_TAG("Stateful UDP with network delay", stateful_register_udp_delayed),
+	TEST_NO_TAG("Stateful UDP with send error", stateful_register_udp_with_send_error),
+	TEST_NO_TAG("Stateful UDP with outbound proxy", stateful_register_udp_with_outbound_proxy),
+	TEST_NO_TAG("Stateful TCP", stateful_register_tcp),
+	TEST_NO_TAG("Stateful TLS", stateful_register_tls),
+	TEST_NO_TAG("Stateful TLS with wrong cname", stateful_register_tls_with_wrong_cname),
+	TEST_NO_TAG("Stateful TLS with http proxy", stateful_register_tls_with_http_proxy),
+	TEST_NO_TAG("Stateful TLS with wrong http proxy", stateful_register_tls_with_wrong_http_proxy),
+	TEST_NO_TAG("Stateless UDP", stateless_register_udp),
+	TEST_NO_TAG("Stateless TCP", stateless_register_tcp),
+	TEST_NO_TAG("Stateless TLS", stateless_register_tls),
+	TEST_NO_TAG("Bad TCP request", test_bad_request),
+	TEST_NO_TAG("Authenticate", test_register_authenticate),
+	TEST_NO_TAG("TLS client cert authentication", test_register_client_authenticated),
+	TEST_NO_TAG("Channel inactive", test_register_channel_inactive),
+	TEST_NO_TAG("Channel moving to error test and cleaned", test_channel_moving_to_error_and_cleaned),
+	TEST_NO_TAG("TCP connection failure", test_connection_failure),
+	TEST_NO_TAG("TCP connection too long", test_connection_too_long_tcp),
+	TEST_NO_TAG("TLS connection too long", test_connection_too_long_tls),
+	TEST_NO_TAG("TLS connection to TCP server", test_tls_to_tcp),
+	TEST_NO_TAG("Register with DNS SRV failover TCP", register_dns_srv_tcp),
+	TEST_NO_TAG("Register with DNS SRV failover TLS", register_dns_srv_tls),
+	TEST_NO_TAG("Register with DNS SRV failover TLS with http proxy", register_dns_srv_tls_with_http_proxy),
+	TEST_NO_TAG("Register with DNS load-balancing", register_dns_load_balancing),
+	TEST_NO_TAG("Nonce reutilization", reuse_nonce),
+	TEST_NO_TAG("Next Nonce", test_register_with_next_nonce)
 };
 
 test_suite_t register_test_suite = {"Register", register_before_all, register_after_all, NULL,
