@@ -85,7 +85,7 @@ static void destroy_endpoint(endpoint_t *endpoint) {
 	belle_sip_uninit_sockets();
 }
 
-static void a_resolve_done(void *data, const char *name, struct addrinfo *ai_list) {
+static void a_resolve_done(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl) {
 	endpoint_t *client = (endpoint_t *)data;
 	BELLESIP_UNUSED(name);
 	client->resolve_done = 1;
@@ -96,7 +96,7 @@ static void a_resolve_done(void *data, const char *name, struct addrinfo *ai_lis
 		client->resolve_ko = 1;
 }
 
-static void srv_resolve_done(void *data, const char *name, belle_sip_list_t *srv_list) {
+static void srv_resolve_done(void *data, const char *name, belle_sip_list_t *srv_list, uint32_t ttl) {
 	endpoint_t *client = (endpoint_t *)data;
 	BELLESIP_UNUSED(name);
 	client->resolve_done = 1;
@@ -440,15 +440,10 @@ static void set_custom_resolv_conf(belle_sip_stack_t *stack, const char *ns[3]){
 	free(resolv_file);
 }
 
-static void dns_fallback(void) {
+static void _dns_fallback(const char *nameservers[]) {
 	struct addrinfo *ai;
 	int timeout;
 	endpoint_t *client = create_endpoint();
-	const char *nameservers[]={
-		"94.23.19.176", /*linphone.org ; this is not a name server, it will not respond*/
-		"8.8.8.8", /* public nameserver, should work*/
-		NULL
-	};
 
 	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
@@ -471,6 +466,33 @@ static void dns_fallback(void) {
 	destroy_endpoint(client);
 }
 
+static void dns_fallback(void) {
+	const char *nameservers[]={
+		"94.23.19.176", /*linphone.org ; this is not a name server, it will not respond*/
+		"8.8.8.8", /* public nameserver, should work*/
+		NULL
+	};
+	_dns_fallback(nameservers);
+}
+
+static void dns_fallback_because_of_scope_link_ipv6(void) {
+	const char *nameservers[]={
+		"fe80::fdc5:99ef:ac05:5c55%enp0s25", /* Scope link IPv6 name server that will not respond */
+		"8.8.8.8", /* public nameserver, should work*/
+		NULL
+	};
+	_dns_fallback(nameservers);
+}
+
+static void dns_fallback_because_of_invalid_ipv6(void) {
+	const char *nameservers[]={
+		"fe80::ba26:6cff:feb9:145c", /* Invalid IPv6 name server */
+		"8.8.8.8", /* public nameserver, should work*/
+		NULL
+	};
+	_dns_fallback(nameservers);
+}
+
 static void ipv6_dns_server(void) {
 	struct addrinfo *ai;
 	int timeout;
@@ -544,23 +566,25 @@ static void ipv4_and_ipv6_dns_server(void) {
 }
 
 test_t resolver_tests[] = {
-	{ "A query (IPv4)", ipv4_a_query },
-	{ "A query (IPv4) with CNAME", ipv4_cname_a_query },
-	{ "A query (IPv4) with no result", ipv4_a_query_no_result },
-	{ "A query (IPv4) with send failure", ipv4_a_query_send_failure },
-	{ "A query (IPv4) with timeout", ipv4_a_query_timeout },
-	{ "A query (IPv4) with multiple results", ipv4_a_query_multiple_results },
-	{ "A query (IPv4) with AI_V4MAPPED results", ipv4_a_query_with_v4mapped_results },
-	{ "Local query", local_query },
-	{ "AAAA query (IPv6)", ipv6_aaaa_query },
-	{ "SRV query", srv_query },
-	{ "SRV + A query", srv_a_query },
-	{ "SRV + A query with no SRV result", srv_a_query_no_srv_result },
-	{ "Local SRV+A query", local_full_query },
-	{ "No query needed", no_query_needed },
-	{ "DNS fallback", dns_fallback },
-	{ "IPv6 DNS server", ipv6_dns_server },
-	{ "IPv4 and v6 DNS servers", ipv4_and_ipv6_dns_server }
+	TEST_NO_TAG("A query (IPv4)", ipv4_a_query),
+	TEST_NO_TAG("A query (IPv4) with CNAME", ipv4_cname_a_query),
+	TEST_NO_TAG("A query (IPv4) with no result", ipv4_a_query_no_result),
+	TEST_NO_TAG("A query (IPv4) with send failure", ipv4_a_query_send_failure),
+	TEST_NO_TAG("A query (IPv4) with timeout", ipv4_a_query_timeout),
+	TEST_NO_TAG("A query (IPv4) with multiple results", ipv4_a_query_multiple_results),
+	TEST_NO_TAG("A query (IPv4) with AI_V4MAPPED results", ipv4_a_query_with_v4mapped_results),
+	TEST_NO_TAG("Local query", local_query),
+	TEST_NO_TAG("AAAA query (IPv6)", ipv6_aaaa_query),
+	TEST_NO_TAG("SRV query", srv_query),
+	TEST_NO_TAG("SRV + A query", srv_a_query),
+	TEST_NO_TAG("SRV + A query with no SRV result", srv_a_query_no_srv_result),
+	TEST_NO_TAG("Local SRV+A query", local_full_query),
+	TEST_NO_TAG("No query needed", no_query_needed),
+	TEST_NO_TAG("DNS fallback", dns_fallback),
+	TEST_NO_TAG("DNS fallback because of scope link IPv6", dns_fallback_because_of_scope_link_ipv6),
+	TEST_NO_TAG("DNS fallback because of invalid IPv6", dns_fallback_because_of_invalid_ipv6),
+	TEST_NO_TAG("IPv6 DNS server", ipv6_dns_server),
+	TEST_NO_TAG("IPv4 and v6 DNS servers", ipv4_and_ipv6_dns_server)
 };
 
 test_suite_t resolver_test_suite = {"Resolver", NULL, NULL, belle_sip_tester_before_each, belle_sip_tester_after_each,
