@@ -94,6 +94,8 @@ void belle_sip_tester_init(void(*ftester_printf)(int level, const char *fmt, va_
 	ipv6_available=_belle_sip_tester_ipv6_available();
 	bc_tester_add_suite(&cast_test_suite);
 	bc_tester_add_suite(&sip_uri_test_suite);
+	bc_tester_add_suite(&fast_sip_uri_test_suite);
+	bc_tester_add_suite(&perf_sip_uri_test_suite);
 	bc_tester_add_suite(&generic_uri_test_suite);
 	bc_tester_add_suite(&headers_test_suite);
 	bc_tester_add_suite(&core_test_suite);
@@ -120,12 +122,12 @@ void belle_sip_tester_uninit(void) {
 	bc_tester_uninit();
 }
 
-void belle_sip_tester_before_each() {
+void belle_sip_tester_before_each(void) {
 	belle_sip_object_enable_leak_detector(TRUE);
 	leaked_objects_count = belle_sip_object_get_object_count();
 }
 
-int belle_sip_tester_after_each() {
+void belle_sip_tester_after_each(void) {
 	int leaked_objects = belle_sip_object_get_object_count() - leaked_objects_count;
 	if (leaked_objects > 0) {
 		char* format = belle_sip_strdup_printf("%d object%s leaked in suite [%s] test [%s], please fix that!",
@@ -147,17 +149,17 @@ int belle_sip_tester_after_each() {
 		// if the test is NOT marked as leaking memory and it actually is, we should make it fail
 		if (!leaks_expected && leaked_objects > 0) {
 			BC_FAIL("This test is leaking memory!");
-			return 1;
 			// and reciprocally
 		} else if (leaks_expected && leaked_objects == 0) {
 			BC_FAIL("This test is not leaking anymore, please remove LeaksMemory tag!");
-			return 1;
 		}
 	}
-	return 0;
 }
 
 int belle_sip_tester_set_log_file(const char *filename) {
+	bctbx_log_handler_t *filehandler;
+	char* dir;
+	char* base;
 	if (log_file) {
 		fclose(log_file);
 	}
@@ -166,13 +168,19 @@ int belle_sip_tester_set_log_file(const char *filename) {
 		belle_sip_error("Cannot open file [%s] for writing logs because [%s]", filename, strerror(errno));
 		return -1;
 	}
+	dir = bctbx_dirname(filename);
+	base = bctbx_basename(filename);
 	belle_sip_message("Redirecting traces to file [%s]", filename);
-	belle_sip_set_log_file(log_file);
+	filehandler = bctbx_create_file_log_handler(0, dir, base, log_file);
+	bctbx_add_log_handler(filehandler);
+	bctbx_add_log_handler(filehandler);
+	if (dir) bctbx_free(dir);
+	if (base) bctbx_free(base);
 	return 0;
 }
 
 
-#if !defined(ANDROID) && !defined(TARGET_OS_IPHONE) && !(defined(BELLE_SIP_WINDOWS_PHONE) || defined(BELLE_SIP_WINDOWS_UNIVERSAL))
+#if !defined(__ANDROID__) && !defined(TARGET_OS_IPHONE) && !(defined(BELLE_SIP_WINDOWS_PHONE) || defined(BELLE_SIP_WINDOWS_UNIVERSAL))
 
 static const char* belle_sip_helper =
 		"\t\t\t--verbose\n"
@@ -205,7 +213,8 @@ int main (int argc, char *argv[]) {
 	if (env_domain) {
 		test_domain=env_domain;
 	}
-
+	bctbx_init_logger(TRUE);
+	
 	for(i=1;i<argc;++i){
 		if (strcmp(argv[i],"--verbose")==0){
 			belle_sip_set_log_level(BELLE_SIP_LOG_DEBUG);
@@ -241,6 +250,7 @@ int main (int argc, char *argv[]) {
 
 	ret = bc_tester_start(argv[0]);
 	belle_sip_tester_uninit();
+	bctbx_uninit_logger();
 	return ret;
 }
 
