@@ -32,6 +32,8 @@ extern "C" {
 extern test_suite_t cast_test_suite;
 extern test_suite_t generic_uri_test_suite;
 extern test_suite_t sip_uri_test_suite;
+extern test_suite_t fast_sip_uri_test_suite;
+extern test_suite_t perf_sip_uri_test_suite;
 extern test_suite_t headers_test_suite;
 extern test_suite_t core_test_suite;
 extern test_suite_t sdp_test_suite;
@@ -54,7 +56,7 @@ void belle_sip_tester_set_root_ca_path(const char *root_ca_path);
 void belle_sip_tester_init(void(*ftester_printf)(int level, const char *fmt, va_list args));
 void belle_sip_tester_uninit(void);
 void belle_sip_tester_before_each(void);
-int belle_sip_tester_after_each(void);
+void belle_sip_tester_after_each(void);
 int belle_sip_tester_set_log_file(const char *filename);
 
 #ifdef __cplusplus
